@@ -9,13 +9,16 @@ to access online Astronomical data. Each web service has its own sub-package.
 # Affiliated packages may add whatever they like to this file, but
 # should keep this content at the top.
 # ----------------------------------------------------------------------------
-from ._astropy_init import *
+from ._astropy_init import __version__, __githash__, test
 # ----------------------------------------------------------------------------
 
 import os
 import logging
 
 from .logger import _init_log
+from astropy import config as _config
+
+__all__ = ["__version__", "__githash__", "__citation__", "__bibtex__", "test", "log"]
 
 
 # Set the bibtex entry to the article referenced in CITATION.
@@ -24,7 +27,8 @@ def _get_bibtex():
 
     with open(citation_file, 'r') as citation:
         refs = citation.read().split('@ARTICLE')[1:]
-        if len(refs) == 0: return ''
+        if len(refs) == 0:
+            return ''
         bibtexreference = "@ARTICLE{0}".format(refs[0])
     return bibtexreference
 
@@ -32,19 +36,27 @@ def _get_bibtex():
 __citation__ = __bibtex__ = _get_bibtex()
 
 
-try:
-    from .version import version as __version__
-except ImportError:
-    # TODO: Issue a warning using the logging framework
-    __version__ = ''
-try:
-    from .version import githash as __githash__
-except ImportError:
-    # TODO: Issue a warning using the logging framework
-    __githash__ = ''
-
-
 # Setup logging for astroquery
 logging.addLevelName(5, "TRACE")
 log = logging.getLogger()
 log = _init_log()
+
+
+# Set up cache configuration
+class Cache_Conf(_config.ConfigNamespace):
+
+    cache_timeout = _config.ConfigItem(
+        604800,
+        ('Astroquery-wide cache timeout (seconds). Default is 1 week (604800). '
+         'Setting to None prevents the cache from expiring (not recommended).'),
+        cfgtype='integer'
+    )
+
+    cache_active = _config.ConfigItem(
+        True,
+        "Astroquery global cache usage, False turns off all caching.",
+        cfgtype='boolean'
+    )
+
+
+cache_conf = Cache_Conf()
