@@ -19,26 +19,28 @@ from astroquery import log
 from astropy.utils.console import ProgressBar
 from astropy import units as u
 from astropy.time import Time
-from pyvo.dal.sia2 import SIA_PARAMETERS_DESC
+
+try:
+    from pyvo.dal.sia2 import SIA2_PARAMETERS_DESC, SIA2Service
+except ImportError:
+    # Can be removed once min version of pyvo is 1.5
+    from pyvo.dal.sia2 import SIA_PARAMETERS_DESC as SIA2_PARAMETERS_DESC
+    from pyvo.dal.sia2 import SIAService as SIA2Service
 
 from ..exceptions import LoginError
 from ..utils import commons
 from ..utils.process_asyncs import async_to_sync
-from ..query import QueryWithLogin
-from .tapsql import _gen_pos_sql, _gen_str_sql, _gen_numeric_sql,\
-    _gen_band_list_sql, _gen_datetime_sql, _gen_pol_sql, _gen_pub_sql,\
-    _gen_science_sql, _gen_spec_res_sql, ALMA_DATE_FORMAT
+from ..query import BaseQuery, QueryWithLogin, BaseVOQuery
+from .tapsql import (_gen_pos_sql, _gen_str_sql, _gen_numeric_sql,
+                     _gen_band_list_sql, _gen_datetime_sql, _gen_pol_sql, _gen_pub_sql,
+                     _gen_science_sql, _gen_spec_res_sql, ALMA_DATE_FORMAT)
 from . import conf, auth_urls
-from astroquery.utils.commons import ASTROPY_LT_4_1
 from astroquery.exceptions import CorruptDataWarning
 
 __all__ = {'AlmaClass', 'ALMA_BANDS'}
 
 __doctest_skip__ = ['AlmaClass.*']
 
-ALMA_TAP_PATH = 'tap'
-ALMA_SIA_PATH = 'sia2'
-ALMA_DATALINK_PATH = 'datalink/sync'
 
 # Map from ALMA ObsCore result to ALMA original query result
 # The map is provided in order to preserve the name of the columns in the
@@ -159,9 +161,23 @@ ALMA_FORM_KEYS = {
 }
 
 
+# used to lookup the TAP service on an ARC
+TAP_SERVICE_PATH = 'tap'
+
+# standard ID URI to look for when expanding TAR files
+DATALINK_STANDARD_ID = 'ivo://ivoa.net/std/DataLink#links-1.0'
+
+# used to lookup the DataLink service on an ARC
+DATALINK_SERVICE_PATH = 'datalink/sync'
+
+# used to lookup the SIA service on an ARC
+SIA_SERVICE_PATH = 'sia2'
+
+
 def _gen_sql(payload):
     sql = 'select * from ivoa.obscore'
     where = ''
+    unused_payload = payload.copy()
     if payload:
         for constraint in payload:
             for attrib_category in ALMA_FORM_KEYS.values():
@@ -181,9 +197,116 @@ def _gen_sql(payload):
                             else:
                                 where = ' WHERE '
                             where += attrib_where
+
+                        # Delete this key to see what's left over afterward
+                        #
+                        # Use pop to avoid the slight possibility of trying to remove
+                        # an already removed key
+                        unused_payload.pop(constraint)
+
+    if unused_payload:
+        # Left over (unused) constraints passed.  Let the user know.
+        remaining = [f'{p} -> {unused_payload[p]}' for p in unused_payload]
+        raise TypeError(f'Unsupported arguments were passed:\n{remaining}')
+
     return sql + where
 
 
+class AlmaAuth(BaseVOQuery, BaseQuery):
+    """Authentication session information for passing credentials to an OIDC instance
+
+    Assumes an OIDC system like Keycloak with a preconfigured client app called "oidc" to validate against.
+    This does not use Tokens in the traditional OIDC sense, but rather uses the Keycloak specific endpoint
+    to validate a username and password.  Passwords are then kept in a Python keyring.
+    """
+
+    _CLIENT_ID = 'oidc'
+    _GRANT_TYPE = 'password'
+    _INVALID_PASSWORD_MESSAGE = 'Invalid user credentials'
+    _REALM_ENDPOINT = '/auth/realms/ALMA'
+    _LOGIN_ENDPOINT = f'{_REALM_ENDPOINT}/protocol/openid-connect/token'
+    _VERIFY_WELL_KNOWN_ENDPOINT = f'{_REALM_ENDPOINT}/.well-known/openid-configuration'
+
+    def __init__(self):
+        super().__init__()
+        self._auth_hosts = auth_urls
+        self._auth_host = None
+
+    @property
+    def auth_hosts(self):
+        return self._auth_hosts
+
+    @auth_hosts.setter
+    def auth_hosts(self, auth_hosts):
+        """
+        Set the available hosts to check for login endpoints.
+
+        Parameters
+        ----------
+        auth_hosts : array
+            Available hosts name.  Checking each one until one returns a 200 for
+            the well-known endpoint.
+        """
+        if auth_hosts is None:
+            raise LoginError('Valid authentication hosts cannot be None')
+        else:
+            self._auth_hosts = auth_hosts
+
+    def get_valid_host(self):
+        if self._auth_host is None:
+            for auth_url in self._auth_hosts:
+                # set session cookies (they do not get set otherwise)
+                url_to_check = f'https://{auth_url}{self._VERIFY_WELL_KNOWN_ENDPOINT}'
+                response = self._request("HEAD", url_to_check, cache=False)
+
+                if response.status_code == 200:
+                    self._auth_host = auth_url
+                    log.debug(f'Set auth host to {self._auth_host}')
+                    break
+
+        if self._auth_host is None:
+            raise LoginError(f'No useable hosts to login to: {self._auth_hosts}')
+        else:
+            return self._auth_host
+
+    def login(self, username, password):
+        """
+        Authenticate to one of the configured hosts.
+
+        Parameters
+        ----------
+        username : str
+            The username to authenticate with
+        password : str
+            The user's password
+        """
+        data = {
+            'username': username,
+            'password': password,
+            'grant_type': self._GRANT_TYPE,
+            'client_id': self._CLIENT_ID
+        }
+
+        login_url = f'https://{self.get_valid_host()}{self._LOGIN_ENDPOINT}'
+        log.info(f'Authenticating {username} on {login_url}.')
+        login_response = self._request('POST', login_url, data=data, cache=False)
+        json_auth = login_response.json()
+
+        if 'error' in json_auth:
+            log.debug(f'{json_auth}')
+            error_message = json_auth['error_description']
+            if self._INVALID_PASSWORD_MESSAGE not in error_message:
+                raise LoginError("Could not log in to ALMA authorization portal: "
+                                 f"{self.get_valid_host()} Message from server: {error_message}")
+            else:
+                raise LoginError(error_message)
+        elif 'access_token' not in json_auth:
+            raise LoginError("Could not log in to any of the known ALMA authorization portals: \n"
+                             f"No error from server, but missing access token from host: {self.get_valid_host()}")
+        else:
+            log.info(f'Successfully logged in to {self._auth_host}')
+
+
 @async_to_sync
 class AlmaClass(QueryWithLogin):
 
@@ -193,48 +316,70 @@ class AlmaClass(QueryWithLogin):
 
     def __init__(self):
         # sia service does not need disambiguation but tap does
-        super(AlmaClass, self).__init__()
+        super().__init__()
         self._sia = None
         self._tap = None
         self._datalink = None
-        self.sia_url = None
-        self.tap_url = None
-        self.datalink_url = None
+        self._sia_url = None
+        self._tap_url = None
+        self._datalink_url = None
+        self._auth = AlmaAuth()
+
+    @property
+    def auth(self):
+        return self._auth
 
     @property
     def datalink(self):
         if not self._datalink:
-            base_url = self._get_dataarchive_url()
-            if base_url.endswith('/'):
-                self.datalink_url = base_url + ALMA_DATALINK_PATH
-            else:
-                self.datalink_url = base_url + '/' + ALMA_DATALINK_PATH
-            self._datalink = pyvo.dal.adhoc.DatalinkService(
-                baseurl=self.datalink_url)
+            self._datalink = pyvo.dal.adhoc.DatalinkService(self.datalink_url)
         return self._datalink
 
+    @property
+    def datalink_url(self):
+        if not self._datalink_url:
+            try:
+                self._datalink_url = urljoin(self._get_dataarchive_url(), DATALINK_SERVICE_PATH)
+            except requests.exceptions.HTTPError as err:
+                log.debug(
+                    f"ERROR getting the ALMA Archive URL: {str(err)}")
+                raise err
+        return self._datalink_url
+
     @property
     def sia(self):
         if not self._sia:
-            base_url = self._get_dataarchive_url()
-            if base_url.endswith('/'):
-                self.sia_url = base_url + ALMA_SIA_PATH
-            else:
-                self.sia_url = base_url + '/' + ALMA_SIA_PATH
-            self._sia = pyvo.dal.sia2.SIAService(baseurl=self.sia_url)
+            self._sia = SIA2Service(baseurl=self.sia_url)
         return self._sia
 
+    @property
+    def sia_url(self):
+        if not self._sia_url:
+            try:
+                self._sia_url = urljoin(self._get_dataarchive_url(), SIA_SERVICE_PATH)
+            except requests.exceptions.HTTPError as err:
+                log.debug(
+                    f"ERROR getting the  ALMA Archive URL: {str(err)}")
+                raise err
+        return self._sia_url
+
     @property
     def tap(self):
         if not self._tap:
-            base_url = self._get_dataarchive_url()
-            if base_url.endswith('/'):
-                self.tap_url = base_url + ALMA_TAP_PATH
-            else:
-                self.tap_url = base_url + '/' + ALMA_TAP_PATH
-            self._tap = pyvo.dal.tap.TAPService(baseurl=self.tap_url)
+            self._tap = pyvo.dal.tap.TAPService(baseurl=self.tap_url, session=self._session)
         return self._tap
 
+    @property
+    def tap_url(self):
+        if not self._tap_url:
+            try:
+                self._tap_url = urljoin(self._get_dataarchive_url(), TAP_SERVICE_PATH)
+            except requests.exceptions.HTTPError as err:
+                log.debug(
+                    f"ERROR getting the  ALMA Archive URL: {str(err)}")
+                raise err
+        return self._tap_url
+
     def query_object_async(self, object_name, *, public=True,
                            science=True, payload=None, **kwargs):
         """
@@ -296,7 +441,8 @@ class AlmaClass(QueryWithLogin):
                                 payload=payload, **kwargs)
 
     def query_async(self, payload, *, public=True, science=True,
-                    legacy_columns=False, get_query_payload=None, **kwargs):
+                    legacy_columns=False, get_query_payload=False,
+                    maxrec=None, **kwargs):
         """
         Perform a generic query with user-specified payload
 
@@ -313,6 +459,10 @@ class AlmaClass(QueryWithLogin):
         legacy_columns : bool
             True to return the columns from the obsolete ALMA advanced query,
             otherwise return the current columns based on ObsCore model.
+        get_query_payload : bool
+            Flag to indicate whether to simply return the payload.
+        maxrec : integer
+            Cap on the amount of records returned.  Default is no limit.
 
         Returns
         -------
@@ -336,11 +486,16 @@ class AlmaClass(QueryWithLogin):
             payload['science_observation'] = science
         if public is not None:
             payload['public_data'] = public
-        if get_query_payload:
-            return payload
 
         query = _gen_sql(payload)
-        result = self.query_tap(query, maxrec=payload.get('maxrec', None))
+
+        if get_query_payload:
+            # Return the TAP query payload that goes out to the server rather
+            # than the unprocessed payload dict from the python side
+            return query
+
+        result = self.query_tap(query, maxrec=maxrec)
+
         if result is not None:
             result = result.to_table()
         else:
@@ -385,7 +540,7 @@ class AlmaClass(QueryWithLogin):
 
         Returns
         -------
-        Results in `pyvo.dal.SIAResults` format.
+        Results in ``pyvo.dal.sia2.SIA2Results`` format.
         result.table in Astropy table format
         """
         return self.sia.search(
@@ -409,11 +564,11 @@ class AlmaClass(QueryWithLogin):
             maxrec=maxrec,
             **kwargs)
 
-    # SIA_PARAMETERS_DESC contains links that Sphinx can't resolve.
+    # SIA2_PARAMETERS_DESC contains links that Sphinx can't resolve.
     for var in ('POLARIZATION_STATES', 'CALIBRATION_LEVELS'):
-        SIA_PARAMETERS_DESC = SIA_PARAMETERS_DESC.replace(f'`pyvo.dam.obscore.{var}`',
-                                                          f'pyvo.dam.obscore.{var}')
-    query_sia.__doc__ = query_sia.__doc__.replace('_SIA2_PARAMETERS', SIA_PARAMETERS_DESC)
+        SIA2_PARAMETERS_DESC = SIA2_PARAMETERS_DESC.replace(f'`pyvo.dam.obscore.{var}`',
+                                                            f'pyvo.dam.obscore.{var}')
+    query_sia.__doc__ = query_sia.__doc__.replace('_SIA2_PARAMETERS', SIA2_PARAMETERS_DESC)
 
     def query_tap(self, query, maxrec=None):
         """
@@ -449,11 +604,11 @@ class AlmaClass(QueryWithLogin):
 
     # update method pydocs
     query_region_async.__doc__ = query_region_async.__doc__.replace(
-        '_SIA2_PARAMETERS', pyvo.dal.sia2.SIA_PARAMETERS_DESC)
+        '_SIA2_PARAMETERS', SIA2_PARAMETERS_DESC)
     query_object_async.__doc__ = query_object_async.__doc__.replace(
-        '_SIA2_PARAMETERS', pyvo.dal.sia2.SIA_PARAMETERS_DESC)
+        '_SIA2_PARAMETERS', SIA2_PARAMETERS_DESC)
     query_async.__doc__ = query_async.__doc__.replace(
-        '_SIA2_PARAMETERS', pyvo.dal.sia2.SIA_PARAMETERS_DESC)
+        '_SIA2_PARAMETERS', SIA2_PARAMETERS_DESC)
 
     def _get_dataarchive_url(self):
         """
@@ -462,7 +617,7 @@ class AlmaClass(QueryWithLogin):
         """
         if not hasattr(self, 'dataarchive_url'):
             if self.archive_url in ('http://almascience.org', 'https://almascience.org'):
-                response = self._request('GET', self.archive_url,
+                response = self._request('GET', self.archive_url, timeout=self.TIMEOUT,
                                          cache=False)
                 response.raise_for_status()
                 # Jan 2017: we have to force https because the archive doesn't
@@ -484,7 +639,6 @@ class AlmaClass(QueryWithLogin):
 
     def get_data_info(self, uids, *, expand_tarfiles=False,
                       with_auxiliary=True, with_rawdata=True):
-
         """
         Return information about the data associated with ALMA uid(s)
 
@@ -505,7 +659,7 @@ class AlmaClass(QueryWithLogin):
         Returns
         -------
         Table with results or None. Table has the following columns: id (UID),
-        access_url (URL to access data), content_length, content_type (MIME
+        access_url (URL to access data), service_def, content_length, content_type (MIME
         type), semantics, description (optional), error_message (optional)
         """
         if uids is None:
@@ -516,17 +670,21 @@ class AlmaClass(QueryWithLogin):
             raise TypeError("Datasets must be given as a list of strings.")
         # TODO remove this loop and send uids at once when pyvo fixed
         result = None
+        datalink_service_def_dict = {}
         for uid in uids:
             res = self.datalink.run_sync(uid)
             if res.status[0] != 'OK':
                 raise Exception('ERROR {}: {}'.format(res.status[0],
                                                       res.status[1]))
+
+            # Collect the ad-hoc DataLink services for later retrieval if expand_tarballs is set
+            if expand_tarfiles:
+                for adhoc_service in res.iter_adhocservices():
+                    if self.is_datalink_adhoc_service(adhoc_service):
+                        datalink_service_def_dict[adhoc_service.ID] = adhoc_service
+
             temp = res.to_table()
-            if ASTROPY_LT_4_1:
-                # very annoying
-                for col in [x for x in temp.colnames
-                            if x not in ['content_length', 'readable']]:
-                    temp[col] = temp[col].astype(str)
+
             result = temp if result is None else vstack([result, temp])
             to_delete = []
             for index, rr in enumerate(result):
@@ -543,24 +701,20 @@ class AlmaClass(QueryWithLogin):
         if not with_rawdata:
             result = result[np.core.defchararray.find(
                 result['semantics'], '#progenitor') == -1]
-        # primary data delivery type is files packaged in tarballs. However
-        # some type of data has an alternative way to retrieve each individual
-        # file as an alternative (semantics='#datalink' and
-        # 'content_type=application/x-votable+xml;content=datalink'). They also
-        # require an extra call to the datalink service to get the list of
-        # files.
-        DATALINK_FILE_TYPE = 'application/x-votable+xml;content=datalink'
-        DATALINK_SEMANTICS = '#datalink'
+        # if expand_tarfiles:
+        # identify the tarballs that can be expandable and replace them
+        # with the list of components
+        expanded_result = None
+        to_delete = []
         if expand_tarfiles:
-            # identify the tarballs that can be expandable and replace them
-            # with the list of components
-            expanded_result = None
-            to_delete = []
             for index, row in enumerate(result):
-                if DATALINK_SEMANTICS in row['semantics'] and \
-                        row['content_type'] == DATALINK_FILE_TYPE:
+                service_def_id = row['service_def']
+                # service_def record, so check if it points to a DataLink document
+                if service_def_id and service_def_id in datalink_service_def_dict:
                     # subsequent call to datalink
-                    file_id = row['access_url'].split('ID=')[1]
+                    adhoc_service = datalink_service_def_dict[service_def_id]
+                    recursive_access_url = self.get_adhoc_service_access_url(adhoc_service)
+                    file_id = recursive_access_url.split('ID=')[1]
                     expanded_tar = self.get_data_info(file_id)
                     expanded_tar = expanded_tar[
                         expanded_tar['semantics'] != '#cutout']
@@ -569,26 +723,37 @@ class AlmaClass(QueryWithLogin):
                     else:
                         expanded_result = vstack(
                             [expanded_result, expanded_tar], join_type='exact')
+
+                    # These DataLink entries have no access_url and are links to service_def RESOURCEs only,
+                    # so they can be removed if expanded.
                     to_delete.append(index)
-            # cleanup
-            result.remove_rows(to_delete)
-            # add the extra rows
-            if expanded_result:
-                result = vstack([result, expanded_result], join_type='exact')
-        else:
-            result = result[np.logical_or(np.core.defchararray.find(
-                result['semantics'].astype(str), DATALINK_SEMANTICS) == -1,
-                result['content_type'].astype(str) != DATALINK_FILE_TYPE)]
+        # cleanup
+        result.remove_rows(to_delete)
+        # add the extra rows
+        if expanded_result:
+            result = vstack([result, expanded_result], join_type='exact')
 
         return result
 
+    def is_datalink_adhoc_service(self, adhoc_service):
+        standard_id = self.get_adhoc_service_parameter(adhoc_service, 'standardID')
+        return standard_id == DATALINK_STANDARD_ID
+
+    def get_adhoc_service_access_url(self, adhoc_service):
+        return self.get_adhoc_service_parameter(adhoc_service, 'accessURL')
+
+    def get_adhoc_service_parameter(self, adhoc_service, parameter_id):
+        for p in adhoc_service.params:
+            if p.ID == parameter_id:
+                return p.value
+
     def is_proprietary(self, uid):
         """
         Given an ALMA UID, query the servers to determine whether it is
         proprietary or not.
         """
         query = "select distinct data_rights from ivoa.obscore where " \
-                "obs_id='{}'".format(uid)
+                "member_ous_uid='{}'".format(uid)
         result = self.query_tap(query)
         if result:
             tableresult = result.to_table()
@@ -660,7 +825,7 @@ class AlmaClass(QueryWithLogin):
         for file_link in unique(files):
             log.debug("Downloading {0} to {1}".format(file_link, savedir))
             try:
-                check_filename = self._request('HEAD', file_link, auth=auth)
+                check_filename = self._request('HEAD', file_link, auth=auth, timeout=self.TIMEOUT)
                 check_filename.raise_for_status()
             except requests.HTTPError as ex:
                 if ex.response.status_code == 401:
@@ -669,11 +834,11 @@ class AlmaClass(QueryWithLogin):
                                  " next file".format(url=file_link))
                         continue
                     else:
-                        raise(ex)
+                        raise (ex)
 
             try:
-                filename = re.search("filename=(.*)",
-                                     check_filename.headers['Content-Disposition']).groups()[0]
+                filename = os.path.basename(re.search("filename=(.*)",
+                                            check_filename.headers['Content-Disposition']).groups()[0])
             except KeyError:
                 log.info(f"Unable to find filename for {file_link}  "
                          "(missing Content-Disposition in header).  "
@@ -689,7 +854,7 @@ class AlmaClass(QueryWithLogin):
                 if 'content-length' in check_filename.headers:
                     length = int(check_filename.headers['content-length'])
                     if length == 0:
-                        warnings.warn('URL {0} has length=0'.format(url))
+                        warnings.warn('URL {0} has length=0'.format(file_link))
                     elif existing_file_length == length:
                         log.info(f"Found cached file {filename} with expected size {existing_file_length}.")
                     elif existing_file_length < length:
@@ -700,7 +865,7 @@ class AlmaClass(QueryWithLogin):
                                       f"size {length}.  The download is likely corrupted.",
                                       CorruptDataWarning)
                 else:
-                    warnings.warn(f"Could not verify {url} because it has no 'content-length'")
+                    warnings.warn(f"Could not verify {file_link} because it has no 'content-length'")
 
             try:
                 if not verify_only:
@@ -721,7 +886,7 @@ class AlmaClass(QueryWithLogin):
                                  " next file".format(url=file_link))
                         continue
                     else:
-                        raise(ex)
+                        raise (ex)
                 elif ex.response.status_code == 403:
                     log.error("Access denied to {url}".format(url=file_link))
                     if 'dataPortal' in file_link and 'sso' not in file_link:
@@ -780,7 +945,15 @@ class AlmaClass(QueryWithLogin):
             raise TypeError("Datasets must be given as a list of strings.")
 
         files = self.get_data_info(uids)
-        file_urls = files['access_url']
+        # filter out blank access URLs
+        # it is possible for there to be length-1 lists
+        if len(files) == 1:
+            file_urls = files['access_url']
+            if isinstance(file_urls, str) and file_urls == '':
+                raise ValueError(f"Cannot download uid {uids} because it has no file")
+        else:
+            file_urls = [url for url in files['access_url'] if url]
+
         totalsize = files['content_length'].sum()*u.B
 
         # each_size, totalsize = self.data_size(files)
@@ -802,11 +975,7 @@ class AlmaClass(QueryWithLogin):
             else:
                 username = self.USERNAME
 
-        if hasattr(self, '_auth_url'):
-            auth_url = self._auth_url
-        else:
-            raise LoginError("Login with .login() to acquire the appropriate"
-                             " login URL")
+        auth_url = self.auth.get_valid_host()
 
         # Get password from keyring or prompt
         password, password_from_keyring = self._get_password(
@@ -836,76 +1005,23 @@ class AlmaClass(QueryWithLogin):
             on the keyring. Default is False.
         """
 
-        success = False
-        for auth_url in auth_urls:
-            # set session cookies (they do not get set otherwise)
-            cookiesetpage = self._request("GET",
-                                          urljoin(self._get_dataarchive_url(),
-                                                  'rh/forceAuthentication'),
-                                          cache=False)
-            self._login_cookiepage = cookiesetpage
-            cookiesetpage.raise_for_status()
-
-            if (auth_url+'/cas/login' in cookiesetpage.request.url):
-                # we've hit a target, we're good
-                success = True
-                break
-        if not success:
-            raise LoginError("Could not log in to any of the known ALMA "
-                             "authorization portals: {0}".format(auth_urls))
-
-        # Check if already logged in
-        loginpage = self._request("GET", "https://{auth_url}/cas/login".format(auth_url=auth_url),
-                                  cache=False)
-        root = BeautifulSoup(loginpage.content, 'html5lib')
-        if root.find('div', class_='success'):
-            log.info("Already logged in.")
-            return True
-
-        self._auth_url = auth_url
+        self.auth.auth_hosts = auth_urls
 
         username, password = self._get_auth_info(username=username,
                                                  store_password=store_password,
                                                  reenter_password=reenter_password)
 
-        # Authenticate
-        log.info("Authenticating {0} on {1} ...".format(username, auth_url))
-        # Do not cache pieces of the login process
-        data = {kw: root.find('input', {'name': kw})['value']
-                for kw in ('execution', '_eventId')}
-        data['username'] = username
-        data['password'] = password
-        data['submit'] = 'LOGIN'
+        self.auth.login(username, password)
+        self.USERNAME = username
 
-        login_response = self._request("POST", "https://{0}/cas/login".format(auth_url),
-                                       params={'service': self._get_dataarchive_url()},
-                                       data=data,
-                                       cache=False)
-
-        # save the login response for debugging purposes
-        self._login_response = login_response
-        # do not expose password back to user
-        del data['password']
-        # but save the parameters for debug purposes
-        self._login_parameters = data
-
-        authenticated = ('You have successfully logged in' in
-                         login_response.text)
-
-        if authenticated:
-            log.info("Authentication successful!")
-            self.USERNAME = username
-        else:
-            log.exception("Authentication failed!")
-
-        return authenticated
+        return True
 
     def get_cycle0_uid_contents(self, uid):
         """
         List the file contents of a UID from Cycle 0.  Will raise an error
         if the UID is from cycle 1+, since those data have been released in
         a different and more consistent format.  See
-        http://almascience.org/documents-and-tools/cycle-2/ALMAQA2Productsv1.01.pdf
+        https://almascience.org/documents-and-tools/cycle-2/ALMAQA2Productsv1.01.pdf
         for details.
         """
 
@@ -932,7 +1048,7 @@ class AlmaClass(QueryWithLogin):
         if not hasattr(self, '_cycle0_tarfile_content_table'):
             url = urljoin(self._get_dataarchive_url(),
                           'alma-data/archive/cycle-0-tarfile-content')
-            response = self._request('GET', url, cache=True)
+            response = self._request('GET', url, cache=True, timeout=self.TIMEOUT)
 
             # html.parser is needed because some <tr>'s have form:
             # <tr width="blah"> which the default parser does not pick up
@@ -1142,10 +1258,8 @@ class AlmaClass(QueryWithLogin):
         result = self.query_tap(
             "select distinct proposal_abstract from "
             "ivoa.obscore where proposal_id='{}'".format(projectid))
-        if ASTROPY_LT_4_1:
-            return [result[0]['proposal_abstract'].astype(str)]
-        else:
-            return [result[0]['proposal_abstract']]
+
+        return [result[0]['proposal_abstract']]
 
 
 Alma = AlmaClass()
