@@ -1,20 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-import tempfile
-import shutil
-import numpy as np
-import pytest
-import warnings
-from datetime import datetime
+from datetime import datetime, timezone
 import os
+from pathlib import Path
 from urllib.parse import urlparse
 import re
 from unittest.mock import Mock, MagicMock, patch
 
 from astropy import coordinates
 from astropy import units as u
+import numpy as np
+import pytest
+try:
+    # This requires pyvo 1.4
+    from pyvo.dal.exceptions import DALOverflowWarning
+except ImportError:
+    pass
 
 from astroquery.exceptions import CorruptDataWarning
-from astroquery.utils.commons import ASTROPY_LT_4_1
 from .. import Alma
 
 # ALMA tests involving staging take too long, leading to travis timeouts
@@ -29,6 +31,8 @@ all_colnames = {'Project code', 'Source name', 'RA', 'Dec', 'Band',
                 'Spatial resolution', 'Largest angular scale',
                 'QA2 Status', 'Group ous id', 'Pub'}
 
+download_hostname = 'almascience.eso.org'
+
 
 @pytest.fixture
 def alma(request):
@@ -40,34 +44,30 @@ def alma(request):
     """
     alma = Alma()
     alma_site = request.config.getoption('--alma-site',
-                                         'almascience.org')
+                                         'almascience.eso.org')
     alma.archive_url = 'https://{}'.format(alma_site)
     return alma
 
 
 @pytest.mark.remote_data
 class TestAlma:
-    @pytest.fixture()
-    def temp_dir(self, request):
-        my_temp_dir = tempfile.mkdtemp()
-
-        def fin():
-            shutil.rmtree(my_temp_dir)
-        request.addfinalizer(fin)
-        return my_temp_dir
-
     def test_public(self, alma):
-        results = alma.query(payload=None, public=True, maxrec=100)
+        # This warning was added in pyvo 1.4, but we don't test older versions with remote-data
+        with pytest.warns(expected_warning=DALOverflowWarning,
+                          match="Partial result set. Potential causes MAXREC, async storage space, etc."):
+            results = alma.query(payload=None, public=True, maxrec=100)
         assert len(results) == 100
         for row in results:
             assert row['data_rights'] == 'Public'
-        results = alma.query(payload=None, public=False, maxrec=100)
+        with pytest.warns(expected_warning=DALOverflowWarning,
+                          match="Partial result set. Potential causes MAXREC, async storage space, etc."):
+            results = alma.query(payload=None, public=False, maxrec=100)
         assert len(results) == 100
         for row in results:
             assert row['data_rights'] == 'Proprietary'
 
-    def test_SgrAstar(self, temp_dir, alma):
-        alma.cache_location = temp_dir
+    def test_SgrAstar(self, tmp_path, alma):
+        alma.cache_location = tmp_path
 
         result_s = alma.query_object('Sgr A*', legacy_columns=True)
 
@@ -85,12 +85,13 @@ class TestAlma:
             assert row['frequency'] <= 100
             assert '3' in row['band_list']
 
-    @pytest.mark.skipif("SKIP_SLOW",
-                        reason="Extremely slow due to limitations of "
-                               "the implementation")
     def test_bands(self, alma):
         payload = {'band_list': ['5', '7']}
-        result = alma.query(payload)
+        # Added maxrec here as downloading and reading the results take too long.
+        # This warning was added in pyvo 1.4, but we don't test older versions with remote-data
+        with pytest.warns(expected_warning=DALOverflowWarning,
+                          match="Partial result set. Potential causes MAXREC, async storage space, etc."):
+            result = alma.query(payload, maxrec=1000)
         assert len(result) > 0
         for row in result:
             assert ('5' in row['band_list']) or ('7' in row['band_list'])
@@ -116,15 +117,15 @@ class TestAlma:
         for row in result:
             assert 'GRB021004' == row['target_name']
 
-    @pytest.mark.skipif("SKIP_SLOW", reason="Known issue")
     def test_ra_dec(self, alma):
         payload = {'ra_dec': '181.0192d -0.01928d'}
         result = alma.query(payload)
         assert len(result) > 0
 
     @pytest.mark.skipif("SKIP_SLOW")
-    def test_m83(self, temp_dir, alma):
-        alma.cache_location = temp_dir
+    def test_m83(self, tmp_path, alma):
+        # Runs for over 9 minutes
+        alma.cache_location = tmp_path
 
         m83_data = alma.query_object('M83', science=True, legacy_columns=True)
         uids = np.unique(m83_data['Member ous id'])
@@ -138,19 +139,31 @@ class TestAlma:
         # public
         assert not alma.is_proprietary('uid://A001/X12a3/Xe9')
         IVOA_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"
-        now = datetime.utcnow().strftime(IVOA_DATE_FORMAT)[:-3]
-        query = "select top 1 obs_id from ivoa.obscore where " \
+        now = datetime.now(timezone.utc).strftime(IVOA_DATE_FORMAT)[:-3]
+        query = "select top 1 member_ous_uid from ivoa.obscore where " \
                 "obs_release_date > '{}'".format(now)
         result = alma.query_tap(query)
-        assert len(result.table) == 1
+        assert len(result.to_table()) == 1
         # proprietary
-        assert alma.is_proprietary(result.table[0][0])
+        assert alma.is_proprietary(result.to_table()[0][0])
         # non existent
         with pytest.raises(AttributeError):
             alma.is_proprietary('uid://NON/EXI/STING')
 
-    def test_data_info(self, temp_dir, alma):
-        alma.cache_location = temp_dir
+    def test_retrieve_data(self, tmp_path, alma):
+        """
+        Regression test for issue 2490 (the retrieval step will simply fail if
+        given a blank line, so all we're doing is testing that it runs)
+        """
+        alma.cache_location = tmp_path
+
+        # small solar TP-only data set (<1 GB)
+        uid = 'uid://A001/X87c/X572'
+
+        alma.retrieve_data_from_uid([uid])
+
+    def test_data_info(self, tmp_path, alma):
+        alma.cache_location = tmp_path
 
         uid = 'uid://A001/X12a3/Xe9'
         data_info = alma.get_data_info(uid, expand_tarfiles=True)
@@ -164,6 +177,8 @@ class TestAlma:
 
         # compare with tarball version
         data_info_tar = alma.get_data_info(uid, expand_tarfiles=False)
+
+        # The expanded table should be much longer than the non-expanded table.
         assert len(data_info) > len(data_info_tar)
         # size is the same - not working because service inconsistencies
         # assert sum(data_info['content_length']) == \
@@ -175,20 +190,24 @@ class TestAlma:
                 file_url = url
                 break
         assert file_url
-        alma.download_files([file_url], savedir=temp_dir)
-        assert os.stat(os.path.join(temp_dir, file)).st_size
+        alma.download_files([file_url], savedir=tmp_path)
+        assert Path(tmp_path, file).stat().st_size
 
         # mock downloading an entire program
         download_files_mock = Mock()
         alma.download_files = download_files_mock
         alma.retrieve_data_from_uid([uid])
+        trimmed_access_url_list = [e for e in data_info_tar['access_url'].data if len(e) > 0]
+        trimmed_access_urls = (trimmed_access_url_list,)
+        mock_calls = download_files_mock.mock_calls[0][1]
+        print(f"\n\nComparing {mock_calls} to {trimmed_access_urls}\n\n")
+        # comparison = download_files_mock.mock_calls[0][1] == data_info_tar['access_url']
+        assert mock_calls == trimmed_access_urls
+        # assert comparison.all()
 
-        comparison = download_files_mock.mock_calls[0][1] == data_info_tar['access_url']
-        assert comparison.all()
-
-    def test_download_data(self, temp_dir, alma):
+    def test_download_data(self, tmp_path, alma):
         # test only fits files from a program
-        alma.cache_location = temp_dir
+        alma.cache_location = tmp_path
 
         uid = 'uid://A001/X12a3/Xe9'
         data_info = alma.get_data_info(uid, expand_tarfiles=True)
@@ -200,12 +219,14 @@ class TestAlma:
         alma._download_file = download_mock
         urls = [x['access_url'] for x in data_info
                 if fitsre.match(x['access_url'])]
-        results = alma.download_files(urls, savedir=temp_dir)
+        results = alma.download_files(urls, savedir=tmp_path)
         alma._download_file.call_count == len(results)
         assert len(results) == len(urls)
 
-    def test_download_and_extract(self, temp_dir, alma):
-        alma.cache_location = temp_dir
+    def test_download_and_extract(self, tmp_path, alma):
+        # TODO: slowish, runs for ~90s
+
+        alma.cache_location = tmp_path
         alma._cycle0_tarfile_content_table = {'ID': ''}
 
         uid = 'uid://A001/X12a3/Xe9'
@@ -245,11 +266,10 @@ class TestAlma:
                     [asdm_url], include_asdm=True, regex=r'.*\.py')
         delete_mock.assert_called_once_with(
             'cache_path/' + asdm_url.split('/')[-1])
-        assert downloaded_asdm == [os.path.join(temp_dir, 'foo.py')]
+        assert Path(*downloaded_asdm) == Path(tmp_path, 'foo.py')
 
-    @pytest.mark.skipif("SKIP_SLOW", reason="Known issue")
-    def test_doc_example(self, temp_dir, alma):
-        alma.cache_location = temp_dir
+    def test_doc_example(self, tmp_path, alma):
+        alma.cache_location = tmp_path
         m83_data = alma.query_object('M83', legacy_columns=True)
         # the order can apparently sometimes change
         # These column names change too often to keep testing.
@@ -259,32 +279,29 @@ class TestAlma:
         gc_data = alma.query_region(galactic_center, 1 * u.deg)
         # assert len(gc_data) >= 425 # Feb 8, 2016
         assert len(gc_data) >= 50  # Nov 16, 2016
+        content_length_column_name = 'content_length'
 
         uids = np.unique(m83_data['Member ous id'])
-        if ASTROPY_LT_4_1:
-            assert b'uid://A001/X11f/X30' in uids
-            X30 = (m83_data['Member ous id'] == b'uid://A001/X11f/X30')
-            X31 = (m83_data['Member ous id'] == b'uid://A002/X3216af/X31')
-        else:
-            assert 'uid://A001/X11f/X30' in uids
-            X30 = (m83_data['Member ous id'] == 'uid://A001/X11f/X30')
-            X31 = (m83_data['Member ous id'] == 'uid://A002/X3216af/X31')
+
+        assert 'uid://A001/X11f/X30' in uids
+        X30 = (m83_data['Member ous id'] == 'uid://A001/X11f/X30')
+        X31 = (m83_data['Member ous id'] == 'uid://A002/X3216af/X31')
 
         assert X30.sum() == 4  # Jul 13, 2020
         assert X31.sum() == 4  # Jul 13, 2020
         mous1 = alma.get_data_info('uid://A001/X11f/X30')
-        totalsize_mous1 = mous1['size'].sum() * u.Unit(mous1['size'].unit)
+        totalsize_mous1 = mous1[content_length_column_name].sum() * u.Unit(mous1[content_length_column_name].unit)
         assert (totalsize_mous1.to(u.B) > 1.9*u.GB)
 
         mous = alma.get_data_info('uid://A002/X3216af/X31')
-        totalsize_mous = mous['size'].sum() * u.Unit(mous['size'].unit)
+        totalsize_mous = mous[content_length_column_name].sum() * u.Unit(mous[content_length_column_name].unit)
         # More recent ALMA request responses do not include any information
         # about file size, so we have to allow for the possibility that all
         # file sizes are replaced with -1
         assert (totalsize_mous.to(u.GB).value > 52)
 
-    def test_query(self, temp_dir, alma):
-        alma.cache_location = temp_dir
+    def test_query(self, tmp_path, alma):
+        alma.cache_location = tmp_path
 
         result = alma.query(payload={'start_date': '<11-11-2011'},
                             public=False, legacy_columns=True, science=True)
@@ -304,7 +321,6 @@ class TestAlma:
         #                     science=True)
         # assert len(result) == 1
 
-    @pytest.mark.skipif("SKIP_SLOW", reason="ra dec search known issue")
     def test_misc(self, alma):
         # miscellaneous set of common tests
         #
@@ -316,11 +332,13 @@ class TestAlma:
         result = alma.query(payload={'pi_name': '*Bally*'}, public=False,
                             maxrec=10)
         assert result
-        result.write('/tmp/alma-onerow.txt', format='ascii')
+        # Add overwrite=True in case the test previously died unexpectedly
+        # and left the temp file.
+        result.write('/tmp/alma-onerow.txt', format='ascii', overwrite=True)
         for row in result:
             assert 'Bally' in row['obs_creator_name']
         result = alma.query(payload=dict(project_code='2016.1.00165.S'),
-                            public=False, cache=False)
+                            public=False)
         assert result
         for row in result:
             assert '2016.1.00165.S' == row['proposal_id']
@@ -339,8 +357,7 @@ class TestAlma:
 
         result = alma.query_region(
             coordinates.SkyCoord('5:35:14.461 -5:21:54.41', frame='fk5',
-                                 unit=(u.hour, u.deg)), radius=0.034 * u.deg,
-            payload={'energy.frequency-asu': '215 .. 220'})
+                                 unit=(u.hour, u.deg)), radius=0.034 * u.deg)
 
         result = alma.query(payload=dict(project_code='2012.*',
                                          public_data=True))
@@ -367,7 +384,6 @@ class TestAlma:
             assert '6' == row['band_list']
             assert 'ginsburg' in row['obs_creator_name'].lower()
 
-    @pytest.mark.skipif("SKIP_SLOW")
     def test_user(self, alma):
         # miscellaneous set of tests from current users
         rslt = alma.query({'band_list': [6], 'project_code': '2012.1.*'},
@@ -380,10 +396,9 @@ class TestAlma:
     # This has been reported, as it is definitely a bug.
     @pytest.mark.xfail
     @pytest.mark.bigdata
-    @pytest.mark.skipif("SKIP_SLOW")
-    def test_cycle1(self, temp_dir, alma):
+    def test_cycle1(self, tmp_path, alma):
         # About 500 MB
-        alma.cache_location = temp_dir
+        alma.cache_location = tmp_path
         target = 'NGC4945'
         project_code = '2012.1.00912.S'
         payload = {'project_code': project_code,
@@ -427,10 +442,10 @@ class TestAlma:
         assert len(data) == 6
 
     @pytest.mark.skipif("SKIP_SLOW")
-    @pytest.mark.skip("Not working anymore")
-    def test_cycle0(self, temp_dir, alma):
+    @pytest.mark.xfail(reason="Not working anymore")
+    def test_cycle0(self, tmp_path, alma):
         # About 20 MB
-        alma.cache_location = temp_dir
+        alma.cache_location = tmp_path
 
         target = 'NGC4945'
         project_code = '2011.0.00121.S'
@@ -463,7 +478,7 @@ class TestAlma:
         # There are 10 small files, but only 8 unique
         assert len(data) == 8
 
-    def test_keywords(self, temp_dir, alma):
+    def test_keywords(self, tmp_path, alma):
 
         alma.help_tap()
         result = alma.query_tap(
@@ -531,45 +546,42 @@ def test_big_download_regression(alma):
 
 
 @pytest.mark.remote_data
-def test_download_html_file(alma):
-    result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'])
+def test_download_html_file(alma, tmp_path):
+    alma.cache_location = tmp_path
+    result = alma.download_files(
+        ['https://{}/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'.format(download_hostname)])
     assert result
 
 
 @pytest.mark.remote_data
-def test_verify_html_file(alma, caplog):
-    # first, make sure the file is not cached (in case this test gets called repeatedly)
-    # (we are hacking the file later in this test to trigger different failure modes so
-    # we need it fresh)
-    try:
-        result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'], verify_only=True)
-        local_filepath = result[0]
-        os.remove(local_filepath)
-    except FileNotFoundError:
-        pass
-
-    caplog.clear()
+def test_verify_html_file(alma, caplog, tmp_path):
+    alma.cache_location = tmp_path
 
     # download the file
-    result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'])
+    result = alma.download_files(
+        ['https://{}/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'.format(download_hostname)])
     assert 'member.uid___A001_X1284_X1353.qa2_report.html' in result[0]
 
-    result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'], verify_only=True)
+    result = alma.download_files(
+        ['https://{}/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'.format(download_hostname)],
+        verify_only=True)
     assert 'member.uid___A001_X1284_X1353.qa2_report.html' in result[0]
-    local_filepath = result[0]
-    existing_file_length = 66336
-    assert f"Found cached file {local_filepath} with expected size {existing_file_length}." in caplog.text
+    local_filepath = Path(result[0])
+    expected_file_length = local_filepath.stat().st_size
+    assert f"Found cached file {local_filepath} with expected size {expected_file_length}." in caplog.text
 
     # manipulate the file
     with open(local_filepath, 'ab') as fh:
         fh.write(b"Extra Text")
 
     caplog.clear()
-    length = 66336
-    existing_file_length = length + 10
+    new_file_length = expected_file_length + 10
     with pytest.warns(expected_warning=CorruptDataWarning,
-            match=f"Found cached file {local_filepath} with size {existing_file_length} > expected size {length}.  The download is likely corrupted."):
-        result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'], verify_only=True)
+                      match=(f"Found cached file {local_filepath} with size {new_file_length} > expected size "
+                             f"{expected_file_length}.  The download is likely corrupted.")):
+        result = alma.download_files(
+            ['https://{}/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'.format(download_hostname)],
+            verify_only=True)
     assert 'member.uid___A001_X1284_X1353.qa2_report.html' in result[0]
 
     # manipulate the file: make it small
@@ -577,12 +589,10 @@ def test_verify_html_file(alma, caplog):
         fh.write(b"Empty Text")
 
     caplog.clear()
-    result = alma.download_files(['https://almascience.nao.ac.jp/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'], verify_only=True)
+    result = alma.download_files(
+        ['https://{}/dataPortal/member.uid___A001_X1284_X1353.qa2_report.html'.format(download_hostname)],
+        verify_only=True)
     assert 'member.uid___A001_X1284_X1353.qa2_report.html' in result[0]
-    length = 66336
     existing_file_length = 10
-    assert f"Found cached file {local_filepath} with size {existing_file_length} < expected size {length}.  The download should be continued." in caplog.text
-
-    # cleanup: we don't want `test_download_html_file` to fail if this test is re-run
-    if os.path.exists(local_filepath):
-        os.remove(local_filepath)
+    assert (f"Found cached file {local_filepath} with size {existing_file_length} < expected size "
+            f"{expected_file_length}.  The download should be continued.") in caplog.text
