@@ -4,7 +4,6 @@
 import json
 
 from astropy.io import fits
-from astroquery import log
 from astropy.stats import sigma_clipped_stats
 from astropy.coordinates import SkyCoord
 
@@ -16,7 +15,7 @@ else:
     _HAVE_CCDDATA = True
 
 try:
-    from photutils import DAOStarFinder
+    from photutils.detection import DAOStarFinder
 except ImportError:
     _HAVE_SOURCE_DETECTION = False
 else:
@@ -33,6 +32,24 @@ import time
 __all__ = ['AstrometryNet', 'AstrometryNetClass']
 
 
+MISSING_API_KEY = """
+Astrometry.net API key not set. You should either set this in the astroquery configuration file using:
+
+    [astrometry_net]
+    api_key = ADD_YOUR_API_KEY_HERE
+
+or you can set it for this session only using the ``conf`` object:
+
+    from astroquery.astrometry_net import conf
+    conf.api_key = 'ADD_YOUR_API_KEY_HERE'
+
+or using the ``api_key`` property on the ``AstrometryNet`` class:
+
+    from astroquery.astrometry_net import AstrometryNet
+    AstrometryNet.api_key = 'ADD_YOUR_API_KEY_HERE'
+""".lstrip()
+
+
 @async_to_sync
 class AstrometryNetClass(BaseQuery):
     """
@@ -43,7 +60,7 @@ class AstrometryNetClass(BaseQuery):
     TIMEOUT = conf.timeout
     API_URL = url_helpers.join(URL, 'api')
 
-    # These are drawn from http://astrometry.net/doc/net/api.html#submitting-a-url
+    # These are drawn from https://astrometry.net/doc/net/api.html#submitting-a-url
     _constraints = {
         'allow_commercial_use': {'default': 'd', 'type': str, 'allowed': ('d', 'y', 'n')},
         'allow_modifications': {'default': 'd', 'type': str, 'allowed': ('d', 'y', 'n')},
@@ -71,7 +88,7 @@ class AstrometryNetClass(BaseQuery):
     def api_key(self):
         """ Return the Astrometry.net API key. """
         if not conf.api_key:
-            log.error("Astrometry.net API key not in configuration file")
+            raise RuntimeError(MISSING_API_KEY)
         return conf.api_key
 
     @api_key.setter
@@ -103,18 +120,10 @@ class AstrometryNetClass(BaseQuery):
                             values=key_info['allowed']))
 
     def __init__(self):
-        """ Show a warning message if the API key is not in the configuration file. """
-        super(AstrometryNetClass, self).__init__()
-        if not conf.api_key:
-            log.warning("Astrometry.net API key not found in configuration file")
-            log.warning("You need to manually edit the configuration file and add it")
-            log.warning(
-                "You may also register it for this session with AstrometryNet.key = 'XXXXXXXX'")
+        super().__init__()
         self._session_id = None
 
     def _login(self):
-        if not self.api_key:
-            raise RuntimeError('You must set the API key before using this service.')
         login_url = url_helpers.join(self.API_URL, 'login')
         payload = self._construct_payload({'apikey': self.api_key})
         result = self._request('POST', login_url,
@@ -189,8 +198,8 @@ class AstrometryNetClass(BaseQuery):
                 raise ValueError('Scale type {} requires '
                                  'values for {}'.format(scale_type, required_keys))
 
-    def monitor_submission(self, submission_id,
-                           solve_timeout=TIMEOUT):
+    def monitor_submission(self, submission_id, *,
+                           solve_timeout=TIMEOUT, verbose=True, return_submission_id=False):
         """
         Monitor the submission for completion.
 
@@ -202,14 +211,20 @@ class AstrometryNetClass(BaseQuery):
         solve_timeout : ``int``
             Time, in seconds, to wait for the astrometry.net solver to find
             a solution.
+        verbose : bool, optional
+            Whether to print out information about the solving.
+        return_submission_id : bool, optional
+            Whether to return the Submission ID number.
 
         Returns
         -------
 
-        None or `astropy.io.fits.Header`
+        None or `~astropy.io.fits.Header` or (`~astropy.io.fits.Header`, str)
             The contents of the returned object depend on whether the solve
             succeeds or fails. If the solve succeeds the header with
-            the WCS solution generated by astrometry.net is returned. If the solve
+            the WCS solution generated by astrometry.net is returned. A tuple
+            containing WCS solution and Submission ID is return if the
+            return_submission_id parameter is set True. If the solve
             fails then an empty dictionary is returned. See below for the outcome
             if the solve times out.
 
@@ -223,7 +238,8 @@ class AstrometryNetClass(BaseQuery):
         """
         has_completed = False
         job_id = None
-        print('Solving', end='', flush=True)
+        if verbose:
+            print('Solving', end='', flush=True)
         start_time = time.time()
         status = ''
         while not has_completed:
@@ -242,7 +258,8 @@ class AstrometryNetClass(BaseQuery):
             elapsed = now - start_time
             timed_out = elapsed > solve_timeout
             has_completed = (status in ['success', 'failure'] or timed_out)
-            print('.', end='', flush=True)
+            if verbose:
+                print('.', end='', flush=True)
         if status == 'success':
             wcs_url = url_helpers.join(self.URL, 'wcs_file', str(job_id))
             wcs_response = self._request('GET', wcs_url)
@@ -255,10 +272,15 @@ class AstrometryNetClass(BaseQuery):
         else:
             # Try to future-proof a little bit
             raise RuntimeError('Unrecognized status {}'.format(status))
-        return wcs
+        if return_submission_id is False:
+            return wcs
+        else:
+            return (wcs, submission_id)
 
-    def solve_from_source_list(self, x, y, image_width, image_height,
+    def solve_from_source_list(self, x, y, image_width, image_height, *,
                                solve_timeout=TIMEOUT,
+                               verbose=True,
+                               return_submission_id=False,
                                **settings
                                ):
         """
@@ -278,6 +300,10 @@ class AstrometryNetClass(BaseQuery):
         solve_timeout : int
             Time, in seconds, to wait for the astrometry.net solver to find
             a solution.
+        verbose : bool, optional
+            Whether to print out information about the solving.
+        return_submission_id : bool, optional
+            Whether to return the Submission ID number.
 
         For a list of the remaining settings, use the method
         `~AstrometryNetClass.show_allowed_settings`.
@@ -301,13 +327,17 @@ class AstrometryNetClass(BaseQuery):
         response_d = response.json()
         submission_id = response_d['subid']
         return self.monitor_submission(submission_id,
-                                       solve_timeout=solve_timeout)
+                                       solve_timeout=solve_timeout,
+                                       verbose=verbose,
+                                       return_submission_id=return_submission_id)
 
-    def solve_from_image(self, image_file_path, force_image_upload=False,
+    def solve_from_image(self, image_file_path, *, force_image_upload=False,
                          ra_key=None, dec_key=None,
                          ra_dec_units=None,
                          fwhm=3, detect_threshold=5,
                          solve_timeout=TIMEOUT,
+                         verbose=True,
+                         return_submission_id=False,
                          **settings):
         """
         Plate solve from an image, either by uploading the image to
@@ -343,10 +373,17 @@ class AstrometryNetClass(BaseQuery):
         ra_dec_units : tuple, optional
             Tuple specifying the units of the right ascension and declination in
             the header. The default value is ``('hour', 'degree')``.
+
         solve_timeout : int
             Time, in seconds, to wait for the astrometry.net solver to find
             a solution.
 
+        verbose : bool, optional
+            Whether to print out information about the solving.
+
+        return_submission_id : bool, optional
+            Whether to return the Submission ID number.
+
         For a list of the remaining settings, use the method
         `~AstrometryNetClass.show_allowed_settings`.
         """
@@ -386,32 +423,44 @@ class AstrometryNetClass(BaseQuery):
             else:
                 with fits.open(image_file_path) as f:
                     data = f[0].data
-
-            print("Determining background stats", flush=True)
+            if verbose:
+                print("Determining background stats", flush=True)
             mean, median, std = sigma_clipped_stats(data, sigma=3.0,
                                                     maxiters=5)
             daofind = DAOStarFinder(fwhm=fwhm,
                                     threshold=detect_threshold * std)
-            print("Finding sources", flush=True)
+            if verbose:
+                print("Finding sources", flush=True)
             sources = daofind(data - median)
-            print('Found {} sources'.format(len(sources)), flush=True)
+            if verbose:
+                print('Found {} sources'.format(len(sources)), flush=True)
             # astrometry.net wants a sorted list of sources
             # Sort first (which puts things in ascending order)
             sources.sort('flux')
             # Reverse to get descending order
             sources.reverse()
-            print(sources)
+            if verbose:
+                print(sources)
+
+            # It turns out astrometry.net is 1-indexed, so add 1 to the source positions.
+            sources['xcentroid'] += 1
+            sources['ycentroid'] += 1
             return self.solve_from_source_list(sources['xcentroid'],
                                                sources['ycentroid'],
                                                ccd.header['naxis1'],
                                                ccd.header['naxis2'],
+                                               solve_timeout=solve_timeout,
+                                               verbose=verbose,
+                                               return_submission_id=return_submission_id,
                                                **settings)
         if response.status_code != 200:
             raise RuntimeError('Post of job failed')
         response_d = response.json()
         submission_id = response_d['subid']
         return self.monitor_submission(submission_id,
-                                       solve_timeout=solve_timeout)
+                                       solve_timeout=solve_timeout,
+                                       verbose=verbose,
+                                       return_submission_id=return_submission_id)
 
 
 # the default tool for users to interact with is an instance of the Class
