@@ -65,11 +65,10 @@ def patch_get_readable_fileobj(request):
         if isinstance(filename, str):
             if '1376235131.430670' in filename:
                 is_binary = kwargs.get('encoding', None) == 'binary'
-                file_obj = open(data_path('1376235131.430670.resu'),
-                                "r" + ('b' if is_binary else ''))
+                with open(data_path('1376235131.430670.resu'), "r" + ('b' if is_binary else '')) as file_obj:
+                    yield file_obj
         else:
-            file_obj = filename
-        yield file_obj
+            yield filename
 
     mp = request.getfixturevalue("monkeypatch")
 
@@ -80,7 +79,8 @@ def patch_get_readable_fileobj(request):
 
 def post_mockreturn(method, url, data, timeout=10, stream=True, **kwargs):
     filename = data_path('query_return.iframe.html')
-    content = open(filename, 'rb').read()
+    with open(filename, 'rb') as infile:
+        content = infile.read()
     return MockResponseBesancon(content, filename, **kwargs)
 
 
@@ -105,6 +105,6 @@ def test_default_params():
 class MockResponseBesancon(MockResponse):
 
     def __init__(self, content=None, url=None, headers={}, **kwargs):
-        super(MockResponseBesancon, self).__init__(content)
+        super().__init__(content)
         self.raw = url  # StringIO.StringIO(url)
         self.headers = headers
