@@ -1,5 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import os
+from pathlib import Path
+
+from astropy.utils import minversion
+import numpy as np
+import pytest
 # this contains imports plugins that configure py.test for astropy tests.
 # by importing them here in conftest.py they are discoverable by py.test
 # no matter how it is invoked within the source tree.
@@ -8,6 +13,11 @@ from pytest_astropy_header.display import (PYTEST_HEADER_MODULES,
                                            TESTED_VERSIONS)
 
 
+# Keep this until we require numpy to be >=2.0
+if minversion(np, "2.0.0.dev0+git20230726"):
+    np.set_printoptions(legacy="1.25")
+
+
 def pytest_configure(config):
     config.option.astropy_header = True
 
@@ -17,8 +27,6 @@ def pytest_configure(config):
 
 try:
     PYTEST_HEADER_MODULES['Astropy'] = 'astropy'
-    PYTEST_HEADER_MODULES['APLpy'] = 'aplpy'
-    PYTEST_HEADER_MODULES['pyregion'] = 'pyregion'
     PYTEST_HEADER_MODULES['regions'] = 'regions'
     PYTEST_HEADER_MODULES['pyVO'] = 'pyvo'
     PYTEST_HEADER_MODULES['mocpy'] = 'mocpy'
@@ -53,7 +61,18 @@ def pytest_addoption(parser):
     parser.addoption(
         '--alma-site',
         action='store',
-        default='almascience.org',
+        default='almascience.eso.org',
         help='ALMA site (almascience.nrao.edu, almascience.eso.org or '
              'almascience.nao.ac.jp for example)'
     )
+
+
+@pytest.fixture(scope='function')
+def tmp_cwd(tmp_path):
+    """Perform test in a pristine temporary working directory."""
+    old_dir = Path.cwd()
+    os.chdir(tmp_path)
+    try:
+        yield tmp_path
+    finally:
+        os.chdir(old_dir)
