@@ -40,7 +40,7 @@ class CosmoSimClass(QueryWithLogin):
     USERNAME = conf.username
 
     def __init__(self):
-        super(CosmoSimClass, self).__init__()
+        super().__init__()
 
     def _login(self, username=None, password=None, store_password=False,
                reenter_password=False):
@@ -91,8 +91,8 @@ class CosmoSimClass(QueryWithLogin):
                                       cache=False)
         if authenticated.status_code == 200:
             warnings.warn("Authentication successful!")
-        elif (authenticated.status_code == 401 or
-              authenticated.status_code == 403):
+        elif (authenticated.status_code == 401
+              or authenticated.status_code == 403):
             warnings.warn("Authentication failed!")
         elif authenticated.status_code == 503:
             warnings.warn("Service Temporarily Unavailable...")
@@ -100,8 +100,8 @@ class CosmoSimClass(QueryWithLogin):
         # Generating dictionary of existing tables
         self._existing_tables()
 
-        if (authenticated.status_code == 200 and
-                password_from_keyring is None and store_password):
+        if (authenticated.status_code == 200
+                and password_from_keyring is None and store_password):
             keyring.set_password("astroquery:www.cosmosim.org",
                                  self.username, self.password)
 
@@ -128,8 +128,8 @@ class CosmoSimClass(QueryWithLogin):
         -------
         """
 
-        if (hasattr(self, 'username') and hasattr(self, 'password') and
-                hasattr(self, 'session')):
+        if (hasattr(self, 'username') and hasattr(self, 'password')
+                and hasattr(self, 'session')):
             if deletepw is True:
                 try:
                     keyring.delete_password("astroquery:www.cosmosim.org",
@@ -151,8 +151,8 @@ class CosmoSimClass(QueryWithLogin):
         Public function which checks the status of a user login attempt.
         """
 
-        if (hasattr(self, 'username') and hasattr(self, 'password') and
-                hasattr(self, 'session')):
+        if (hasattr(self, 'username') and hasattr(self, 'password')
+                and hasattr(self, 'session')):
             authenticated = self._request('POST', CosmoSim.QUERY_URL,
                                           auth=(self.username, self.password),
                                           cache=False)
@@ -365,8 +365,8 @@ class CosmoSimClass(QueryWithLogin):
                 else:
                     matching_tables = [[self.table_dict[i]
                                         for i in self.table_dict.keys()
-                                        if self.table_dict[i] == miter and
-                                        self.job_dict[i] in phase][0]
+                                        if self.table_dict[i] == miter
+                                        and self.job_dict[i] in phase][0]
                                        for miter in matching_tables]
             self._existing_tables()  # creates a fresh up-to-date table_dict
 
@@ -904,16 +904,16 @@ class CosmoSimClass(QueryWithLogin):
             tmp2_largest = 0
             for proj in projects:
                 size = len((self.db_dict['{0}'.format(proj)].keys()))
-                proj_list += (['@ {}'.format(proj)] +
-                              ['' for i in range(size - 1)] +
-                              ['-' * (largest + 2)])
+                proj_list += (['@ {}'.format(proj)]
+                              + ['' for i in range(size - 1)]
+                              + ['-' * (largest + 2)])
                 tmp_largest = max([len(str(key))
                                    for key in self.db_dict[proj].keys()])
                 attr_list += (['@ {}'.format(key)
                               if isinstance(self.db_dict[proj][key], dict)
                               else '{}:'.format(key)
-                              for key in self.db_dict[proj].keys()] +
-                              ['-' * (tmp_largest + 2)])
+                              for key in self.db_dict[proj].keys()]
+                              + ['-' * (tmp_largest + 2)])
                 tmpinfosize = max([len(self.db_dict[proj][key])
                                    if isinstance(self.db_dict[proj][key], str)
                                    else 0
@@ -924,8 +924,7 @@ class CosmoSimClass(QueryWithLogin):
                 info_list += ([self.db_dict[proj][key]
                               if isinstance(self.db_dict[proj][key], str)
                               else ""
-                              for key in self.db_dict[proj].keys()] +
-                              ['-' * tmp2_largest])
+                              for key in self.db_dict[proj].keys()] + ['-' * tmp2_largest])
             t['Projects'] = proj_list
             t['Project Items'] = attr_list
             t['Information'] = info_list
@@ -953,7 +952,7 @@ class CosmoSimClass(QueryWithLogin):
                             size2 = max(size2, len(list(self.db_dict[db]['tables']
                                                    [table]['columns']
                                                    [col].keys())))
-                        except(KeyError, NameError):
+                        except (KeyError, NameError):
                             log.error("Must first specify a valid column "
                                       "of the `{0}` table within the `{1}`"
                                       " db".format(table, db))
@@ -963,20 +962,18 @@ class CosmoSimClass(QueryWithLogin):
                               "the `{0}` db.".format(db))
                     return
 
-            t['Projects'] = (['--> @ {}:'.format(db)] +
-                             ['' for i in range(size2 - 1)])
+            t['Projects'] = (['--> @ {}:'.format(db)] + ['' for i in range(size2 - 1)])
             t['Project Items'] = (
                 ['--> @ {}:'.format(key)
-                 if (isinstance(self.db_dict[db][key], dict) and
-                     (len(list(self.db_dict[db][key].keys())) ==
-                      len(list(self.db_dict[db]['tables'].keys()))))
+                 if (isinstance(self.db_dict[db][key], dict)
+                     and (len(list(self.db_dict[db][key].keys()))
+                          == len(list(self.db_dict[db]['tables'].keys()))))
                  else '@ {}'.format(key)
-                 if (isinstance(self.db_dict[db][key], dict) and
-                     (len(list(self.db_dict[db][key].keys())) !=
-                      len(self.db_dict[db]['tables'].keys())))
+                 if (isinstance(self.db_dict[db][key], dict)
+                     and (len(list(self.db_dict[db][key].keys()))
+                          != len(self.db_dict[db]['tables'].keys())))
                  else str(key)
-                 for key in self.db_dict[db].keys()] +
-                ['' for i in range(size2 - size1)])
+                 for key in self.db_dict[db].keys()] + ['' for i in range(size2 - size1)])
             # if only db is specified
             if not table:
                 if not col:
@@ -989,31 +986,27 @@ class CosmoSimClass(QueryWithLogin):
             # if table has been specified
             else:
                 reordered = (
-                    [str(table)] +
-                    sorted([key for key in self.db_dict[db]['tables'].keys()
-                            if key != table], key=len))
+                    [str(table)]
+                    + sorted([key for key in self.db_dict[db]['tables'].keys()
+                              if key != table], key=len))
                 t['Tables'] = (
                     ['--> @ {}:'.format(i)
-                     if (i == table and
-                         isinstance(self.db_dict[db]['tables'][i], dict))
+                     if (i == table and isinstance(self.db_dict[db]['tables'][i], dict))
                      else '@ {}'.format(i)
-                     if (i != table and
-                         isinstance(self.db_dict[db]['tables'][i], dict))
+                     if (i != table and isinstance(self.db_dict[db]['tables'][i], dict))
                      else str(i)
-                     for i in reordered] +
-                    ['' for j in range(size2 - len(reordered))])
+                     for i in reordered] + ['' for j in range(size2 - len(reordered))])
 
                 # if column has been specified
                 if col:
                     tblcols_dict = self.db_dict[db]['tables'][table].keys()
                     t['Table Items'] = (
-                        ['--> @ columns:'] +
-                        [i for i in tblcols_dict if i != 'columns'] +
-                        ['' for j in range(size2 - len(tblcols_dict))])
+                        ['--> @ columns:']
+                        + [i for i in tblcols_dict if i != 'columns']
+                        + ['' for j in range(size2 - len(tblcols_dict))])
                     col_dict = (self.db_dict[db]['tables'][table]
                                 ['columns'].keys())
-                    reordered = ([str(col)] +
-                                 [i for i in col_dict if i != col])
+                    reordered = ([str(col)] + [i for i in col_dict if i != col])
 
                     temp_columns = []
 
@@ -1031,14 +1024,12 @@ class CosmoSimClass(QueryWithLogin):
 
                     if len(col_dict) < size2:
                         size_diff = size2 - len(col_dict)
-                        t['Columns'] = (temp_columns +
-                                        ['' for j in range(size_diff)])
+                        t['Columns'] = (temp_columns + ['' for j in range(size_diff)])
 
                         colinfo_dict = col_dict = columns[col]
                         t['Col. Info'] = (
                             ['{} : {}'.format(i, colinfo_dict[i])
-                             for i in colinfo_dict.keys()] +
-                            ['' for j in range(size2 - len(colinfo_dict))])
+                             for i in colinfo_dict.keys()] + ['' for j in range(size2 - len(colinfo_dict))])
                     else:
                         t['Columns'] = temp_columns
 
@@ -1053,20 +1044,17 @@ class CosmoSimClass(QueryWithLogin):
                         tmp_table = self.db_dict[db]['tables'][table]
                         t['Table Items'] = (
                             ['@ {}'.format(i) if isinstance(tmp_table[i], dict)
-                             else '{}:'.format(i) for i in tblcols_dict] +
-                            ['' for i in range(size_diff)])
+                             else '{}:'.format(i) for i in tblcols_dict] + ['' for i in range(size_diff)])
                         t['Table Info'] = (
                             [str(tmp_table[i])
                              if not isinstance(tmp_table[i], dict)
-                             else '' for i in tblcols_dict] +
-                            ['' for i in range(size_diff)])
+                             else '' for i in tblcols_dict] + ['' for i in range(size_diff)])
                         if len(col_dict) < size2:
                             t['Columns'] = (
                                 ['@ {}'.format(i)
                                  if isinstance(tmp_table['columns'][i], dict)
                                  else str(i)
-                                 for i in reordered] +
-                                ['' for i in range(size2 - len(col_dict))])
+                                 for i in reordered] + ['' for i in range(size2 - len(col_dict))])
                         else:
                             t['Columns'] = reordered
                     else:
