@@ -1,13 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-
-@author: Elena Colomo
-@contact: ecolomo@esa.int
+========================
+XMM-Newton Setup Package
+========================
 
 European Space Astronomy Centre (ESAC)
 European Space Agency (ESA)
 
-Created on 4 Sept. 2019
 """
 
 
