@@ -4,17 +4,10 @@
 Gaia TAP plus
 =============
 
-@author: Juan Carlos Segovia
-@contact: juan.carlos.segovia@sciops.esa.int
-
 European Space Astronomy Centre (ESAC)
 European Space Agency (ESA)
 
-Created on 30 jun. 2016
-
-
 """
-from astroquery.utils.tap.core import TapPlus
 from astropy import config as _config
 
 
@@ -23,7 +16,7 @@ class Conf(_config.ConfigNamespace):
     Configuration parameters for `astroquery.gaia`.
     """
 
-    MAIN_GAIA_TABLE = _config.ConfigItem("gaiadr2.gaia_source",
+    MAIN_GAIA_TABLE = _config.ConfigItem("gaiadr3.gaia_source",
                                          "GAIA source data table")
     MAIN_GAIA_TABLE_RA = _config.ConfigItem("ra",
                                             "Name of RA parameter in table")
@@ -37,7 +30,14 @@ class Conf(_config.ConfigNamespace):
                                       'XP_SAMPLED',
                                       'RVS',
                                       'MCMC_GSPPHOT',
-                                      'MCMC_MSC']
+                                      'MCMC_MSC',
+                                      'EPOCH_ASTROMETRY',
+                                      'RV_EPOCH_SINGLE',
+                                      'RV_EPOCH_DOUBLE',
+                                      'RVS_EPOCH',
+                                      'RVS_TRANSIT']
+
+    VALID_LINKING_PARAMETERS = {'SOURCE_ID', 'TRANSIT_ID', 'IMAGE_ID'}
 
 
 conf = Conf()
