@@ -48,19 +48,19 @@ def test_input():
     b = core.Skybot.cone_search(SkyCoord(ra=100, dec=20, unit=(u.deg, u.deg),
                                          frame='icrs'),
                                 1, 2451200, get_query_payload=True)
-    assert(a['-ra'] == b['-ra'])
-    assert(a['-dec'] == b['-dec'])
+    assert (a['-ra'] == b['-ra'])
+    assert (a['-dec'] == b['-dec'])
 
     # test rad input
     a = core.Skybot.cone_search((100, 20), 1, 2451200, get_query_payload=True)
     b = core.Skybot.cone_search((100, 20), Angle(60*u.arcmin),
                                 2451200, get_query_payload=True)
-    assert(a['-rd'] == b['-rd'])
+    assert (a['-rd'] == b['-rd'])
 
     with pytest.warns(UserWarning, match='search cone radius'):
         a = core.Skybot.cone_search((100, 20), 100, 2451200,
                                     get_query_payload=True)
-        assert(a['-rd'] == 10)
+        assert (a['-rd'] == 10)
 
     # test epoch input
     a = core.Skybot.cone_search((100, 20), 1, 2451200, get_query_payload=True)
@@ -69,7 +69,7 @@ def test_input():
     c = core.Skybot.cone_search((100, 20), 1, Time('1999-01-21 12:00',
                                                    format='iso'),
                                 get_query_payload=True)
-    assert(a['-ep'] == b['-ep'] == c['-ep'])
+    assert (a['-ep'] == b['-ep'] == c['-ep'])
 
     # test position_error input
     a = core.Skybot.cone_search((100, 20), 1, 2451200, position_error=10,
@@ -77,29 +77,29 @@ def test_input():
     b = core.Skybot.cone_search((100, 20), 1, 2451200,
                                 position_error=Angle(10*u.arcsec),
                                 get_query_payload=True)
-    assert(a['-filter'] == b['-filter'])
+    assert (a['-filter'] == b['-filter'])
 
     with pytest.warns(UserWarning, match='positional error'):
         a = core.Skybot.cone_search((100, 20), 1, 2451200, position_error=1000,
                                     get_query_payload=True)
-        assert(a['-filter'] == 120)
+        assert (a['-filter'] == 120)
 
     # test target filters
     a = core.Skybot.cone_search((100, 20), 1, 2451200, get_query_payload=True)
-    assert(a['-objFilter'] == '111')
+    assert (a['-objFilter'] == '111')
     a = core.Skybot.cone_search((100, 20), 1, 2451200, find_planets=False,
                                 get_query_payload=True)
-    assert(a['-objFilter'] == '101')
+    assert (a['-objFilter'] == '101')
     a = core.Skybot.cone_search((100, 20), 1, 2451200, find_asteroids=False,
                                 get_query_payload=True)
-    assert(a['-objFilter'] == '011')
+    assert (a['-objFilter'] == '011')
     a = core.Skybot.cone_search((100, 20), 1, 2451200, find_comets=False,
                                 get_query_payload=True)
-    assert(a['-objFilter'] == '110')
+    assert (a['-objFilter'] == '110')
 
     # test coordinate reference system
     a = core.Skybot.cone_search((100, 20), 1, 2451200, get_query_payload=True)
-    assert(a['-refsys'] == 'EQJ2000')
+    assert (a['-refsys'] == 'EQJ2000')
 
 
 def general_query(patch_request):
@@ -107,10 +107,10 @@ def general_query(patch_request):
 
     a = core.Skybot.cone_search((0, 0), 0.5, 2451200)
 
-    assert(len(a) == 141)
-    assert(type(a['Number'][0]) == int)
-    assert(a['RA'][0] == 359.94077541666667*u.deg)
-    assert(a['DEC'][0] == -0.013904166666666667*u.deg)
+    assert (len(a) == 141)
+    assert isinstance(a['Number'][0], int)
+    assert (a['RA'][0] == 359.94077541666667*u.deg)
+    assert (a['DEC'][0] == -0.013904166666666667*u.deg)
 
 
 def test_get_raw_response(patch_request):
@@ -131,6 +131,6 @@ def test_parsing_unnumbered_asteroids(patch_request):
                                 location=260,
                                 position_error=15*u.arcsec)
 
-    assert(isinstance(a['Number'], MaskedColumn))
+    assert (isinstance(a['Number'], MaskedColumn))
 
-    assert(a['Number'].mask.sum() > 0)
+    assert (a['Number'].mask.sum() > 0)
