@@ -17,13 +17,11 @@ from astropy.coordinates import SkyCoord
 from astropy.io import ascii
 from astropy.io.votable import parse_single_table
 from astropy.table import QTable
-from astropy.utils import deprecated, deprecated_renamed_argument
 from astropy.utils.exceptions import AstropyWarning
 
 # Import astroquery utilities
-from astroquery.exceptions import (InputWarning, InvalidQueryError, NoResultsWarning,
-                          RemoteServiceError)
-from astroquery.query import BaseQuery
+from astroquery.exceptions import InputWarning, InvalidQueryError, NoResultsWarning, RemoteServiceError
+from astroquery.query import BaseQuery, BaseVOQuery
 from astroquery.utils import async_to_sync, commons
 from astroquery.utils.class_or_instance import class_or_instance
 from astroquery.ipac.nexsci.nasa_exoplanet_archive import conf
@@ -116,10 +114,9 @@ def get_tap_tables():
     """Tables accessed by API are gradually migrating to TAP service. Generate current list of tables in TAP."""
     tap = pyvo.dal.tap.TAPService(baseurl=conf.url_tap)
     response = tap.search(query="select * from TAP_SCHEMA.tables", language="ADQL")
-    if not commons.ASTROPY_LT_4_1:
-        tables = [table for table in response["table_name"].data if "TAP_SCHEMA." not in table]
-    else:
-        tables = [table.decode() for table in response["table_name"].data if b"TAP_SCHEMA." not in table]
+
+    tables = [table for table in response["table_name"].data if "TAP_SCHEMA." not in table]
+
     return tables
 
 
@@ -132,7 +129,7 @@ class InvalidTableError(InvalidQueryError):
 # Class decorator, async_to_sync, modifies NasaExoplanetArchiveClass to convert
 # all query_x_async methods to query_x methods
 @async_to_sync
-class NasaExoplanetArchiveClass(BaseQuery):
+class NasaExoplanetArchiveClass(BaseVOQuery, BaseQuery):
     """
     The interface for querying the NASA Exoplanet Archive TAP and API services
 
@@ -225,24 +222,29 @@ class NasaExoplanetArchiveClass(BaseQuery):
         if "json" in criteria["format"].lower():
             raise InvalidQueryError("The 'json' format is not supported")
 
-        # Build the query (and return it if requested)
+        # Build the query
         request_payload = dict(table=table, **criteria)
-        if get_query_payload:
-            return request_payload
 
         # Use the default cache setting if one was not provided
         if cache is None:
             cache = self.CACHE
 
         if table in self.TAP_TABLES:
-            tap = pyvo.dal.tap.TAPService(baseurl=self.URL_TAP)
+            tap = pyvo.dal.tap.TAPService(baseurl=self.URL_TAP, session=self._session)
             # construct query from table and request_payload (including format)
             tap_query = self._request_to_sql(request_payload)
+
+            if get_query_payload:
+                return tap_query
+
             try:
                 response = tap.search(query=tap_query, language='ADQL')  # Note that this returns a VOTable
             except Exception as err:
                 raise InvalidQueryError(str(err))
         else:
+            if get_query_payload:
+                return request_payload
+
             response = self._request(
                 "GET", self.URL_API, params=request_payload, timeout=self.TIMEOUT, cache=cache,
             )
@@ -374,7 +376,7 @@ class NasaExoplanetArchiveClass(BaseQuery):
         return self.query_criteria_async(table, get_query_payload=get_query_payload, cache=cache, **criteria)
 
     @class_or_instance
-    def query_aliases(self, object_name, *, cache=None):
+    def query_aliases(self, object_name, *, cache=False):
         """
         Search for aliases for a given confirmed planet or planet host
 
@@ -394,19 +396,23 @@ class NasaExoplanetArchiveClass(BaseQuery):
         data = self._request_query_aliases(object_name)
 
         try:
-            objname_split = object_name.split()
-            if len(objname_split) > 1 and len(objname_split[-1]) == 1 and objname_split[-1].isalpha():
-                pl_letter = object_name.split()[-1]
+            resolved_name = data['manifest']['resolved_name']
+            resolved_name_split = resolved_name.split()
+            system_name = data['manifest']['system_name']
+            n_stars = data['manifest']['system_snapshot']['number_of_stars']
+
+            # Asked for system
+            if resolved_name == system_name and n_stars > 1:
+                aliases = data['system']['system_info']['alias_set']['aliases']
+            # Asked for planet
+            elif (len(resolved_name_split) > 1 and len(resolved_name_split[-1]) == 1
+                  and resolved_name_split[-1].isalpha() and resolved_name_split[-1].islower()):
+                aliases = data['system']['objects']['planet_set']['planets'][resolved_name]['alias_set']['aliases']
+            # Asked for star
             else:
-                pl_letter = ''
+                aliases = data['system']['objects']['stellar_set']['stars'][resolved_name]['alias_set']['aliases']
 
-            default_objname = [data['system']['system_info']['alias_set']['default_name']]
-            other_objnames = list(set(data['system']['objects']['stellar_set']['stars'][default_objname[0]]['alias_set']['aliases']) - set(default_objname))
-            other_objnames.sort()
-            aliases = default_objname + other_objnames
-
-            if pl_letter:
-                aliases = [a + ' ' + pl_letter for a in aliases]
+            aliases = [resolved_name] + list(set(aliases) - set([resolved_name]))
 
         except KeyError:
             aliases = []
@@ -570,7 +576,16 @@ class NasaExoplanetArchiveClass(BaseQuery):
             column_data.append(data[col])
 
         # Build the new `QTable` and copy over the data masks if there are any
-        result = QTable(column_data, names=column_names, masked=len(column_masks) > 0)
+        # Some units cannot be used for MaskedQuantities, we catch the specific UserWarning
+        # about them here as the end user can do nothing about this
+        with warnings.catch_warnings():
+            colnames_to_ignore = ['surface_gravity', 'st_lum', 'st_lumerr1', 'st_lumerr2',
+                                  'st_logg', 'st_loggerr1', 'st_loggerr2']
+            for column in colnames_to_ignore:
+                warnings.filterwarnings('ignore', message=f'column {column} has a unit but',
+                                        category=UserWarning)
+            result = QTable(column_data, names=column_names, masked=len(column_masks) > 0)
+
         for key, mask in column_masks.items():
             result[key].mask = mask
 
@@ -629,25 +644,6 @@ class NasaExoplanetArchiveClass(BaseQuery):
 
         return data
 
-    def _handle_all_columns_argument(self, **kwargs):
-        """
-        Deal with the ``all_columns`` argument that was exposed by earlier versions
-
-        This method will warn users about this deprecated argument and update the query syntax
-        to use ``select='*'``.
-        """
-        # We also have to manually pop these arguments from the dict because
-        # `deprecated_renamed_argument` doesn't do that for some reason for all supported astropy
-        # versions (v3.1 was beheaving as expected)
-        kwargs.pop("show_progress", None)
-        kwargs.pop("table_path", None)
-
-        # Deal with `all_columns` properly
-        if kwargs.pop("all_columns", None):
-            kwargs["select"] = kwargs.get("select", "*")
-
-        return kwargs
-
     @class_or_instance
     def _request_to_sql(self, request_payload):
         """Convert request_payload dict to SQL query string to be parsed by TAP."""
@@ -658,7 +654,7 @@ class NasaExoplanetArchiveClass(BaseQuery):
             request_payload["order by"] = request_payload.pop("order")
         if "format" in request_payload.keys():
             # TODO figure out what to do with the format keyword
-            responseformat = request_payload.pop("format")
+            request_payload.pop("format")
         if "ra" in request_payload.keys():  # means this is a `query_region` call
             request_payload["where"] = ("contains(point('icrs',ra,dec),circle('icrs',{0},{1},{2}))=1"
                                         .format(request_payload["ra"], request_payload["dec"],
@@ -677,51 +673,5 @@ class NasaExoplanetArchiveClass(BaseQuery):
 
         return tap_query
 
-    @deprecated(since="v0.4.1", alternative="query_object")
-    @deprecated_renamed_argument(["show_progress", "table_path"],
-                                 [None, None], "v0.4.1", arg_in_kwargs=True)
-    def query_planet(self, planet_name, cache=None, **criteria):
-        """
-        Search the ``exoplanets`` table for a confirmed planet
-
-        Parameters
-        ----------
-        planet_name : str
-            The name of a confirmed planet. If ``regularize`` is ``True``, an attempt will be made
-            to regularize this name using the ``aliastable`` table.
-        cache : bool, optional
-            Should the request result be cached? This can be useful for large repeated queries,
-            but since the data in the archive is updated regularly, this defaults to ``False``.
-        **criteria
-            Any other filtering criteria to apply. Values provided using the ``where`` keyword will
-            be ignored.
-        """
-        criteria = self._handle_all_columns_argument(**criteria)
-        criteria["where"] = "pl_name='{0}'".format(planet_name.strip())
-        return self.query_criteria("exoplanets", cache=cache, **criteria)
-
-    @deprecated(since="v0.4.1", alternative="query_object")
-    @deprecated_renamed_argument(["show_progress", "table_path"],
-                                 [None, None], "v0.4.1", arg_in_kwargs=True)
-    def query_star(self, host_name, cache=None, **criteria):
-        """
-        Search the ``exoplanets`` table for a confirmed planet host
-
-        Parameters
-        ----------
-        host_name : str
-            The name of a confirmed planet host. If ``regularize`` is ``True``, an attempt will be
-            made to regularize this name using the ``aliastable`` table.
-        cache : bool, optional
-            Should the request result be cached? This can be useful for large repeated queries,
-            but since the data in the archive is updated regularly, this defaults to ``False``.
-        **criteria
-            Any other filtering criteria to apply. Values provided using the ``where`` keyword will
-            be ignored.
-        """
-        criteria = self._handle_all_columns_argument(**criteria)
-        criteria["where"] = "pl_hostname='{0}'".format(host_name.strip())
-        return self.query_criteria("exoplanets", cache=cache, **criteria)
-
 
 NasaExoplanetArchive = NasaExoplanetArchiveClass()
