@@ -5,6 +5,7 @@ import os
 
 
 def get_package_data():
-    paths = [os.path.join('data', '*.txt')]  # etc, add other extensions
+    paths = [os.path.join('data', '*.txt'),
+             os.path.join('data', 'README')]  # etc, add other extensions
 
     return {'astroquery.jplhorizons.tests': paths}
