@@ -1,28 +1,27 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import print_function
 
+import json
 import os
 import re
-
 from shutil import copyfile
 
+import pytest
+
 from astropy.table import Table
-from astropy.tests.helper import pytest
 from astropy.coordinates import SkyCoord
 from astropy.io import fits
-from astropy.tests.helper import catch_warnings
-from astropy.utils.exceptions import AstropyDeprecationWarning
 
 import astropy.units as u
 
-from ...utils.testing_tools import MockResponse
-from ...exceptions import (InvalidQueryError, InputWarning)
-
-from ... import mast
+from astroquery.mast.services import _json_to_table
+from astroquery.utils.mocks import MockResponse
+from astroquery.exceptions import InvalidQueryError, InputWarning
 
+from astroquery import mast
 
 DATA_FILES = {'Mast.Caom.Cone': 'caom.json',
               'Mast.Name.Lookup': 'resolver.json',
+              'mission_search_results': 'mission_results.json',
               'columnsconfig': 'columnsconfig.json',
               'ticcolumns': 'ticcolumns.json',
               'ticcol_filtered': 'ticcolumns_filtered.json',
@@ -48,7 +47,10 @@ DATA_FILES = {'Mast.Caom.Cone': 'caom.json',
               'Mast.HscSpectra.Db.All': 'spectra.json',
               'panstarrs': 'panstarrs.json',
               'tess_cutout': 'astrocut_107.27_-70.0_5x5.zip',
-              'tess_sector': 'tess_sector.json'}
+              'tess_sector': 'tess_sector.json',
+              'z_cutout_fit': 'astrocut_189.49206_62.20615_100x100px_f.zip',
+              'z_cutout_jpg': 'astrocut_189.49206_62.20615_100x100px.zip',
+              'z_survey': 'zcut_survey.json'}
 
 
 def data_path(filename):
@@ -58,26 +60,23 @@ def data_path(filename):
 
 @pytest.fixture
 def patch_post(request):
-    try:
-        mp = request.getfixturevalue("monkeypatch")
-    except AttributeError:  # pytest < 3
-        mp = request.getfuncargvalue("monkeypatch")
-    mp.setattr(mast.Mast, '_request', post_mockreturn)
-    mp.setattr(mast.Mast, '_fabric_request', post_mockreturn)
-    mp.setattr(mast.Observations, '_request', post_mockreturn)
-    mp.setattr(mast.Catalogs, '_request', post_mockreturn)
-    mp.setattr(mast.Catalogs, '_fabric_request', post_mockreturn)
-    mp.setattr(mast.Mast, '_download_file', download_mockreturn)
+    mp = request.getfixturevalue("monkeypatch")
+
+    mp.setattr(mast.utils, '_simple_request', resolver_mockreturn)
+    mp.setattr(mast.discovery_portal.PortalAPI, '_request', post_mockreturn)
+    mp.setattr(mast.services.ServiceAPI, '_request', service_mockreturn)
+    mp.setattr(mast.auth.MastAuth, 'session_info', session_info_mockreturn)
+
     mp.setattr(mast.Observations, '_download_file', download_mockreturn)
+    mp.setattr(mast.Observations, 'download_file', download_mockreturn)
     mp.setattr(mast.Catalogs, '_download_file', download_mockreturn)
-    mp.setattr(mast.Mast, 'session_info', session_info_mockreturn)
-    mp.setattr(mast.Observations, 'session_info', session_info_mockreturn)
-    mp.setattr(mast.Tesscut, "_request", tesscut_get_mockreturn)
-    mp.setattr(mast.Tesscut, '_download_file', tess_download_mockreturn)
+    mp.setattr(mast.Tesscut, '_download_file', tesscut_download_mockreturn)
+    mp.setattr(mast.Zcut, '_download_file', zcut_download_mockreturn)
+
     return mp
 
 
-def post_mockreturn(method="POST", url=None, data=None, timeout=10, **kwargs):
+def post_mockreturn(self, method="POST", url=None, data=None, timeout=10, **kwargs):
     if "columnsconfig" in url:
         if "Mast.Catalogs.Tess.Cone" in data:
             service = "ticcolumns"
@@ -100,25 +99,61 @@ def post_mockreturn(method="POST", url=None, data=None, timeout=10, **kwargs):
     if ("Filtered" in service) and (re.search(r"COUNT_BIG%28%2A%29", data)):
         service = "Counts"
     filename = data_path(DATA_FILES[service])
-    content = open(filename, 'rb').read()
+    with open(filename, 'rb') as infile:
+        content = infile.read()
 
     # returning as list because this is what the mast _request function does
     return [MockResponse(content)]
 
 
+def service_mockreturn(self, method="POST", url=None, data=None, timeout=10, use_json=False, **kwargs):
+    if "panstarrs" in url:
+        filename = data_path(DATA_FILES["panstarrs"])
+    elif "tesscut" in url:
+        if "sector" in url:
+            filename = data_path(DATA_FILES['tess_sector'])
+        else:
+            filename = data_path(DATA_FILES['tess_cutout'])
+    elif "zcut" in url:
+        if "survey" in url:
+            filename = data_path(DATA_FILES['z_survey'])
+        else:
+            filename = data_path(DATA_FILES['z_cutout_fit'])
+    elif use_json and data['radius'] == 5:
+        filename = data_path(DATA_FILES["mission_incorrect_results"])
+    elif use_json:
+        filename = data_path(DATA_FILES["mission_search_results"])
+    with open(filename, 'rb') as infile:
+        content = infile.read()
+    return MockResponse(content)
+
+
+def resolver_mockreturn(*args, **kwargs):
+    filename = data_path(DATA_FILES["Mast.Name.Lookup"])
+    with open(filename, 'rb') as infile:
+        content = infile.read()
+    return MockResponse(content)
+
+
 def download_mockreturn(*args, **kwargs):
-    return
+    return ('COMPLETE', None, None)
 
 
-def session_info_mockreturn(silent=False):
-    anon_session = {'eppn': '',
-                   'ezid': 'anonymous',
-                   'attrib': {},
-                   'anon': True,
-                   'scopes': [],
-                   'session': None,
-                   'token': None}
-    return anon_session
+def session_info_mockreturn(self, silent=False):
+    test_session = {'eppn': 'alice@stsci.edu',
+                    'ezid': 'alice',
+                    'attrib': {'uuid': '2913e6f7-e863-4f94-9416-a6af27258ba7',
+                               'first_name': 'A.',
+                               'last_name': 'User',
+                               'display_name': 'A. User',
+                               'internal': '0',
+                               'email': 'alice@gmail.com',
+                               'Jwstcalengdataaccess': 'false'},
+                    'anon': False,
+                    'scopes': ['mast:user:info', 'mast:exclusive_access'],
+                    'session': None,
+                    'token': '56a9cf3d...'}
+    return test_session
 
 
 def tesscut_get_mockreturn(method="GET", url=None, data=None, timeout=10, **kwargs):
@@ -127,15 +162,109 @@ def tesscut_get_mockreturn(method="GET", url=None, data=None, timeout=10, **kwar
     else:
         filename = data_path(DATA_FILES['tess_cutout'])
 
-    content = open(filename, 'rb').read()
+    with open(filename, 'rb') as infile:
+        content = infile.read()
     return MockResponse(content)
 
 
-def tess_download_mockreturn(url, file_path):
+def tesscut_download_mockreturn(url, file_path):
     filename = data_path(DATA_FILES['tess_cutout'])
     copyfile(filename, file_path)
     return
 
+
+def zcut_download_mockreturn(url, file_path):
+    if "jpg" in url:
+        filename = data_path(DATA_FILES['z_cutout_jpg'])
+    else:
+        filename = data_path(DATA_FILES['z_cutout_fit'])
+    copyfile(filename, file_path)
+    return
+
+
+###########################
+# MissionSearchClass Test #
+###########################
+
+
+def test_missions_query_region_async(patch_post):
+    responses = mast.MastMissions.query_region_async(regionCoords, radius=0.002, sci_pi_last_name='GORDON')
+    assert isinstance(responses, MockResponse)
+
+
+def test_missions_query_object_async(patch_post):
+    responses = mast.MastMissions.query_object_async("M101", radius="0.002 deg")
+    assert isinstance(responses, MockResponse)
+
+
+def test_missions_query_object(patch_post):
+    result = mast.MastMissions.query_object("M101", radius=".002 deg")
+    assert isinstance(result, Table)
+    assert len(result) > 0
+
+
+def test_missions_query_region(patch_post):
+    result = mast.MastMissions.query_region(regionCoords, radius=0.002 * u.deg)
+    assert isinstance(result, Table)
+    assert len(result) > 0
+
+
+def test_missions_query_criteria_async(patch_post):
+    pep_id = {'sci_pep_id': '12556'}
+    obs_type = {'sci_obs_type': "SPECTRUM"}
+    instruments = {'sci_instrume': "stis,acs,wfc3,cos,fos,foc,nicmos,ghrs"}
+    datasets = {'sci_data_set_name': ""}
+    pi_lname = {'sci_pi_last_name': ""}
+    spec_1234 = {'sci_spec_1234': ""}
+    release_date = {'sci_release_date': ""}
+    start_time = {'sci_start_time': ""}
+    obs_type = {'sci_obs_type': 'all'}
+    aec = {'sci_aec': 'S'}
+    responses = mast.MastMissions.query_criteria_async(coordinates=regionCoords,
+                                                       radius=3,
+                                                       conditions=[pep_id,
+                                                                   obs_type,
+                                                                   instruments,
+                                                                   datasets,
+                                                                   pi_lname,
+                                                                   spec_1234,
+                                                                   release_date,
+                                                                   start_time,
+                                                                   obs_type,
+                                                                   aec])
+    assert isinstance(responses, MockResponse)
+
+
+def test_missions_query_criteria_async_with_missing_results(patch_post):
+    pep_id = {'sci_pep_id': '12556'}
+    obs_type = {'sci_obs_type': "SPECTRUM"}
+    instruments = {'sci_instrume': "stis,acs,wfc3,cos,fos,foc,nicmos,ghrs"}
+    datasets = {'sci_data_set_name': ""}
+    pi_lname = {'sci_pi_last_name': ""}
+    spec_1234 = {'sci_spec_1234': ""}
+    release_date = {'sci_release_date': ""}
+    start_time = {'sci_start_time': ""}
+    obs_type = {'sci_obs_type': 'all'}
+    aec = {'sci_aec': 'S'}
+    aperture = {'sci_aper_1234': 'WF3'}
+
+    with pytest.raises(KeyError):
+        responses = mast.MastMissions.query_criteria_async(coordinates=regionCoords,
+                                                           radius=5,
+                                                           conditions=[pep_id,
+                                                                       obs_type,
+                                                                       instruments,
+                                                                       datasets,
+                                                                       pi_lname,
+                                                                       spec_1234,
+                                                                       release_date,
+                                                                       start_time,
+                                                                       obs_type,
+                                                                       aec,
+                                                                       aperture])
+        _json_to_table(json.loads(responses), 'results')
+
+
 ###################
 # MastClass tests #
 ###################
@@ -170,9 +299,51 @@ def test_mast_service_request(patch_post):
     assert isinstance(result, Table)
 
 
+def test_mast_query(patch_post):
+    # cone search
+    result = mast.Mast.mast_query('Mast.Caom.Cone', ra=23.34086, dec=60.658, radius=0.2)
+    assert isinstance(result, Table)
+
+    # filtered search
+    result = mast.Mast.mast_query('Mast.Caom.Filtered',
+                                  dataproduct_type=['image'],
+                                  proposal_pi=['Osten, Rachel A.'],
+                                  s_dec=[{'min': 43.5, 'max': 45.5}])
+    pp_list = result['proposal_pi']
+    sd_list = result['s_dec']
+    assert isinstance(result, Table)
+    assert len(set(pp_list)) == 1
+    assert max(sd_list) < 45.5
+    assert min(sd_list) > 43.5
+
+    # error handling
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Mast.mast_query('Mast.Caom.Filtered')
+    assert "Please provide at least one filter." in str(invalid_query.value)
+
+
 def test_resolve_object(patch_post):
     m103_loc = mast.Mast.resolve_object("M103")
-    assert m103_loc.separation(SkyCoord("23.34086 60.658", unit='deg')).value == 0
+    print(m103_loc)
+    assert round(m103_loc.separation(SkyCoord("23.34086 60.658", unit='deg')).value, 10) == 0
+
+
+def test_login_logout(patch_post):
+    test_token = "56a9cf3df4c04052atest43feb87f282"
+
+    mast.Mast.login(token=test_token)
+    assert mast.Mast._authenticated is True
+    assert mast.Mast._session.cookies.get("mast_token") == test_token
+
+    mast.Mast.logout()
+    assert mast.Mast._authenticated is False
+    assert not mast.Mast._session.cookies.get("mast_token")
+
+
+def test_session_info(patch_post):
+    info = mast.Mast.session_info(verbose=False)
+    assert isinstance(info, dict)
+    assert info['ezid'] == 'alice'
 
 
 ###########################
@@ -182,9 +353,8 @@ def test_resolve_object(patch_post):
 
 regionCoords = SkyCoord(23.34086, 60.658, unit=('deg', 'deg'))
 
+
 # query functions
-
-
 def test_observations_query_region_async(patch_post):
     responses = mast.Observations.query_region_async(regionCoords, radius=0.2)
     assert isinstance(responses, list)
@@ -230,16 +400,6 @@ def test_observations_query_criteria(patch_post):
                                               objectname="M101")
     assert isinstance(result, Table)
 
-    # TEMPORARY test the obstype deprecation
-    with catch_warnings(AstropyDeprecationWarning) as warning_lines:
-        result = mast.Observations.query_criteria(objectname="M101",
-                                                  dataproduct_type="IMAGE", obstype="science")
-        assert isinstance(result, Table)
-
-    with pytest.warns(InputWarning) as i_w:
-        mast.Observations.query_criteria(obstype="science", intentType="science")
-    assert "obstype and intentType" in str(i_w[0].message)
-
     with pytest.raises(InvalidQueryError) as invalid_query:
         mast.Observations.query_criteria(objectname="M101")
     assert "least one non-positional criterion" in str(invalid_query.value)
@@ -306,6 +466,10 @@ def test_observations_get_product_list(patch_post):
     result = mast.Observations.get_product_list(observations[0:4])
     assert isinstance(result, Table)
 
+    in_obsids = ['83229830', '1829332', '26074149', '24556715']
+    result = mast.Observations.get_product_list(in_obsids)
+    assert isinstance(result, Table)
+
 
 def test_observations_filter_products(patch_post):
     products = mast.Observations.get_product_list('2003738726')
@@ -332,6 +496,21 @@ def test_observations_download_products(patch_post, tmpdir):
                                                  mrp_only=False)
     assert isinstance(result, Table)
 
+    # passing row product
+    products = mast.Observations.get_product_list('2003738726')
+    result1 = mast.Observations.download_products(products[0])
+    assert isinstance(result1, Table)
+
+
+def test_observations_download_file(patch_post, tmpdir):
+    # pull a single data product
+    products = mast.Observations.get_product_list('2003738726')
+    uri = products['dataURI'][0]
+
+    # download it
+    result = mast.Observations.download_file(uri)
+    assert result == ('COMPLETE', None, None)
+
 
 ######################
 # CatalogClass tests #
@@ -345,7 +524,7 @@ def test_catalogs_query_region_async(patch_post):
 
 def test_catalogs_fabric_query_region_async(patch_post):
     responses = mast.Catalogs.query_region_async(regionCoords, radius=0.002, catalog="panstarrs", table="mean")
-    assert isinstance(responses, list)
+    assert isinstance(responses, MockResponse)
 
 
 def test_catalogs_query_region(patch_post):
@@ -388,7 +567,7 @@ def test_catalogs_query_object_async(patch_post):
 
 def test_catalogs_fabric_query_object_async(patch_post):
     responses = mast.Catalogs.query_object_async("M101", radius="0.002 deg", catalog="panstarrs", table="mean")
-    assert isinstance(responses, list)
+    assert isinstance(responses, MockResponse)
 
 
 def test_catalogs_query_object(patch_post):
@@ -421,7 +600,7 @@ def test_catalogs_query_criteria_async(patch_post):
 
     responses = mast.Catalogs.query_criteria_async(catalog="panstarrs", objectname="M10", radius=2,
                                                    table="mean", qualityFlag=48)
-    assert isinstance(responses, list)
+    assert isinstance(responses, MockResponse)
 
     with pytest.raises(InvalidQueryError) as invalid_query:
         mast.Catalogs.query_criteria_async(catalog="Tic")
@@ -456,7 +635,7 @@ def test_catalogs_query_criteria(patch_post):
     assert isinstance(result, Table)
 
     with pytest.raises(InvalidQueryError) as invalid_query:
-        mast.Catalogs.query_criteria(catalog="Tic", objectName="M10")
+        mast.Catalogs.query_criteria(catalog="Tic", objectname="M10")
     assert "non-positional" in str(invalid_query.value)
 
 
@@ -531,6 +710,33 @@ def test_tesscut_get_sector(patch_post):
     assert sector_table['camera'][0] == 1
     assert sector_table['ccd'][0] == 3
 
+    # Exercising the search by moving target
+    sector_table = mast.Tesscut.get_sectors(objectname="Ceres",
+                                            moving_target=True)
+    assert isinstance(sector_table, Table)
+    assert len(sector_table) == 1
+    assert sector_table['sectorName'][0] == "tess-s0001-1-3"
+    assert sector_table['sector'][0] == 1
+    assert sector_table['camera'][0] == 1
+    assert sector_table['ccd'][0] == 3
+
+    # Testing catch for multiple designators'
+    error_str = ("Only one of moving_target and coordinates may be specified. "
+                 "Please remove coordinates if using moving_target and objectname.")
+
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.get_sectors(objectname='Ceres', moving_target=True, coordinates=coord)
+    assert error_str in str(invalid_query.value)
+
+    # Testing invalid queries
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.get_sectors(objectname="M101", product="spooc")
+    assert "Input product must either be SPOC or TICA." in str(invalid_query.value)
+
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.get_sectors(objectname="M101", product="TICA", moving_target=True)
+    assert "Only SPOC is available for moving targets queries." in str(invalid_query.value)
+
 
 def test_tesscut_download_cutouts(patch_post, tmpdir):
 
@@ -558,6 +764,37 @@ def test_tesscut_download_cutouts(patch_post, tmpdir):
     assert manifest["Local Path"][0][-4:] == "fits"
     assert os.path.isfile(manifest[0]['Local Path'])
 
+    # Exercising the search by moving target
+    manifest = mast.Tesscut.download_cutouts(objectname="Eleonora",
+                                             moving_target=True,
+                                             size=5,
+                                             path=str(tmpdir))
+    assert isinstance(manifest, Table)
+    assert len(manifest) == 1
+    assert manifest["Local Path"][0][-4:] == "fits"
+    assert os.path.isfile(manifest[0]['Local Path'])
+
+    # Testing catch for multiple designators'
+    error_str = ("Only one of moving_target and coordinates may be specified. "
+                 "Please remove coordinates if using moving_target and objectname.")
+
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.download_cutouts(objectname="Eleonora",
+                                      moving_target=True,
+                                      coordinates=coord,
+                                      size=5,
+                                      path=str(tmpdir))
+    assert error_str in str(invalid_query.value)
+
+    # Testing invalid queries
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.download_cutouts(objectname="M101", product="spooc")
+    assert "Input product must either be SPOC or TICA." in str(invalid_query.value)
+
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.download_cutouts(objectname="M101", product="TICA", moving_target=True)
+    assert "Only SPOC is available for moving targets queries." in str(invalid_query.value)
+
 
 def test_tesscut_get_cutouts(patch_post, tmpdir):
 
@@ -572,3 +809,91 @@ def test_tesscut_get_cutouts(patch_post, tmpdir):
     assert isinstance(cutout_hdus_list, list)
     assert len(cutout_hdus_list) == 1
     assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+    # Exercising the search by object name
+    cutout_hdus_list = mast.Tesscut.get_cutouts(objectname='Eleonora',
+                                                moving_target=True,
+                                                size=5)
+    assert isinstance(cutout_hdus_list, list)
+    assert len(cutout_hdus_list) == 1
+    assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+    # Testing catch for multiple designators'
+    error_str = ("Only one of moving_target and coordinates may be specified. "
+                 "Please remove coordinates if using moving_target and objectname.")
+
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.get_cutouts(objectname="Eleonora",
+                                 moving_target=True,
+                                 coordinates=coord,
+                                 size=5)
+    assert error_str in str(invalid_query.value)
+
+    # Testing invalid queries
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.get_cutouts(objectname="M101", product="spooc")
+    assert "Input product must either be SPOC or TICA." in str(invalid_query.value)
+
+    with pytest.raises(InvalidQueryError) as invalid_query:
+        mast.Tesscut.get_cutouts(objectname="M101", product="TICA", moving_target=True)
+    assert "Only SPOC is available for moving targets queries." in str(invalid_query.value)
+
+
+######################
+# ZcutClass tests #
+######################
+
+
+def test_zcut_get_survey(patch_post):
+
+    coord = SkyCoord(189.49206, 62.20615, unit="deg")
+    survey_list = mast.Zcut.get_surveys(coordinates=coord)
+    assert isinstance(survey_list, list)
+    assert len(survey_list) == 3
+    assert survey_list[0] == 'candels_gn_60mas'
+    assert survey_list[1] == 'candels_gn_30mas'
+    assert survey_list[2] == 'goods_north'
+
+    survey_list = mast.Zcut.get_surveys(coordinates=coord, radius=0.2)
+    assert isinstance(survey_list, list)
+    assert len(survey_list) == 3
+    assert survey_list[0] == 'candels_gn_60mas'
+    assert survey_list[1] == 'candels_gn_30mas'
+    assert survey_list[2] == 'goods_north'
+
+
+def test_zcut_download_cutouts(patch_post, tmpdir):
+
+    coord = SkyCoord(189.49206, 62.20615, unit="deg")
+
+    # Testing with fits
+    cutout_table = mast.Zcut.download_cutouts(coordinates=coord, size=5, path=str(tmpdir))
+    assert isinstance(cutout_table, Table)
+    assert len(cutout_table) == 1
+    assert cutout_table["Local Path"][0][-4:] == "fits"
+    assert os.path.isfile(cutout_table[0]['Local Path'])
+
+    # Testing with png
+    cutout_table = mast.Zcut.download_cutouts(coordinates=coord, size=5,
+                                              cutout_format="jpg", path=str(tmpdir))
+    assert isinstance(cutout_table, Table)
+    assert len(cutout_table) == 3
+    assert cutout_table["Local Path"][0][-4:] == ".jpg"
+    assert os.path.isfile(cutout_table[0]['Local Path'])
+
+    # Testing with img_param
+    cutout_table = mast.Zcut.download_cutouts(coordinates=coord, size=5,
+                                              cutout_format="jpg", path=str(tmpdir), invert=True)
+    assert isinstance(cutout_table, Table)
+    assert len(cutout_table) == 3
+    assert cutout_table["Local Path"][0][-4:] == ".jpg"
+    assert os.path.isfile(cutout_table[0]['Local Path'])
+
+
+def test_zcut_get_cutouts(patch_post, tmpdir):
+
+    coord = SkyCoord(189.49206, 62.20615, unit="deg")
+    cutout_list = mast.Zcut.get_cutouts(coordinates=coord, size=5)
+    assert isinstance(cutout_list, list)
+    assert len(cutout_list) == 1
+    assert isinstance(cutout_list[0], fits.HDUList)
