@@ -1,5 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+from pathlib import Path
 import numpy as np
 import os
 import pytest
@@ -14,12 +15,26 @@ import astropy.units as u
 from astroquery import mast
 
 from ..utils import ResolverError
-from ...exceptions import InvalidQueryError, MaxResultsWarning, NoResultsWarning, RemoteServiceError
+from ...exceptions import (InputWarning, InvalidQueryError, MaxResultsWarning,
+                           NoResultsWarning)
 
 
 OBSID = '1647157'
 
 
+@pytest.fixture(scope="module")
+def msa_product_table():
+    # Pull products for a JWST NIRSpec MSA observation with 6 known
+    # duplicates of the MSA configuration file, propID=2736
+    products = mast.Observations.get_product_list("87602009")
+
+    # Filter out everything but the MSA config file
+    mask = np.char.find(products["dataURI"], "_msa.fits") != -1
+    products = products[mask]
+
+    return products
+
+
 @pytest.mark.remote_data
 class TestMast:
 
@@ -68,6 +83,21 @@ class TestMast:
         # Are the two GALEX observations with obs_id 6374399093149532160 in the results table
         assert len(result[np.where(result["obs_id"] == "6374399093149532160")]) == 2
 
+    def test_mast_query(self):
+        # clear columns config
+        mast.Mast._column_configs = dict()
+
+        result = mast.Mast.mast_query('Mast.Caom.Cone', ra=184.3, dec=54.5, radius=0.2)
+
+        # Is result in the right format
+        assert isinstance(result, Table)
+
+        # Are the GALEX observations in the results table
+        assert "GALEX" in result['obs_collection']
+
+        # Are the two GALEX observations with obs_id 6374399093149532160 in the results table
+        assert len(result[np.where(result["obs_id"] == "6374399093149532160")]) == 2
+
     def test_mast_session_info(self):
         sessionInfo = mast.Mast.session_info(verbose=False)
         assert sessionInfo['ezid'] == 'anonymous'
@@ -274,7 +304,7 @@ class TestMast:
                 assert os.path.isfile(row['Local Path'])
 
         # just get the curl script
-        result = mast.Observations.download_products(test_obs[0]["obsid"],
+        result = mast.Observations.download_products(test_obs_id[0]["obsid"],
                                                      download_dir=str(tmpdir),
                                                      curl_flag=True,
                                                      productType=["SCIENCE"],
@@ -283,17 +313,75 @@ class TestMast:
         assert os.path.isfile(result['Local Path'][0])
 
         # check for row input
-        result1 = mast.Observations.get_product_list(test_obs[0]["obsid"])
+        result1 = mast.Observations.get_product_list(test_obs_id[0]["obsid"])
         result2 = mast.Observations.download_products(result1[0])
         assert isinstance(result2, Table)
         assert os.path.isfile(result2['Local Path'][0])
         assert len(result2) == 1
 
-    def test_observations_download_file(self, tmpdir):
+    def test_observations_download_products_flat(self, tmp_path, msa_product_table):
 
-        # enabling cloud connection
+        # Get a product list with 6 duplicate JWST MSA config files
+        products = msa_product_table
+
+        assert len(products) == 6
+
+        # Download with flat=True
+        manifest = mast.Observations.download_products(products, flat=True,
+                                                       download_dir=tmp_path)
+
+        assert Path(manifest["Local Path"][0]).parent == tmp_path
+
+    def test_observations_download_products_flat_curl(self, tmp_path, msa_product_table):
+
+        # Get a product list with 6 duplicate JWST MSA config files
+        products = msa_product_table
+
+        assert len(products) == 6
+
+        # Download with flat=True, curl_flag=True, look for warning
+        with pytest.warns(InputWarning):
+            mast.Observations.download_products(products, flat=True,
+                                                curl_flag=True,
+                                                download_dir=tmp_path)
+
+    def test_observations_download_products_no_duplicates(self, tmp_path, caplog, msa_product_table):
+
+        # Get a product list with 6 duplicate JWST MSA config files
+        products = msa_product_table
+
+        assert len(products) == 6
+
+        # Download the product
+        manifest = mast.Observations.download_products(products,
+                                                       download_dir=tmp_path)
+
+        # Check that it downloads the MSA config file only once
+        assert len(manifest) == 1
+
+        # Check that an INFO message about duplicates was logged
+        with caplog.at_level("INFO", logger="astroquery"):
+            assert "products were duplicates" in caplog.text
+
+    def test_observations_get_cloud_uris_no_duplicates(self, msa_product_table):
+
+        # Get a product list with 6 duplicate JWST MSA config files
+        products = msa_product_table
+
+        assert len(products) == 6
+
+        # enable access to public AWS S3 bucket
         mast.Observations.enable_cloud_dataset(provider='AWS')
 
+        # Check for cloud URIs.  Accept a NoResultsWarning if AWS S3
+        # doesn't have the file.  It doesn't matter as we're only checking
+        # that the duplicate products have been culled to a single one.
+        with pytest.warns(NoResultsWarning):
+            uris = mast.Observations.get_cloud_uris(products)
+        assert len(uris) == 1
+
+    def test_observations_download_file(self, tmp_path):
+
         # get observations from GALEX instrument with query_criteria
         observations = mast.Observations.query_criteria(objectname='M1',
                                                         radius=0.2,
@@ -306,34 +394,42 @@ class TestMast:
 
         # pull the URI of a single product
         uri = products['dataURI'][0]
+        local_path = Path(tmp_path, Path(uri).name)
 
         # download it
-        result = mast.Observations.download_file(uri, cloud_only=True)
+        result = mast.Observations.download_file(uri, local_path=local_path)
         assert result == ('COMPLETE', None, None)
 
-    def test_get_cloud_uri(self):
-        test_obs_id = '25568122'
-
+    @pytest.mark.parametrize("test_data_uri, expected_cloud_uri", [
+        ("mast:HST/product/u24r0102t_c1f.fits",
+         "s3://stpubdata/hst/public/u24r/u24r0102t/u24r0102t_c1f.fits"),
+        ("mast:PS1/product/rings.v3.skycell.1334.061.stk.r.unconv.exp.fits",
+         "s3://stpubdata/panstarrs/ps1/public/rings.v3.skycell/1334/061/"
+         "rings.v3.skycell.1334.061.stk.r.unconv.exp.fits")
+    ])
+    def test_get_cloud_uri(self, test_data_uri, expected_cloud_uri):
+        pytest.importorskip("boto3")
         # get a product list
-        product = mast.Observations.get_product_list(test_obs_id)[24]
-
-        assert len(product) > 0, f'No product found for OBSID {test_obs_id}. Unable to move forward with getting URIs from the cloud.'
-
+        product = Table()
+        product['dataURI'] = [test_data_uri]
         # enable access to public AWS S3 bucket
         mast.Observations.enable_cloud_dataset()
 
         # get uri
-        uri = mast.Observations.get_cloud_uri(product)
+        uri = mast.Observations.get_cloud_uri(product[0])
 
-        assert len(uri) > 0, f'Product for OBSID {test_obs_id} was not found in the cloud.'
+        assert len(uri) > 0, f'Product for dataURI {test_data_uri} was not found in the cloud.'
+        assert uri == expected_cloud_uri, f'Cloud URI does not match expected. ({uri} != {expected_cloud_uri})'
 
     def test_get_cloud_uris(self):
+        pytest.importorskip("boto3")
         test_obs_id = '25568122'
 
         # get a product list
         products = mast.Observations.get_product_list(test_obs_id)[24:]
 
-        assert len(products) > 0, f'No products found for OBSID {test_obs_id}. Unable to move forward with getting URIs from the cloud.'
+        assert len(products) > 0, (f'No products found for OBSID {test_obs_id}. '
+                                   'Unable to move forward with getting URIs from the cloud.')
 
         # enable access to public AWS S3 bucket
         mast.Observations.enable_cloud_dataset()
@@ -615,6 +711,17 @@ class TestMast:
         assert isinstance(result, Table)
         assert 'PSO J254.2861-04.1091' in result['objName']
 
+        result = mast.Catalogs.query_criteria(coordinates="158.47924 -7.30962",
+                                              radius=0.01,
+                                              catalog="PANSTARRS",
+                                              table="mean",
+                                              data_release="dr2",
+                                              nStackDetections=[("gte", "1")],
+                                              columns=["objName", "distance"],
+                                              sort_by=[("asc", "distance")])
+        assert isinstance(result, Table)
+        assert result['distance'][0] <= result['distance'][1]
+
     def test_catalogs_query_hsc_matchid_async(self):
         catalogData = mast.Catalogs.query_object("M10",
                                                  radius=.001,
@@ -683,42 +790,37 @@ class TestMast:
     # TesscutClass tests #
     ######################
 
-    def test_tesscut_get_sectors(self):
+    @pytest.mark.parametrize("product", ["tica", "spoc"])
+    def test_tesscut_get_sectors(self, product):
 
         coord = SkyCoord(349.62609, -47.12424, unit="deg")
-        sector_table = mast.Tesscut.get_sectors(coordinates=coord)
+        sector_table = mast.Tesscut.get_sectors(coordinates=coord, product=product)
         assert isinstance(sector_table, Table)
         assert len(sector_table) >= 1
-        assert "tess-s00" in sector_table['sectorName'][0]
+
+        name = "tess" if product == "spoc" else product
+        assert f"{name}-s00" in sector_table['sectorName'][0]
+
         assert sector_table['sector'][0] > 0
         assert sector_table['camera'][0] > 0
         assert sector_table['ccd'][0] > 0
 
-        # This should always return no results
-
-        with pytest.warns(NoResultsWarning):
-            coord = SkyCoord(90, -66.5, unit="deg")
-            sector_table = mast.Tesscut.get_sectors(coordinates=coord,
-                                                    radius=0)
-            assert isinstance(sector_table, Table)
-            assert len(sector_table) == 0
-
-        coord = SkyCoord(90, -66.5, unit="deg")
-        with pytest.warns(NoResultsWarning):
-            sector_table = mast.Tesscut.get_sectors(coordinates=coord, radius=0)
-        assert isinstance(sector_table, Table)
-        assert len(sector_table) == 0
-
-        sector_table = mast.Tesscut.get_sectors(objectname="M104")
+        sector_table = mast.Tesscut.get_sectors(objectname="M104", product=product)
         assert isinstance(sector_table, Table)
         assert len(sector_table) >= 1
-        assert "tess-s00" in sector_table['sectorName'][0]
+
+        name = "tess" if product == "spoc" else product
+        assert f"{name}-s00" in sector_table['sectorName'][0]
+
         assert sector_table['sector'][0] > 0
         assert sector_table['camera'][0] > 0
         assert sector_table['ccd'][0] > 0
 
+    def test_tesscut_get_sectors_mt(self):
+
         # Moving target functionality testing
 
+        coord = SkyCoord(349.62609, -47.12424, unit="deg")
         moving_target_name = 'Eleonora'
 
         sector_table = mast.Tesscut.get_sectors(objectname=moving_target_name,
@@ -730,10 +832,13 @@ class TestMast:
         assert sector_table['camera'][0] == 1
         assert sector_table['ccd'][0] == 1
 
-        error_noname = "Please specify the object name or ID (as understood by the `JPL ephemerides service <https://ssd.jpl.nasa.gov/horizons.cgi>`__) of a moving target such as an asteroid or comet."
+        error_noname = ("Please specify the object name or ID (as understood by the "
+                        "`JPL ephemerides service <https://ssd.jpl.nasa.gov/horizons.cgi>`__) "
+                        "of a moving target such as an asteroid or comet.")
         error_nameresolve = f"Could not resolve {moving_target_name} to a sky position."
         error_mt_coord = "Only one of moving_target and coordinates may be specified."
         error_name_coord = "Only one of objectname and coordinates may be specified."
+        error_tica_mt = "Only SPOC is available for moving targets queries."
 
         with pytest.raises(InvalidQueryError) as error_msg:
             mast.Tesscut.get_sectors(moving_target=True)
@@ -758,11 +863,19 @@ class TestMast:
                                      moving_target=True)
         assert error_mt_coord in str(error_msg.value)
 
-    def test_tesscut_download_cutouts(self, tmpdir):
+        # The TICA product option is not available for moving targets
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            sector_table = mast.Tesscut.get_sectors(objectname=moving_target_name, product='tica',
+                                                    moving_target=True)
+            assert error_tica_mt in str(error_msg.value)
+
+    @pytest.mark.parametrize("product", ["tica", "spoc"])
+    def test_tesscut_download_cutouts(self, tmpdir, product):
 
         coord = SkyCoord(349.62609, -47.12424, unit="deg")
 
-        manifest = mast.Tesscut.download_cutouts(coordinates=coord, size=5, path=str(tmpdir))
+        manifest = mast.Tesscut.download_cutouts(product=product, coordinates=coord, size=5, path=str(tmpdir))
         assert isinstance(manifest, Table)
         assert len(manifest) >= 1
         assert manifest["Local Path"][0][-4:] == "fits"
@@ -771,34 +884,39 @@ class TestMast:
 
         coord = SkyCoord(107.18696, -70.50919, unit="deg")
 
-        manifest = mast.Tesscut.download_cutouts(coordinates=coord, size=5, sector=1, path=str(tmpdir))
+        manifest = mast.Tesscut.download_cutouts(product=product, coordinates=coord, size=5, sector=27,
+                                                 path=str(tmpdir))
         assert isinstance(manifest, Table)
         assert len(manifest) == 1
         assert manifest["Local Path"][0][-4:] == "fits"
         assert os.path.isfile(manifest[0]['Local Path'])
 
-        manifest = mast.Tesscut.download_cutouts(coordinates=coord, size=[5, 7]*u.pix, sector=8, path=str(tmpdir))
+        manifest = mast.Tesscut.download_cutouts(product=product, coordinates=coord, size=[5, 7]*u.pix, sector=33,
+                                                 path=str(tmpdir))
         assert isinstance(manifest, Table)
         assert len(manifest) >= 1
         assert manifest["Local Path"][0][-4:] == "fits"
         for row in manifest:
             assert os.path.isfile(row['Local Path'])
 
-        manifest = mast.Tesscut.download_cutouts(coordinates=coord, size=5, sector=8, path=str(tmpdir), inflate=False)
+        manifest = mast.Tesscut.download_cutouts(product=product, coordinates=coord, size=5, sector=33,
+                                                 path=str(tmpdir), inflate=False)
         assert isinstance(manifest, Table)
         assert len(manifest) == 1
         assert manifest["Local Path"][0][-3:] == "zip"
         assert os.path.isfile(manifest[0]['Local Path'])
 
-        manifest = mast.Tesscut.download_cutouts(objectname="TIC 32449963", size=5, path=str(tmpdir))
+        manifest = mast.Tesscut.download_cutouts(product=product, objectname="TIC 32449963", size=5, path=str(tmpdir))
         assert isinstance(manifest, Table)
         assert len(manifest) >= 1
         assert manifest["Local Path"][0][-4:] == "fits"
         for row in manifest:
             assert os.path.isfile(row['Local Path'])
 
-        # Moving target functionality testing
+    def test_tesscut_download_cutouts_mt(self, tmpdir):
 
+        # Moving target functionality testing
+        coord = SkyCoord(349.62609, -47.12424, unit="deg")
         moving_target_name = 'Eleonora'
 
         manifest = mast.Tesscut.download_cutouts(objectname=moving_target_name,
@@ -812,76 +930,13 @@ class TestMast:
         for row in manifest:
             assert os.path.isfile(row['Local Path'])
 
-        error_noname = "Please specify the object name or ID (as understood by the `JPL ephemerides service <https://ssd.jpl.nasa.gov/horizons.cgi>`__) of a moving target such as an asteroid or comet."
-        error_nameresolve = f"Could not resolve {moving_target_name} to a sky position."
-        error_mt_coord = "Only one of moving_target and coordinates may be specified."
-        error_name_coord = "Only one of objectname and coordinates may be specified."
-
-        with pytest.raises(InvalidQueryError) as error_msg:
-            mast.Tesscut.download_cutouts(moving_target=True)
-        assert error_noname in str(error_msg.value)
-
-        with pytest.raises(ResolverError) as error_msg:
-            mast.Tesscut.download_cutouts(objectname=moving_target_name)
-        assert error_nameresolve in str(error_msg.value)
-
-        with pytest.raises(InvalidQueryError) as error_msg:
-            mast.Tesscut.download_cutouts(coordinates=coord, moving_target=True)
-        assert error_mt_coord in str(error_msg.value)
-
-        with pytest.raises(InvalidQueryError) as error_msg:
-            mast.Tesscut.download_cutouts(objectname=moving_target_name,
-                                     coordinates=coord)
-        assert error_name_coord in str(error_msg.value)
-
-        with pytest.raises(InvalidQueryError) as error_msg:
-            mast.Tesscut.download_cutouts(objectname=moving_target_name,
-                                     coordinates=coord,
-                                     moving_target=True)
-        assert error_mt_coord in str(error_msg.value)
-
-    def test_tesscut_get_cutouts(self, tmpdir):
-
-        coord = SkyCoord(107.18696, -70.50919, unit="deg")
-
-        cutout_hdus_list = mast.Tesscut.get_cutouts(coordinates=coord, size=5, sector=8,)
-        assert isinstance(cutout_hdus_list, list)
-        assert len(cutout_hdus_list) >= 1
-        assert isinstance(cutout_hdus_list[0], fits.HDUList)
-
-        cutout_hdus_list = mast.Tesscut.get_cutouts(coordinates=coord, size=5, sector=1)
-        assert isinstance(cutout_hdus_list, list)
-        assert len(cutout_hdus_list) == 1
-        assert isinstance(cutout_hdus_list[0], fits.HDUList)
-
-        coord = SkyCoord(349.62609, -47.12424, unit="deg")
-
-        cutout_hdus_list = mast.Tesscut.get_cutouts(coordinates=coord, size=[2, 4]*u.arcmin)
-        assert isinstance(cutout_hdus_list, list)
-        assert len(cutout_hdus_list) >= 1
-        assert isinstance(cutout_hdus_list[0], fits.HDUList)
-
-        cutout_hdus_list = mast.Tesscut.get_cutouts(objectname="TIC 32449963", size=5)
-        assert isinstance(cutout_hdus_list, list)
-        assert len(cutout_hdus_list) >= 1
-        assert isinstance(cutout_hdus_list[0], fits.HDUList)
-
-        # Moving target functionality testing
-
-        moving_target_name = 'Eleonora'
-
-        cutout_hdus_list = mast.Tesscut.get_cutouts(objectname=moving_target_name,
-                                moving_target=True,
-                                sector=6,
-                                size=5)
-        assert isinstance(cutout_hdus_list, list)
-        assert len(cutout_hdus_list) == 1
-        assert isinstance(cutout_hdus_list[0], fits.HDUList)
-
-        error_noname = "Please specify the object name or ID (as understood by the `JPL ephemerides service <https://ssd.jpl.nasa.gov/horizons.cgi>`__) of a moving target such as an asteroid or comet."
+        error_noname = ("Please specify the object name or ID (as understood by the "
+                        "`JPL ephemerides service <https://ssd.jpl.nasa.gov/horizons.cgi>`__) of "
+                        "a moving target such as an asteroid or comet.")
         error_nameresolve = f"Could not resolve {moving_target_name} to a sky position."
         error_mt_coord = "Only one of moving_target and coordinates may be specified."
         error_name_coord = "Only one of objectname and coordinates may be specified."
+        error_tica_mt = "Only SPOC is available for moving targets queries."
 
         with pytest.raises(InvalidQueryError) as error_msg:
             mast.Tesscut.download_cutouts(moving_target=True)
@@ -906,6 +961,92 @@ class TestMast:
                                           moving_target=True)
         assert error_mt_coord in str(error_msg.value)
 
+        # The TICA product option is not available for moving targets
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            mast.Tesscut.download_cutouts(objectname=moving_target_name, product='tica',
+                                          moving_target=True)
+            assert error_tica_mt in str(error_msg.value)
+
+    @pytest.mark.parametrize("product", ["tica", "spoc"])
+    def test_tesscut_get_cutouts(self, product):
+
+        coord = SkyCoord(107.18696, -70.50919, unit="deg")
+
+        cutout_hdus_list = mast.Tesscut.get_cutouts(product=product, coordinates=coord, size=5, sector=33)
+        assert isinstance(cutout_hdus_list, list)
+        assert len(cutout_hdus_list) >= 1
+        assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+        cutout_hdus_list = mast.Tesscut.get_cutouts(product=product, coordinates=coord, size=5, sector=27)
+        assert isinstance(cutout_hdus_list, list)
+        assert len(cutout_hdus_list) == 1
+        assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+        coord = SkyCoord(349.62609, -47.12424, unit="deg")
+
+        cutout_hdus_list = mast.Tesscut.get_cutouts(product=product, coordinates=coord, size=[2, 4]*u.arcmin)
+        assert isinstance(cutout_hdus_list, list)
+        assert len(cutout_hdus_list) >= 1
+        assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+        cutout_hdus_list = mast.Tesscut.get_cutouts(product=product, objectname="TIC 32449963", size=5)
+        assert isinstance(cutout_hdus_list, list)
+        assert len(cutout_hdus_list) >= 1
+        assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+    def test_tesscut_get_cutouts_mt(self):
+
+        # Moving target functionality testing
+        coord = SkyCoord(349.62609, -47.12424, unit="deg")
+        moving_target_name = 'Eleonora'
+
+        cutout_hdus_list = mast.Tesscut.get_cutouts(objectname=moving_target_name,
+                                                    moving_target=True,
+                                                    sector=6,
+                                                    size=5)
+        assert isinstance(cutout_hdus_list, list)
+        assert len(cutout_hdus_list) == 1
+        assert isinstance(cutout_hdus_list[0], fits.HDUList)
+
+        error_noname = ("Please specify the object name or ID (as understood by the "
+                        "`JPL ephemerides service <https://ssd.jpl.nasa.gov/horizons.cgi>`__) of "
+                        "a moving target such as an asteroid or comet.")
+        error_nameresolve = f"Could not resolve {moving_target_name} to a sky position."
+        error_mt_coord = "Only one of moving_target and coordinates may be specified."
+        error_name_coord = "Only one of objectname and coordinates may be specified."
+        error_tica_mt = "Only SPOC is available for moving targets queries."
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            mast.Tesscut.get_cutouts(moving_target=True)
+        assert error_noname in str(error_msg.value)
+
+        with pytest.raises(ResolverError) as error_msg:
+            mast.Tesscut.get_cutouts(objectname=moving_target_name)
+        assert error_nameresolve in str(error_msg.value)
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            mast.Tesscut.get_cutouts(coordinates=coord, moving_target=True)
+        assert error_mt_coord in str(error_msg.value)
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            mast.Tesscut.get_cutouts(objectname=moving_target_name,
+                                     coordinates=coord)
+        assert error_name_coord in str(error_msg.value)
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            mast.Tesscut.get_cutouts(objectname=moving_target_name,
+                                     coordinates=coord,
+                                     moving_target=True)
+        assert error_mt_coord in str(error_msg.value)
+
+        # The TICA product option is not available for moving targets
+
+        with pytest.raises(InvalidQueryError) as error_msg:
+            mast.Tesscut.get_cutouts(objectname=moving_target_name, product='tica',
+                                     moving_target=True)
+            assert error_tica_mt in str(error_msg.value)
+
     ###################
     # ZcutClass tests #
     ###################
@@ -959,7 +1100,8 @@ class TestMast:
         for row in cutout_table:
             assert os.path.isfile(cutout_table[0]['Local Path'])
 
-        cutout_table = mast.Zcut.download_cutouts(coordinates=coord, size=5, units='5*u.arcsec', cutout_format="png", path=str(tmpdir))
+        cutout_table = mast.Zcut.download_cutouts(
+            coordinates=coord, size=5, units='5*u.arcsec', cutout_format="png", path=str(tmpdir))
         assert isinstance(cutout_table, Table)
         assert len(cutout_table) >= 1
         assert cutout_table["Local Path"][0][-4:] == ".png"
@@ -975,14 +1117,16 @@ class TestMast:
             assert isinstance(cutout_table, Table)
             assert len(cutout_table) == 0
 
-        cutout_table = mast.Zcut.download_cutouts(coordinates=coord, survey='goods_north', cutout_format="jpg", path=str(tmpdir))
+        cutout_table = mast.Zcut.download_cutouts(
+            coordinates=coord, survey='goods_north', cutout_format="jpg", path=str(tmpdir))
         assert isinstance(cutout_table, Table)
         assert len(cutout_table) == 4
         assert cutout_table["Local Path"][0][-4:] == ".jpg"
         for row in cutout_table:
             assert os.path.isfile(cutout_table[0]['Local Path'])
 
-        cutout_table = mast.Zcut.download_cutouts(coordinates=coord, cutout_format="jpg", path=str(tmpdir), stretch='asinh', invert=True)
+        cutout_table = mast.Zcut.download_cutouts(
+            coordinates=coord, cutout_format="jpg", path=str(tmpdir), stretch='asinh', invert=True)
         assert isinstance(cutout_table, Table)
         assert len(cutout_table) >= 1
         assert cutout_table["Local Path"][0][-4:] == ".jpg"
@@ -1014,3 +1158,76 @@ class TestMast:
         assert isinstance(cutout_list, list)
         assert len(cutout_list) == 1
         assert isinstance(cutout_list[0], fits.HDUList)
+
+    ###################
+    # HapcutClass tests #
+    ###################
+
+    def test_hapcut_download_cutouts(self, tmpdir):
+
+        # Test 1: Simple API call with expected results
+        coord = SkyCoord(351.347812, 28.497808, unit="deg")
+
+        cutout_table = mast.Hapcut.download_cutouts(coordinates=coord, size=5, path=str(tmpdir))
+        assert isinstance(cutout_table, Table)
+        assert len(cutout_table) >= 1
+        for row in cutout_table:
+            assert os.path.isfile(row['Local Path'])
+            if 'fits' in os.path.basename(row['Local Path']):
+                assert fits.getdata(row['Local Path']).shape == (5, 5)
+
+        # Test 2: Make input size a list
+        cutout_table = mast.Hapcut.download_cutouts(coordinates=coord, size=[2, 3], path=str(tmpdir))
+        assert isinstance(cutout_table, Table)
+        assert len(cutout_table) >= 1
+        for row in cutout_table:
+            assert os.path.isfile(row['Local Path'])
+            if 'fits' in os.path.basename(row['Local Path']):
+                assert fits.getdata(row['Local Path']).shape == (3, 2)
+
+        # Test 3: Specify unit for input size
+        cutout_table = mast.Hapcut.download_cutouts(coordinates=coord, size=5*u.arcsec, path=str(tmpdir))
+        assert isinstance(cutout_table, Table)
+        assert len(cutout_table) >= 1
+        for row in cutout_table:
+            assert os.path.isfile(row['Local Path'])
+
+        # Test 4: Intentional API call with no results
+        bad_coord = SkyCoord(102.7, 70.50, unit="deg")
+        with pytest.warns(NoResultsWarning, match='Missing HAP files for input target. Cutout not performed.'):
+            cutout_table = mast.Hapcut.download_cutouts(coordinates=bad_coord, size=5, path=str(tmpdir))
+            assert isinstance(cutout_table, Table)
+            assert len(cutout_table) == 0
+
+    def test_hapcut_get_cutouts(self):
+
+        # Test 1: Simple API call with expected results
+        coord = SkyCoord(351.347812, 28.497808, unit="deg")
+
+        cutout_list = mast.Hapcut.get_cutouts(coordinates=coord)
+        assert isinstance(cutout_list, list)
+        assert len(cutout_list) >= 1
+        assert isinstance(cutout_list[0], fits.HDUList)
+        assert cutout_list[0][1].data.shape == (5, 5)
+
+        # Test 2: Make input size a list
+        cutout_list = mast.Hapcut.get_cutouts(coordinates=coord, size=[2, 3])
+        assert isinstance(cutout_list, list)
+        assert len(cutout_list) >= 1
+        assert isinstance(cutout_list[0], fits.HDUList)
+        assert cutout_list[0][1].data.shape == (3, 2)
+
+        # Test 3: Specify unit for input size
+        cutout_list = mast.Hapcut.get_cutouts(coordinates=coord, size=5*u.arcsec)
+        assert isinstance(cutout_list, list)
+        assert len(cutout_list) >= 1
+        assert isinstance(cutout_list[0], fits.HDUList)
+        assert cutout_list[0][1].data.shape == (42, 42)
+
+        # Test 4: Intentional API call with no results
+        bad_coord = SkyCoord(102.7, 70.50, unit="deg")
+
+        with pytest.warns(NoResultsWarning, match='Missing HAP files for input target. Cutout not performed.'):
+            cutout_list = mast.Hapcut.get_cutouts(coordinates=bad_coord)
+            assert isinstance(cutout_list, list)
+            assert len(cutout_list) == 0
