@@ -11,16 +11,12 @@ from astropy.time import Time
 from astropy.table import Table, QTable, Column
 import astropy.units as u
 from astropy.coordinates import EarthLocation, Angle, SkyCoord
-try:
-    from astropy.utils.exceptions import ErfaWarning
-except ImportError:
-    # DEPRECATED: remove eventually, but needed in July 2020
-    from astropy._erfa.core import ErfaWarning
+from erfa import ErfaWarning
 
 from ..query import BaseQuery
 from . import conf
 from ..utils import async_to_sync, class_or_instance
-from ..exceptions import InvalidQueryError
+from ..exceptions import InvalidQueryError, EmptyResponseError
 
 
 __all__ = ['MPCClass']
@@ -36,8 +32,8 @@ class MPCClass(BaseQuery):
     MPC_PASSWORD = 'mpc!!ws'
 
     MPES_URL = 'https://' + conf.mpes_server + '/cgi-bin/mpeph2.cgi'
-    OBSERVATORY_CODES_URL = ('https://' + conf.web_service_server +
-                             '/iau/lists/ObsCodes.html')
+    OBSERVATORY_CODES_URL = ('https://' + conf.web_service_server
+                             + '/iau/lists/ObsCodes.html')
 
     MPCOBS_URL = conf.mpcdb_server
 
@@ -63,9 +59,9 @@ class MPCClass(BaseQuery):
     }
 
     def __init__(self):
-        super(MPCClass, self).__init__()
+        super().__init__()
 
-    def query_object_async(self, target_type, get_query_payload=False, *args, **kwargs):
+    def query_object_async(self, target_type, *, get_query_payload=False, **kwargs):
         """
         Query around a specific object within a given mission catalog. When
         searching for a comet, it will return the entry with the latest epoch.
@@ -191,12 +187,12 @@ class MPCClass(BaseQuery):
 
         """
 
-        mpc_endpoint = self.get_mpc_object_endpoint(target_type)
+        self.get_mpc_object_endpoint(target_type)
 
         kwargs['limit'] = 1
-        return self.query_objects_async(target_type, get_query_payload, *args, **kwargs)
+        return self.query_objects_async(target_type, get_query_payload=get_query_payload, **kwargs)
 
-    def query_objects_async(self, target_type, get_query_payload=False, *args, **kwargs):
+    def query_objects_async(self, target_type, *, get_query_payload=False, **kwargs):
         """
         Query around a specific object within a given mission catalog
 
@@ -343,7 +339,7 @@ class MPCClass(BaseQuery):
         return mpc_endpoint
 
     @class_or_instance
-    def get_ephemeris_async(self, target, location='500', start=None, step='1d',
+    def get_ephemeris_async(self, target, *, location='500', start=None, step='1d',
                             number=None, ut_offset=0, eph_type='equatorial',
                             ra_format=None, dec_format=None,
                             proper_motion='total', proper_motion_unit='arcsec/h',
@@ -445,8 +441,9 @@ class MPCClass(BaseQuery):
             Return raw data without parsing into a table (default:
             ``False``).
 
-        cache : bool, optional
-            Cache results or use cached results (default: ``False``).
+        cache : bool
+            Defaults to False. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
 
         Returns
@@ -610,7 +607,7 @@ class MPCClass(BaseQuery):
         return response
 
     @class_or_instance
-    def get_observatory_codes_async(self, get_raw_response=False, cache=True):
+    def get_observatory_codes_async(self, *, get_raw_response=False, cache=True):
         """
         Table of observatory codes from the IAU Minor Planet Center.
 
@@ -621,8 +618,9 @@ class MPCClass(BaseQuery):
             Return raw data without parsing into a table (default:
             `False`).
 
-        cache : bool, optional
-            Cache results or use cached results (default: `True`).
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
 
         Returns
@@ -656,7 +654,7 @@ class MPCClass(BaseQuery):
         return response
 
     @class_or_instance
-    def get_observatory_location(self, code, cache=True):
+    def get_observatory_location(self, code, *, cache=True):
         """
         IAU observatory location.
 
@@ -666,9 +664,9 @@ class MPCClass(BaseQuery):
         code : string
             Three-character IAU observatory code.
 
-        cache : bool, optional
-            Cache observatory table or use cached results (default:
-            `True`).
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
 
         Returns
@@ -774,7 +772,7 @@ class MPCClass(BaseQuery):
         return request_args
 
     @class_or_instance
-    def get_observations_async(self, targetid,
+    def get_observations_async(self, targetid, *,
                                id_type=None,
                                comettype=None,
                                get_mpcformat=False,
@@ -824,8 +822,9 @@ class MPCClass(BaseQuery):
             Return the HTTP request parameters as a dictionary
             (default: ``False``).
 
-        cache : bool, optional
-            If ``True``, queries will be cached. Default: ``True``
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
 
         Raises
@@ -1183,8 +1182,12 @@ class MPCClass(BaseQuery):
                         result.text))
 
             if len(src) == 0:
-                raise RuntimeError(('No data queried. Are the target '
-                                    'identifiers correct?'))
+                raise EmptyResponseError(('No data queried. Are the target '
+                                          'identifiers correct?  Is the MPC '
+                                          'database search working for your '
+                                          'object? The service is hosted at '
+                                          'https://www.minorplanetcenter.net/'
+                                          'search_db'))
 
             # return raw response if requested
             if self.get_raw_response:
@@ -1216,8 +1219,8 @@ class MPCClass(BaseQuery):
                 ident = data['pdesig'][0]
                 if isinstance(ident, np.ma.masked_array) and ident.mask:
                     ident = ''
-                elif (len(ident) < 7 and ident[:4].isdigit() and
-                        ident[4:6].isalpha()):
+                elif (len(ident) < 7 and ident[:4].isdigit()
+                        and ident[4:6].isalpha()):
                     ident = ident[:4]+' '+ident[4:6]
                 # Palomar Survey
                 elif 'PLS' in ident:
@@ -1230,8 +1233,8 @@ class MPCClass(BaseQuery):
                 elif 'T3S' in ident:
                     ident = ident[3:] + " T-3"
                 # standard MPC packed 7-digit designation
-                elif (ident[0].isalpha() and ident[1:3].isdigit() and
-                      ident[-1].isalpha() and ident[-2].isdigit()):
+                elif (ident[0].isalpha() and ident[1:3].isdigit()
+                      and ident[-1].isalpha() and ident[-2].isdigit()):
                     yr = str(conf.pkd.find(ident[0]))+ident[1:3]
                     let = ident[3]+ident[-1]
                     num = str(conf.pkd.find(ident[4]))+ident[5]
@@ -1302,7 +1305,12 @@ class MPCClass(BaseQuery):
         data['epoch'].unit = u.d
         data['RA'].unit = u.deg
         data['DEC'].unit = u.deg
-        data['mag'].unit = u.mag
+
+        # Masked quantities are not supported in older astropy, warnings are raised for <v5.0
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', message='dropping mask in Quantity column',
+                                    category=UserWarning)
+            data['mag'].unit = u.mag
 
         return data
 
