@@ -2,6 +2,7 @@
 import pytest
 import astropy.units as u
 from astropy.coordinates import SkyCoord
+from astropy.utils.exceptions import AstropyDeprecationWarning
 
 from .. import Ogle
 
@@ -25,6 +26,7 @@ def test_ogle_list():
 @pytest.mark.remote_data
 def test_ogle_list_values():
     co_list = [[0, 0, 0], [3, 3, 3]]
-    response = Ogle.query_region(coord=co_list)
+    with pytest.warns(AstropyDeprecationWarning):
+        response = Ogle.query_region(coord=co_list)
     assert len(response) == 3
     assert response['RA[hr]'][0] == response['RA[hr]'][1] == response['RA[hr]'][2]
