@@ -4,7 +4,7 @@ SIMBAD Query Tool
 =================
 
 The SIMBAD query tool creates a `script query
-<http://simbad.u-strasbg.fr/simbad/sim-fscript>`__ that returns VOtable XML
+<https://simbad.cds.unistra.fr/simbad/sim-fscript>`__ that returns VOtable XML
 data that is then parsed into a SimbadResult object.  This object then
 parses the data and returns a table parsed with `astropy.io.votable.parse`.
 """
@@ -15,8 +15,11 @@ class Conf(_config.ConfigNamespace):
     """
     Configuration parameters for `astroquery.simbad`.
     """
+    # the first item is the default configuration
+    servers_list = ['simbad.cds.unistra.fr', 'simbad.harvard.edu']
+
     server = _config.ConfigItem(
-        ['simbad.u-strasbg.fr', 'simbad.harvard.edu'],
+        servers_list,
         'Name of the SIMBAD mirror to use.')
 
     timeout = _config.ConfigItem(
