@@ -3,23 +3,26 @@
 Simbad query class for accessing the Simbad Service
 """
 
-import copy
 import re
 import requests
 import json
 import os
 from collections import namedtuple
 from io import BytesIO
+from functools import lru_cache
 import warnings
 import astropy.units as u
+from astropy.utils import isiterable
 from astropy.utils.data import get_pkg_data_filename
 import astropy.coordinates as coord
 from astropy.table import Table
 import astropy.io.votable as votable
 
-from astroquery.query import BaseQuery
+from astroquery.query import BaseQuery, BaseVOQuery
 from astroquery.utils import commons, async_to_sync
-from astroquery.exceptions import TableParseError, LargeQueryWarning
+from astroquery.exceptions import TableParseError, LargeQueryWarning, BlankResponseWarning
+
+from pyvo.dal import TAPService
 from . import conf
 
 
@@ -82,14 +85,32 @@ def strip_field(field, keep_filters=False):
     """
     if '(' in field:
         root = field[:field.find('(')]
-        if (root in ('ra', 'dec', 'otype', 'id', 'coo', 'bibcodelist') or
-                not keep_filters):
+        if (root in ('ra', 'dec', 'otype', 'id', 'coo', 'bibcodelist')
+                or not keep_filters):
             return root
 
     # the overall else (default option)
     return field
 
 
+def _adql_parameter(entry: str):
+    """Replace single quotes by two single quotes.
+
+    This should be applied to parameters used in ADQL queries.
+    It is not a SQL injection protection: it just allows to search, for example,
+    for authors with quotes in their names or titles/descriptions with apostrophes.
+
+    Parameters
+    ----------
+    entry : str
+
+    Returns
+    -------
+    str
+    """
+    return entry.replace("'", "''")
+
+
 error_regex = re.compile(r'(?ms)\[(?P<line>\d+)\]\s?(?P<msg>.+?)(\[|\Z)')
 SimbadError = namedtuple('SimbadError', ('line', 'msg'))
 VersionInfo = namedtuple('VersionInfo', ('major', 'minor', 'micro', 'patch'))
@@ -135,7 +156,9 @@ class SimbadResult:
             warnings.warn("Warning: The script line number %i raised "
                           "an error (recorded in the `errors` attribute "
                           "of the result table): %s" %
-                          (error.line, error.msg))
+                          (error.line, error.msg),
+                          BlankResponseWarning
+                          )
 
     def __get_section(self, section_name):
         if section_name in self.__indexes:
@@ -202,25 +225,20 @@ class SimbadBibcodeResult(SimbadResult):
     """Bibliography-type Simbad result"""
     @property
     def table(self):
-        bibcode_match = bibcode_regex.search(self.script)
-        splitter = bibcode_match.group(2)
-        ref_list = [splitter + ref for ref in self.data.split(splitter)][1:]
-        max_len = max([len(r) for r in ref_list])
-        table = Table(names=['References'], dtype=['U%i' % max_len])
-        for ref in ref_list:
-            table.add_row([ref])
-        return table
+        splitter = bibcode_regex.search(self.script).group(2)
+        ref_list = [[splitter + ref] for ref in self.data.split(splitter)[1:]]
+        max_len = max(len(r[0]) for r in ref_list)
+        return Table(rows=ref_list, names=['References'], dtype=[f"U{max_len}"])
 
 
 class SimbadObjectIDsResult(SimbadResult):
     """Object identifier list Simbad result"""
     @property
     def table(self):
-        max_len = max([len(i) for i in self.data.splitlines()])
-        table = Table(names=['ID'], dtype=['S%i' % max_len])
-        for id in self.data.splitlines():
-            table.add_row([id.strip()])
-        return table
+        split_lines = self.data.splitlines()
+        ids = [[id.strip()] for id in split_lines]
+        max_len = max(map(len, split_lines))
+        return Table(rows=ids, names=['ID'], dtype=[f"S{max_len}"])
 
 
 class SimbadBaseQuery(BaseQuery):
@@ -230,6 +248,7 @@ class SimbadBaseQuery(BaseQuery):
     in a "connection refused" error (error 61) instead of a more typical "error
     8" that you would get from not having an internet connection at all.
     """
+
     def _request(self, *args, **kwargs):
         try:
             response = super()._request(*args, **kwargs)
@@ -262,16 +281,16 @@ class SimbadBaseQuery(BaseQuery):
 
 
 @async_to_sync
-class SimbadClass(SimbadBaseQuery):
+class SimbadClass(BaseVOQuery, SimbadBaseQuery):
     """
     The class for querying the Simbad web service.
 
     Note that SIMBAD suggests submitting no more than 6 queries per second; if
     you submit more than that, your IP may be temporarily blacklisted
-    (http://simbad.u-strasbg.fr/simbad/sim-help?Page=sim-url)
+    (https://simbad.cds.unistra.fr/guide/sim-url.htx)
 
     """
-    SIMBAD_URL = 'http://' + conf.server + '/simbad/sim-script'
+    SIMBAD_URL = 'https://' + conf.server + '/simbad/sim-script'
     TIMEOUT = conf.timeout
     WILDCARDS = {
         '*': 'Any string of characters (including an empty one)',
@@ -281,8 +300,6 @@ class SimbadClass(SimbadBaseQuery):
                   ),
         '[^0-9]': 'Any (one) character not in the list.'}
 
-    _ORDERED_WILDCARDS = ['*', '?', '[abc]', '[^0-9]']
-
     # query around not included since this is a subcase of query_region
     _function_to_command = {
         'query_object_async': 'query id',
@@ -296,14 +313,58 @@ class SimbadClass(SimbadBaseQuery):
     ROW_LIMIT = conf.row_limit
 
     # also find a way to fetch the votable fields table from
-    # <http://simbad.u-strasbg.fr/simbad/sim-help?Page=sim-fscript#VotableFields>
+    # <https://simbad.cds.unistra.fr/guide/sim-fscript.htx#VotableFields>
     # tried something for this in this ipython nb
     # <http://nbviewer.ipython.org/5851110>
     _VOTABLE_FIELDS = ['main_id', 'coordinates']
 
     def __init__(self):
         super().__init__()
-        self._VOTABLE_FIELDS = copy.copy(self._VOTABLE_FIELDS)
+        self._VOTABLE_FIELDS = self._VOTABLE_FIELDS.copy()
+        self._server = conf.server
+        self._tap = None
+
+    @property
+    def server(self):
+        """The Simbad mirror to use."""
+        return self._server
+
+    @server.setter
+    def server(self, server: str):
+        """Allows to switch server between Simbad mirrors.
+
+        Parameters
+        ----------
+        server : str
+            It should be one of `~astroquery.simbad.conf.servers_list`.
+        """
+        if server in conf.servers_list:
+            self._server = server
+        else:
+            raise ValueError(f"'{server}' does not correspond to a Simbad server, "
+                             f"the two existing ones are {conf.servers_list}.")
+
+    @property
+    def tap(self):
+        """A `~pyvo.dal.TAPService` service for Simbad."""
+        tap_url = f"https://{self.server}/simbad/sim-tap"
+        # only creates a new tap instance if there are no existing one
+        # or if the server property changed since the last getter call.
+        if (not self._tap) or (self._tap.baseurl != tap_url):
+            self._tap = TAPService(baseurl=tap_url, session=self._session)
+        return self._tap
+
+    @property
+    @lru_cache(1)
+    def hardlimit(self):
+        """The maximum number of lines for Simbad's output.
+
+        This property is cached to avoid calls to simbad's capability
+        webpage each time the getter is called.
+        """
+        # replace stack of property and lru_cache by functools.cache_property when
+        # astroquery drops python 3.7 support
+        return self.tap.hardlimit
 
     def list_wildcards(self):
         """
@@ -323,10 +384,7 @@ class SimbadClass(SimbadBaseQuery):
         [abc] : Exactly one character taken in the list.
                 Can also be defined by a range of characters: [A-Z]
         """
-        for key in self._ORDERED_WILDCARDS:
-            print("{key} : {value}\n".format(key=key,
-                                             value=self.WILDCARDS[key]))
-        return
+        print("\n\n".join(f"{k} : {v}" for k, v in self.WILDCARDS.items()))
 
     def list_votable_fields(self):
         """
@@ -353,8 +411,8 @@ class SimbadClass(SimbadBaseQuery):
             fields_dict = json.load(f)
 
         print("Available VOTABLE fields:\n")
-        for field in list(sorted(fields_dict.keys())):
-            print("{}".format(field))
+        for field in sorted(fields_dict.keys()):
+            print(str(field))
         print("For more information on a field:\n"
               "Simbad.get_field_description ('field_name') \n"
               "Currently active VOTABLE fields:\n {0}"
@@ -415,10 +473,7 @@ class SimbadClass(SimbadBaseQuery):
             os.path.join('data', 'votable_fields_dict.json'))
 
         with open(dict_file, "r") as f:
-            fields_dict = json.load(f)
-            fields_dict = dict(
-                ((strip_field(ff), fields_dict[ff])
-                 for ff in fields_dict))
+            fields_dict = {strip_field(k): v for k, v in json.load(f).items()}
 
         for field in args:
             sf = strip_field(field)
@@ -427,34 +482,30 @@ class SimbadClass(SimbadBaseQuery):
             else:
                 self._VOTABLE_FIELDS.append(field)
 
-    def remove_votable_fields(self, *args, **kwargs):
+    def remove_votable_fields(self, *args, strip_params=False):
         """
         Removes the specified field names from ``SimbadClass._VOTABLE_FIELDS``
 
         Parameters
         ----------
         list of field_names to be removed
-        strip_params: bool
+        strip_params: bool, optional
             If true, strip the specified keywords before removing them:
             e.g., ra(foo) would remove ra(bar) if this is True
         """
-        strip_params = kwargs.pop('strip_params', False)
-
         if strip_params:
-            sargs = [strip_field(a) for a in args]
+            sargs = {strip_field(a) for a in args}
             sfields = [strip_field(a) for a in self._VOTABLE_FIELDS]
         else:
-            sargs = args
+            sargs = set(args)
             sfields = self._VOTABLE_FIELDS
-        absent_fields = set(sargs) - set(sfields)
 
-        for b, f in list(zip(sfields, self._VOTABLE_FIELDS)):
-            if b in sargs:
-                self._VOTABLE_FIELDS.remove(f)
-
-        for field in absent_fields:
+        for field in sargs.difference(sfields):
             warnings.warn("{field}: this field is not set".format(field=field))
 
+        zipped_fields = zip(sfields, self._VOTABLE_FIELDS)
+        self._VOTABLE_FIELDS = [f for b, f in zipped_fields if b not in sargs]
+
         # check if all fields are removed
         if not self._VOTABLE_FIELDS:
             warnings.warn("All fields have been removed. "
@@ -489,7 +540,7 @@ class SimbadClass(SimbadBaseQuery):
         result = self.query_criteria_async(*args, **kwargs)
         return self._parse_result(result, SimbadVOTableResult, verbose=verbose)
 
-    def query_criteria_async(self, *args, **kwargs):
+    def query_criteria_async(self, *args, cache=True, **kwargs):
         """
         Query SIMBAD based on any criteria.
 
@@ -502,14 +553,14 @@ class SimbadClass(SimbadBaseQuery):
             Keyword / value pairs passed to SIMBAD's script engine
             (e.g., {'otype':'SNR'} will be rendered as otype=SNR)
         cache : bool
-            Cache the query?
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
         response : `requests.Response`
             Response of the query from the server
         """
-        cache = kwargs.pop('cache', True)
 
         request_payload = self._args_to_payload(caller='query_criteria_async',
                                                 *args, **kwargs)
@@ -517,7 +568,7 @@ class SimbadClass(SimbadBaseQuery):
                                  timeout=self.TIMEOUT, cache=cache)
         return response
 
-    def query_object(self, object_name, wildcard=False, verbose=False,
+    def query_object(self, object_name, *, wildcard=False, verbose=False,
                      get_query_payload=False):
         """
         Queries Simbad for the given object and returns the result as a
@@ -548,9 +599,8 @@ class SimbadClass(SimbadBaseQuery):
         return self._parse_result(response, SimbadVOTableResult,
                                   verbose=verbose)
 
-    def query_object_async(self, object_name, wildcard=False, cache=True,
+    def query_object_async(self, object_name, *, wildcard=False, cache=True,
                            get_query_payload=False):
-
         """
         Serves the same function as `query_object`, but
         only collects the response from the Simbad server and returns.
@@ -565,6 +615,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -581,7 +634,7 @@ class SimbadClass(SimbadBaseQuery):
                                  timeout=self.TIMEOUT, cache=cache)
         return response
 
-    def query_objects(self, object_names, wildcard=False, verbose=False,
+    def query_objects(self, object_names, *, wildcard=False, verbose=False,
                       get_query_payload=False):
         """
         Queries Simbad for the specified list of objects and returns the
@@ -607,7 +660,7 @@ class SimbadClass(SimbadBaseQuery):
         return self.query_object('\n'.join(object_names), wildcard=wildcard,
                                  get_query_payload=get_query_payload)
 
-    def query_objects_async(self, object_names, wildcard=False, cache=True,
+    def query_objects_async(self, object_names, *, wildcard=False, cache=True,
                             get_query_payload=False):
         """
         Same as `query_objects`, but only collects the response from the
@@ -623,6 +676,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -633,7 +689,7 @@ class SimbadClass(SimbadBaseQuery):
                                        wildcard=wildcard, cache=cache,
                                        get_query_payload=get_query_payload)
 
-    def query_region_async(self, coordinates, radius=2*u.arcmin,
+    def query_region_async(self, coordinates, radius=2*u.arcmin, *,
                            equinox=2000.0, epoch='J2000', cache=True,
                            get_query_payload=False):
         """
@@ -644,9 +700,8 @@ class SimbadClass(SimbadBaseQuery):
         ----------
         coordinates : str or `astropy.coordinates` object
             the identifier or coordinates around which to query.
-        radius : str or `~astropy.units.Quantity`, optional
-            the radius of the region. If missing, set to default
-            value of 2 arcmin.
+        radius : str or `~astropy.units.Quantity`
+            the radius of the region.  Defaults to 2 arcmin.
         equinox : float, optional
             the equinox of the coordinates. If missing set to
             default 2000.0.
@@ -656,6 +711,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -663,14 +721,15 @@ class SimbadClass(SimbadBaseQuery):
              Response of the query from the server.
         """
 
+        if radius is None:
+            # this message is specifically for deprecated use of 'None' to mean 'Default'
+            raise ValueError("Radius must be specified as an angle-equivalent quantity, not None")
+
         equinox = validate_equinox(equinox)
         epoch = validate_epoch(epoch)
 
         base_query_str = "query coo {ra} {dec} radius={rad} frame={frame} equi={equinox}"
 
-        if radius is None:
-            radius = 2*u.arcmin
-
         header = self._get_query_header()
         footer = self._get_query_footer()
 
@@ -678,8 +737,6 @@ class SimbadClass(SimbadBaseQuery):
 
         # handle the vector case
         if isinstance(ra, list):
-            vector = True
-
             if len(ra) > 10000:
                 warnings.warn("For very large queries, you may receive a "
                               "timeout error.  SIMBAD suggests splitting "
@@ -689,21 +746,19 @@ class SimbadClass(SimbadBaseQuery):
             if len(set(frame)) > 1:
                 raise ValueError("Coordinates have different frames")
             else:
-                frame = set(frame).pop()
+                frame = frame[0]
 
-            if vector and _has_length(radius) and len(radius) == len(ra):
-                # all good, continue
-                pass
-            elif vector and _has_length(radius) and len(radius) != len(ra):
-                raise ValueError("Mismatch between radii and coordinates")
-            elif vector and not _has_length(radius):
+            # `radius` as `str` is iterable, but contains only one value.
+            if isiterable(radius) and not isinstance(radius, str):
+                if len(radius) != len(ra):
+                    raise ValueError("Mismatch between radii and coordinates")
+            else:
                 radius = [_parse_radius(radius)] * len(ra)
 
-            if vector:
-                query_str = "\n".join([base_query_str
-                                      .format(ra=ra_, dec=dec_, rad=rad_,
-                                              frame=frame, equinox=equinox)
-                                      for ra_, dec_, rad_ in zip(ra, dec, radius)])
+            query_str = "\n".join(base_query_str
+                                  .format(ra=ra_, dec=dec_, rad=rad_,
+                                          frame=frame, equinox=equinox)
+                                  for ra_, dec_, rad_ in zip(ra, dec, radius))
 
         else:
             radius = _parse_radius(radius)
@@ -719,7 +774,7 @@ class SimbadClass(SimbadBaseQuery):
                                  timeout=self.TIMEOUT, cache=cache)
         return response
 
-    def query_catalog(self, catalog, verbose=False, cache=True,
+    def query_catalog(self, catalog, *, verbose=False, cache=True,
                       get_query_payload=False):
         """
         Queries a whole catalog.
@@ -734,6 +789,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -748,7 +806,7 @@ class SimbadClass(SimbadBaseQuery):
         return self._parse_result(response, SimbadVOTableResult,
                                   verbose=verbose)
 
-    def query_catalog_async(self, catalog, cache=True, get_query_payload=False):
+    def query_catalog_async(self, catalog, *, cache=True, get_query_payload=False):
         """
         Serves the same function as `query_catalog`, but
         only collects the response from the Simbad server and returns.
@@ -760,6 +818,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -776,7 +837,7 @@ class SimbadClass(SimbadBaseQuery):
                                  timeout=self.TIMEOUT, cache=cache)
         return response
 
-    def query_bibobj(self, bibcode, verbose=False, get_query_payload=False):
+    def query_bibobj(self, bibcode, *, verbose=False, get_query_payload=False):
         """
         Query all the objects that are contained in the article specified by
         the bibcode, and return results as a `~astropy.table.Table`.
@@ -802,7 +863,7 @@ class SimbadClass(SimbadBaseQuery):
         return self._parse_result(response, SimbadVOTableResult,
                                   verbose=verbose)
 
-    def query_bibobj_async(self, bibcode, cache=True, get_query_payload=False):
+    def query_bibobj_async(self, bibcode, *, cache=True, get_query_payload=False):
         """
         Serves the same function as `query_bibobj`, but only collects the
         response from the Simbad server and returns.
@@ -814,6 +875,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -830,7 +894,7 @@ class SimbadClass(SimbadBaseQuery):
                                  timeout=self.TIMEOUT, cache=cache)
         return response
 
-    def query_bibcode(self, bibcode, wildcard=False, verbose=False,
+    def query_bibcode(self, bibcode, *, wildcard=False, verbose=False,
                       cache=True, get_query_payload=False):
         """
         Queries the references corresponding to a given bibcode, and returns
@@ -847,6 +911,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -864,7 +931,7 @@ class SimbadClass(SimbadBaseQuery):
         return self._parse_result(response, SimbadBibcodeResult,
                                   verbose=verbose)
 
-    def query_bibcode_async(self, bibcode, wildcard=False, cache=True,
+    def query_bibcode_async(self, bibcode, *, wildcard=False, cache=True,
                             get_query_payload=False):
         """
         Serves the same function as `query_bibcode`, but
@@ -880,6 +947,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -899,7 +969,7 @@ class SimbadClass(SimbadBaseQuery):
 
         return response
 
-    def query_objectids(self, object_name, verbose=False, cache=True,
+    def query_objectids(self, object_name, *, verbose=False, cache=True,
                         get_query_payload=False):
         """
         Query Simbad with an object name, and return a table of all
@@ -912,6 +982,9 @@ class SimbadClass(SimbadBaseQuery):
         get_query_payload : bool, optional
             When set to `True` the method returns the HTTP request parameters.
             Defaults to `False`.
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -927,7 +1000,7 @@ class SimbadClass(SimbadBaseQuery):
         return self._parse_result(response, SimbadObjectIDsResult,
                                   verbose=verbose)
 
-    def query_objectids_async(self, object_name, cache=True,
+    def query_objectids_async(self, object_name, *, cache=True,
                               get_query_payload=False):
         """
         Serves the same function as `query_objectids`, but
@@ -937,6 +1010,9 @@ class SimbadClass(SimbadBaseQuery):
         ----------
         object_name : str
             name of object to be queried
+        cache : bool
+            Defaults to True. If set overrides global caching behavior.
+            See :ref:`caching documentation <astroquery_cache>`.
 
         Returns
         -------
@@ -955,21 +1031,266 @@ class SimbadClass(SimbadBaseQuery):
 
         return response
 
+    def list_tables(self, *, get_adql=False):
+        """The names and descriptions of the tables in SIMBAD.
+
+        Parameters
+        ----------
+        get_adql : bool, optional
+            Returns the ADQL string instead of querying SIMBAD.
+
+        Returns
+        -------
+        `~astropy.table.Table`
+        """
+        query = ("SELECT table_name, description"
+                 " FROM TAP_SCHEMA.tables"
+                 " WHERE schema_name = 'public'")
+        if get_adql:
+            return query
+        return self.query_tap(query)
+
+    def list_columns(self, *tables: str, keyword=None, get_adql=False):
+        """
+        Get the list of SIMBAD columns.
+
+        Add tables names to restrict to some tables. Call the function without
+        any parameter to get all columns names from all tables. The keyword argument
+        looks for columns in the selected Simbad tables that contain the
+        given keyword. The keyword search is not case-sensitive.
+
+        Parameters
+        ----------
+        *tables : str, optional
+            Add tables names as strings to restrict to these tables columns.
+        keyword : str, optional
+            A keyword to look for in column names, table names, or descriptions.
+        get_adql : bool, optional
+            Returns the ADQL string instead of querying SIMBAD.
+
+        Examples
+        --------
+        >>> from astroquery.simbad import Simbad
+        >>> Simbad.list_columns("ids", "ident") # doctest: +REMOTE_DATA
+        <Table length=4>
+        table_name column_name datatype ...  unit    ucd
+          object      object    object  ... object  object
+        ---------- ----------- -------- ... ------ -------
+             ident          id  VARCHAR ...        meta.id
+             ident      oidref   BIGINT ...
+               ids         ids  VARCHAR ...        meta.id
+               ids      oidref   BIGINT ...
+
+
+        >>> from astroquery.simbad import Simbad
+        >>> Simbad.list_columns(keyword="filter") # doctest: +REMOTE_DATA
+        <Table length=5>
+         table_name column_name   datatype  ...  unit           ucd
+           object      object      object   ... object         object
+        ----------- ----------- ----------- ... ------ ----------------------
+             filter description UNICODECHAR ...        meta.note;instr.filter
+             filter  filtername     VARCHAR ...                  instr.filter
+             filter        unit     VARCHAR ...                     meta.unit
+               flux      filter     VARCHAR ...                  instr.filter
+        mesDiameter      filter        CHAR ...                  instr.filter
+
+        >>> from astroquery.simbad import Simbad
+        >>> Simbad.list_columns("basic", keyword="object") # doctest: +REMOTE_DATA
+        <Table length=4>
+        table_name column_name datatype ...  unit          ucd
+          object      object    object  ... object        object
+        ---------- ----------- -------- ... ------ -------------------
+             basic     main_id  VARCHAR ...          meta.id;meta.main
+             basic   otype_txt  VARCHAR ...                  src.class
+             basic         oid   BIGINT ...        meta.record;meta.id
+             basic       otype  VARCHAR ...                  src.class
+        """
+        query = ("SELECT table_name, column_name, datatype, description, unit, ucd"
+                 " FROM TAP_SCHEMA.columns"
+                 " WHERE table_name NOT LIKE 'TAP_SCHEMA.%'")
+        # select the tables
+        if len(tables) == 1:
+            query += f" AND table_name = '{tables[0]}'"
+        elif len(tables) > 1:
+            query += f" AND table_name IN {tables}"
+        # add the keyword condition
+        if keyword is not None:
+            condition = f"LIKE LOWERCASE('%{_adql_parameter(keyword)}%')"
+            query += (f" AND ( (LOWERCASE(column_name) {condition})"
+                      f" OR (LOWERCASE(description) {condition})"
+                      f" OR (LOWERCASE(table_name) {condition}))")
+        query += " ORDER BY table_name, principal DESC, column_name"
+        if get_adql:
+            return query
+        return self.query_tap(query)
+
+    def list_linked_tables(self, table: str, *, get_adql=False):
+        """
+        Expose the tables that can be non-obviously linked with the given table.
+
+        This list contains only the links where the column names are not the same in the
+        two tables. For example every ``oidref`` column of any table can be joined with
+        any other ``oidref``. The same goes for every ``otype`` column even if this is not
+        returned by this method.
+
+        Parameters
+        ----------
+        table : str
+            One of SIMBAD's tables name
+        get_adql : bool, optional
+            Returns the ADQL string instead of querying SIMBAD.
+
+        Returns
+        -------
+        `~astropy.table.Table`
+            The information necessary to join the given table to an other.
+
+        Examples
+        --------
+        >>> from astroquery.simbad import Simbad
+        >>> Simbad.list_linked_tables("otypes") # doctest: +REMOTE_DATA
+        <Table length=2>
+        from_table from_column target_table target_column
+          object      object      object        object
+        ---------- ----------- ------------ -------------
+          otypedef       otype       otypes         otype
+            otypes      oidref        basic           oid
+        """
+        query = ("SELECT from_table, from_column, target_table, target_column"
+                 " FROM TAP_SCHEMA.key_columns JOIN TAP_SCHEMA.keys USING (key_id)"
+                 f" WHERE (from_table = '{_adql_parameter(table)}')"
+                 f" OR (target_table = '{_adql_parameter(table)}')")
+        if get_adql:
+            return query
+        return self.query_tap(query)
+
+    @lru_cache(256)
+    def _cached_query_tap(self, query: str, *, maxrec=10000):
+        """Cache version of query TAP
+
+        This private method is called when query_tap is executed without an
+        ``uploads`` extra keyword argument. This is a work around because
+        `~astropy.table.Table` objects are not hashable and thus cannot
+        be used as arguments for a function decorated with lru_cache.
+
+        Parameters
+        ----------
+        query : str
+            A string containing the query written in the
+            Astronomical Data Query Language (ADQL).
+        maxrec : int, optional
+            The number of records to be returned. Its maximum value is 2000000.
+
+        Returns
+        -------
+        `~astropy.table.Table`
+            The response returned by Simbad.
+        """
+        return self.tap.run_async(query, maxrec=maxrec).to_table()
+
+    def query_tap(self, query: str, *, maxrec=10000, **uploads):
+        """
+        Query Simbad TAP service.
+
+        Parameters
+        ----------
+        query : str
+            A string containing the query written in the
+            Astronomical Data Query Language (ADQL).
+        maxrec : int, default: 10000
+            The number of records to be returned. Its maximum value is given by
+            `~astroquery.simbad.SimbadClass.hardlimit`.
+        uploads : `~astropy.table.Table` | `~astropy.io.votable.tree.VOTableFile` | `~pyvo.dal.DALResults`
+            Any number of local tables to be used in the *query*. In the *query*, these tables
+            are referred as *TAP_UPLOAD.table_alias* where *TAP_UPLOAD* is imposed and *table_alias*
+            is the keyword name you chose. The maximum number of lines for the uploaded tables is 200000.
+
+        Returns
+        -------
+        `~astropy.table.Table`
+            The response returned by Simbad.
+
+        Notes
+        -----
+        A TAP (Table Access Protocol) service allows to query data tables with
+        queries written in ADQL (Astronomical Data Query Language), a flavor
+        of the more general SQL (Structured Query Language).
+        For more documentation about writing ADQL queries, you can read its official
+        documentation (`ADQL documentation <https://ivoa.net/documents/ADQL/index.html>`__)
+        or the `Simbad ADQL cheat sheet <http://simbad.cds.unistra.fr/simbad/tap/help/adqlHelp.html>`__.
+        See also: a `graphic representation of Simbad's tables and their relations
+        <http://simbad.cds.unistra.fr/simbad/tap/tapsearch.html>`__.
+
+        See also
+        --------
+        list_tables : The list of SIMBAD's tables.
+        list_columns : SIMBAD's columns list, can be restricted to some tables and some keyword.
+        list_linked_tables : Given a table, expose non-obvious possible joins with other tables.
+
+        Examples
+        --------
+
+        To see the five oldest papers referenced in Simbad
+
+        >>> from astroquery.simbad import Simbad
+        >>> Simbad.query_tap("SELECT top 5 bibcode, title "
+        ...                  "FROM ref ORDER BY bibcode") # doctest: +REMOTE_DATA
+        <Table length=5>
+              bibcode       ...
+               object       ...
+        ------------------- ...
+        1850CDT..1784..227M ...
+        1857AN.....45...89S ...
+        1861MNRAS..21...68B ...
+        1874MNRAS..34...75S ...
+        1877AN.....89...13W ...
+
+        Get the type for a list of objects
+
+        >>> from astroquery.simbad import Simbad
+        >>> Simbad.query_tap("SELECT main_id, otype"
+        ...                  " FROM basic WHERE main_id IN ('m10', 'm13')") # doctest: +REMOTE_DATA
+        <Table length=2>
+        main_id otype
+         object object
+        ------- ------
+          M  10    GlC
+          M  13    GlC
+
+        Upload a table to use in a query
+
+        >>> from astroquery.simbad import Simbad
+        >>> from astropy.table import Table
+        >>> letters_table = Table([["a", "b", "c"]], names=["alphabet"])
+        >>> Simbad.query_tap("SELECT TAP_UPLOAD.my_table_name.* from TAP_UPLOAD.my_table_name",
+        ...                  my_table_name=letters_table) # doctest: +REMOTE_DATA
+        <Table length=3>
+        alphabet
+         object
+        --------
+               a
+               b
+               c
+        """
+        if maxrec > Simbad.hardlimit:
+            raise ValueError(f"The maximum number of records cannot exceed {Simbad.hardlimit}.")
+        if query.count("'") % 2:
+            raise ValueError("Query string contains an odd number of single quotes."
+                             " Escape the unpaired single quote by doubling it.\n"
+                             "ex: 'Barnard's galaxy' -> 'Barnard''s galaxy'.")
+        if uploads == {}:
+            return self._cached_query_tap(query, maxrec=maxrec)
+        return self.tap.run_async(query, maxrec=maxrec, uploads=uploads).to_table()
+
     def _get_query_header(self, get_raw=False):
-        votable_fields = ','.join(self.get_votable_fields())
         # if get_raw is set then don't fetch as votable
         if get_raw:
             return ""
-        votable_def = "votable {" + votable_fields + "}"
-        votable_open = "votable open"
-        return "\n".join([votable_def, votable_open])
+        row_limit = f"set limit {self.ROW_LIMIT}\n" if self.ROW_LIMIT > 0 else ""
+        return f"{row_limit}votable {{{','.join(self.get_votable_fields())}}}\nvotable open"
 
     def _get_query_footer(self, get_raw=False):
-        if get_raw:
-            return ""
-        votable_close = "votable close"
-
-        return votable_close
+        return "" if get_raw else "votable close"
 
     @validate_epoch_decorator
     @validate_equinox_decorator
@@ -988,8 +1309,6 @@ class SimbadClass(SimbadBaseQuery):
         votable_header = self._get_query_header(get_raw)
         votable_footer = self._get_query_footer(get_raw)
 
-        if self.ROW_LIMIT > 0:
-            script = "set limit " + str(self.ROW_LIMIT)
         script = "\n".join([script, votable_header, command])
         using_wildcard = False
         if kwargs.get('wildcard'):
@@ -1004,25 +1323,19 @@ class SimbadClass(SimbadBaseQuery):
             kwargs['equi'] = kwargs['equinox']
             del kwargs['equinox']
         # remove default None from kwargs
-        # be compatible with python3
-        for key in list(kwargs):
-            if not kwargs[key]:
-                del kwargs[key]
+        kwargs = {key: value for key, value in kwargs.items() if value is not None}
         # join in the order specified otherwise results in error
         all_keys = ['radius', 'frame', 'equi', 'epoch']
         present_keys = [key for key in all_keys if key in kwargs]
         if caller == 'query_criteria_async':
-            for k in kwargs:
-                present_keys.append(k)
+            present_keys.extend(kwargs)
             # need ampersands to join args
             args_str = '&'.join([str(val) for val in args])
-            if len(args) > 0 and len(present_keys) > 0:
+            if args and present_keys:
                 args_str += " & "
         else:
             args_str = ' '.join([str(val) for val in args])
-        kwargs_str = ' '.join("{key}={value}".format(key=key,
-                                                     value=kwargs[key])
-                              for key in present_keys)
+        kwargs_str = ' '.join(f"{key}={kwargs[key]}" for key in present_keys)
 
         # For the record, I feel dirty for writing this wildcard-case hack.
         # This entire function should be refactored when someone has time.
@@ -1081,17 +1394,8 @@ def _parse_coordinates(coordinates):
         raise ValueError("Coordinates not specified correctly")
 
 
-def _has_length(x):
-    # some objects have '__len__' attributes but have no len()
-    try:
-        len(x)
-        return True
-    except (TypeError, AttributeError):
-        return False
-
-
 def _get_frame_coords(coordinates):
-    if _has_length(coordinates):
+    if isiterable(coordinates):
         # deal with vectors differently
         parsed = [_get_frame_coords(cc) for cc in coordinates]
         return ([ra for ra, dec, frame in parsed],
