@@ -4,23 +4,20 @@ import re
 
 import pytest
 import astropy.units as u
+from astropy.coordinates import SkyCoord
 from astropy.table import Table
 import numpy as np
 
 from ... import simbad
 from astroquery.utils.mocks import MockResponse
-from ...utils import commons
 from ...query import AstroQuery
 from ...exceptions import TableParseError
 from .test_simbad_remote import multicoords
 
-GALACTIC_COORDS = commons.GalacticCoordGenerator(l=-67.02084, b=-29.75447,
-                                                 unit=(u.deg, u.deg))
-ICRS_COORDS = commons.ICRSCoordGenerator("05h35m17.3s -05h23m28s")
-FK4_COORDS = commons.FK4CoordGenerator(ra=84.90759, dec=-80.89403,
-                                       unit=(u.deg, u.deg))
-FK5_COORDS = commons.FK5CoordGenerator(ra=83.82207, dec=-80.86667,
-                                       unit=(u.deg, u.deg))
+GALACTIC_COORDS = SkyCoord(l=-67.02084 * u.deg, b=-29.75447 * u.deg, frame="galactic")
+ICRS_COORDS = SkyCoord("05h35m17.3s -05h23m28s", frame="icrs")
+FK4_COORDS = SkyCoord(ra=84.90759 * u.deg, dec=-80.89403 * u.deg, frame="fk4")
+FK5_COORDS = SkyCoord(ra=83.82207 * u.deg, dec=-80.86667 * u.deg, frame="fk5")
 
 DATA_FILES = {
     'id': 'query_id.data',
@@ -40,14 +37,15 @@ class MockResponseSimbad(MockResponse):
 
     def __init__(self, script, cache=False, **kwargs):
         # preserve, e.g., headers
-        super(MockResponseSimbad, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.content = self.get_content(script)
 
     def get_content(self, script):
         match = self.query_regex.search(script)
         if match:
             filename = DATA_FILES[match.group(1)]
-            content = open(data_path(filename), "rb").read()
+            with open(data_path(filename), "rb") as infile:
+                content = infile.read()
             return content
 
 
@@ -76,6 +74,33 @@ def post_mockreturn(self, method, url, data, timeout, **kwargs):
     return response
 
 
+def test_simbad_mirror():
+    simbad_instance = simbad.SimbadClass()
+    # default value should be set at instantiation
+    assert simbad_instance.server == "simbad.cds.unistra.fr"
+    # it can be switched to harvard mirror
+    simbad_instance.server = "simbad.harvard.edu"
+    assert simbad_instance.server == "simbad.harvard.edu"
+    # but not to an invalid mirror
+    with pytest.raises(ValueError,
+                       match="'test' does not correspond to a Simbad server, *"):
+        simbad_instance.server = "test"
+
+
+def test_simbad_create_tap_service():
+    simbad_instance = simbad.Simbad()
+    # newly created should have no tap service
+    assert simbad_instance._tap is None
+    # then we create it
+    simbadtap = simbad_instance.tap
+    assert 'simbad/sim-tap' in simbadtap.baseurl
+
+
+def test_adql_parameter():
+    # escape single quotes
+    assert simbad.core._adql_parameter("Barnard's galaxy") == "Barnard''s galaxy"
+
+
 @pytest.mark.parametrize(('radius', 'expected_radius'),
                          [('5d0m0s', '5.0d'),
                           ('5d', '5.0d'),
@@ -124,15 +149,12 @@ def test_parse_result():
     result1 = sb._parse_result(
         MockResponseSimbad('query id '), simbad.core.SimbadVOTableResult)
     assert isinstance(result1, Table)
-    with pytest.raises(TableParseError) as ex:
+    expected_exception = 'Failed to parse SIMBAD result! The raw response can be found in self.last_response, '
+
+    with pytest.raises(TableParseError, match=expected_exception):
         sb._parse_result(MockResponseSimbad('query error '),
-                simbad.core.SimbadVOTableResult)
-    assert str(ex.value) == ('Failed to parse SIMBAD result! The raw response '
-                             'can be found in self.last_response, and the '
-                             'error in self.last_table_parse_error. '
-                             'The attempted parsed result is in '
-                             'self.last_parsed_result.\n Exception: 7:115: '
-                             'no element found')
+                         simbad.core.SimbadVOTableResult)
+
     assert isinstance(sb.last_response.text, str)
     assert isinstance(sb.last_response.content, bytes)
 
@@ -238,11 +260,12 @@ def test_query_catalog(patch_post):
 
 
 @pytest.mark.parametrize(('coordinates', 'radius', 'equinox', 'epoch'),
-                         [(ICRS_COORDS, None, 2000.0, 'J2000'),
+                         [(ICRS_COORDS, 2*u.arcmin, 2000.0, 'J2000'),
                           (GALACTIC_COORDS, 5 * u.deg, 2000.0, 'J2000'),
                           (FK4_COORDS, '5d0m0s', 2000.0, 'J2000'),
-                          (FK5_COORDS, None, 2000.0, 'J2000'),
+                          (FK5_COORDS, 2*u.arcmin, 2000.0, 'J2000'),
                           (multicoords, 0.5*u.arcsec, 2000.0, 'J2000'),
+                          (multicoords, "0.5s", 2000.0, 'J2000'),
                           ])
 def test_query_region_async(patch_post, coordinates, radius, equinox, epoch):
     response1 = simbad.core.Simbad.query_region_async(
@@ -256,10 +279,10 @@ def test_query_region_async(patch_post, coordinates, radius, equinox, epoch):
 
 
 @pytest.mark.parametrize(('coordinates', 'radius', 'equinox', 'epoch'),
-                         [(ICRS_COORDS, None, 2000.0, 'J2000'),
+                         [(ICRS_COORDS, 2*u.arcmin, 2000.0, 'J2000'),
                           (GALACTIC_COORDS, 5 * u.deg, 2000.0, 'J2000'),
                           (FK4_COORDS, '5d0m0s', 2000.0, 'J2000'),
-                          (FK5_COORDS, None, 2000.0, 'J2000')
+                          (FK5_COORDS, 2*u.arcmin, 2000.0, 'J2000')
                           ])
 def test_query_region(patch_post, coordinates, radius, equinox, epoch):
     result1 = simbad.core.Simbad.query_region(coordinates, radius=radius,
@@ -283,6 +306,11 @@ def test_query_region_radius_error(patch_post, coordinates, radius,
             coordinates, radius=radius, equinox=equinox, epoch=epoch)
 
 
+def test_query_region_coord_radius_mismatch():
+    with pytest.raises(ValueError, match="^Mismatch between radii and coordinates$"):
+        simbad.SimbadClass().query_region(multicoords, radius=[1, 2, 3] * u.deg)
+
+
 @pytest.mark.parametrize(('coordinates', 'radius', 'equinox', 'epoch'),
                          [(ICRS_COORDS, "0d", 2000.0, 'J2000'),
                           (GALACTIC_COORDS, 1.0 * u.marcsec, 2000.0, 'J2000')
@@ -317,8 +345,8 @@ def test_query_object_async(patch_post, object_name, wildcard):
 def test_query_object(patch_post, object_name, wildcard):
     payload = simbad.core.Simbad.query_object(
         object_name, wildcard=wildcard, get_query_payload=True)
-    expected_payload = {'script': '\nvotable {main_id,coordinates}\nvotable' +
-                        ' open\nquery id {} {}  \nvotable close'.
+    expected_payload = {'script': '\nvotable {main_id,coordinates}\nvotable'
+                        + ' open\nquery id {} {}  \nvotable close'.
                         format('wildcard' if wildcard else '', object_name)}
     assert payload == expected_payload
     result1 = simbad.core.Simbad.query_object(object_name,
@@ -344,25 +372,25 @@ def test_get_field_description():
 def test_votable_fields():
     sb = simbad.core.Simbad()
     sb.add_votable_fields('rot', 'z_value', 'velocity')
-    assert (set(sb.get_votable_fields()) ==
-            set(['main_id', 'coordinates', 'rot', 'z_value', 'velocity']))
-    try:
-        sb.add_votable_fields('velocity')
-    except KeyError:
-        pass  # this is the expected response
-    assert (set(sb.get_votable_fields()) ==
-            set(['main_id', 'coordinates', 'rot', 'z_value', 'velocity']))
+    assert (set(sb.get_votable_fields())
+            == set(['main_id', 'coordinates', 'rot', 'z_value', 'velocity']))
+
+    assert (set(sb.get_votable_fields())
+            == set(['main_id', 'coordinates', 'rot', 'z_value', 'velocity']))
     sb.remove_votable_fields('rot', 'main_id', 'coordinates')
     assert set(sb.get_votable_fields()) == set(['z_value', 'velocity'])
-    sb.remove_votable_fields('rot', 'main_id', 'coordinates')
+    # Warning is expected as we removed the 'coordinates' field above:
+    with pytest.warns(UserWarning, match="coordinates: this field is not set"):
+        sb.remove_votable_fields('coordinates')
     assert set(sb.get_votable_fields()) == set(['z_value', 'velocity'])
-    sb.remove_votable_fields('z_value', 'velocity')
-    assert (set(sb.get_votable_fields()) ==
-            set(['main_id', 'coordinates']))
+    with pytest.warns(UserWarning, match="All fields have been removed. Resetting"):
+        sb.remove_votable_fields('z_value', 'velocity')
+    assert set(sb.get_votable_fields()) == set(['main_id', 'coordinates'])
     sb.add_votable_fields('rot', 'z_value', 'velocity')
+    assert (set(sb.get_votable_fields())
+            == set(['main_id', 'coordinates', 'rot', 'z_value', 'velocity']))
     sb.reset_votable_fields()
-    assert (set(sb.get_votable_fields()) ==
-            set(['main_id', 'coordinates']))
+    assert set(sb.get_votable_fields()) == set(['main_id', 'coordinates'])
 
 
 def test_query_criteria1(patch_post):
@@ -387,9 +415,9 @@ def test_simbad_settings1():
     sb = simbad.core.Simbad()
     assert sb.get_votable_fields() == ['main_id', 'coordinates']
     sb.add_votable_fields('ra', 'dec(5)')
-    sb.remove_votable_fields('ra', 'dec')
-    assert (sb.get_votable_fields() ==
-            ['main_id', 'coordinates', 'dec(5)'])
+    with pytest.warns(UserWarning, match="dec: this field is not set"):
+        sb.remove_votable_fields('ra', 'dec')
+    assert sb.get_votable_fields() == ['main_id', 'coordinates', 'dec(5)']
     sb.reset_votable_fields()
 
 
@@ -420,9 +448,9 @@ def test_regression_votablesettings2():
     sb.add_votable_fields('fluxdata(J)')
     sb.add_votable_fields('fluxdata(H)')
     sb.add_votable_fields('fluxdata(K)')
-    assert (sb.get_votable_fields() ==
-            ['main_id', 'coordinates',
-             'fluxdata(J)', 'fluxdata(H)', 'fluxdata(K)'])
+    assert (sb.get_votable_fields()
+            == ['main_id', 'coordinates',
+                'fluxdata(J)', 'fluxdata(H)', 'fluxdata(K)'])
     sb.remove_votable_fields('fluxdata', strip_params=True)
     assert sb.get_votable_fields() == ['main_id', 'coordinates']
 
@@ -435,6 +463,45 @@ def test_regression_issue388():
         response.content = f.read()
     parsed_table = simbad.Simbad._parse_result(response,
                                                simbad.core.SimbadVOTableResult)
-    truth = b'M   1' if commons.ASTROPY_LT_4_1 else 'M   1'
+    truth = 'M   1'
     assert parsed_table['MAIN_ID'][0] == truth
     assert len(parsed_table) == 1
+
+# ---------------------------------------------------
+# Test the adql string for query_tap helper functions
+# ---------------------------------------------------
+
+
+def test_simbad_list_tables():
+    tables_adql = "SELECT table_name, description FROM TAP_SCHEMA.tables WHERE schema_name = 'public'"
+    assert simbad.Simbad.list_tables(get_adql=True) == tables_adql
+
+
+def test_simbad_list_columns():
+    # with three table names
+    columns_adql = ("SELECT table_name, column_name, datatype, description, unit, ucd"
+                    " FROM TAP_SCHEMA.columns "
+                    "WHERE table_name NOT LIKE 'TAP_SCHEMA.%'"
+                    " AND table_name IN ('mesPM', 'otypedef', 'journals')"
+                    " ORDER BY table_name, principal DESC, column_name")
+    assert simbad.Simbad.list_columns("mesPM", "otypedef", "journals", get_adql=True) == columns_adql
+    # with only one
+    columns_adql = ("SELECT table_name, column_name, datatype, description, unit, ucd "
+                    "FROM TAP_SCHEMA.columns WHERE table_name NOT LIKE 'TAP_SCHEMA.%' "
+                    "AND table_name = 'basic' ORDER BY table_name, principal DESC, column_name")
+    assert simbad.Simbad.list_columns("basic", get_adql=True) == columns_adql
+    # with only a keyword
+    list_columns_adql = ("SELECT table_name, column_name, datatype, description, unit, ucd "
+                         "FROM TAP_SCHEMA.columns WHERE table_name NOT LIKE 'TAP_SCHEMA.%' "
+                         "AND ( (LOWERCASE(column_name) "
+                         "LIKE LOWERCASE('%stellar%')) OR (LOWERCASE(description) "
+                         "LIKE LOWERCASE('%stellar%')) OR (LOWERCASE(table_name) "
+                         "LIKE LOWERCASE('%stellar%'))) ORDER BY table_name, principal DESC, column_name")
+    assert simbad.Simbad.list_columns(keyword="stellar", get_adql=True) == list_columns_adql
+
+
+def test_list_linked_tables():
+    list_linked_tables_adql = ("SELECT from_table, from_column, target_table, target_column "
+                               "FROM TAP_SCHEMA.key_columns JOIN TAP_SCHEMA.keys USING (key_id) "
+                               "WHERE (from_table = 'basic') OR (target_table = 'basic')")
+    assert simbad.Simbad.list_linked_tables("basic", get_adql=True) == list_linked_tables_adql
