@@ -10,10 +10,18 @@ from astroquery.utils.mocks import MockResponse
 from astroquery.simbad import Simbad
 # Maybe we need to expose SimbadVOTableResult to be in the public API?
 from astroquery.simbad.core import SimbadVOTableResult
+from astroquery.exceptions import BlankResponseWarning
+from packaging import version
+from pyvo import __version__ as pyvo_version
+try:
+    # This requires pyvo 1.4
+    from pyvo.dal.exceptions import DALOverflowWarning
+except ImportError:
+    pass
 
 
 # M42 coordinates
-ICRS_COORDS_M42 = SkyCoord("05h35m17.3s -05h23m28s", frame='icrs')
+ICRS_COORDS_M42 = SkyCoord("05h35m17.3s -05d23m28s", frame='icrs')
 ICRS_COORDS_SgrB2 = SkyCoord(266.835*u.deg, -28.38528*u.deg, frame='icrs')
 multicoords = SkyCoord([ICRS_COORDS_M42, ICRS_COORDS_SgrB2])
 
@@ -99,24 +107,26 @@ class TestSimbad:
 
     def test_query_region_async(self, temp_dir):
         simbad = Simbad()
-        # TODO: rewise once ROW_LIMIT is working
-        simbad.TIMEOUT = 100
+        simbad.ROW_LIMIT = 100
         simbad.cache_location = temp_dir
         response = simbad.query_region_async(
-            ICRS_COORDS_M42, radius=2 * u.deg, equinox=2000.0, epoch='J2000')
+            ICRS_COORDS_M42, radius=5 * u.arcsec, equinox=2000.0, epoch='J2000')
+        # A correct response code
+        assert response.status_code == 200
+        # Check that Orion was found
+        assert "NAME Ori Region" in response.text
 
-        assert response is not None
-
-    def test_query_region_async_vector(self, temp_dir):
+    @pytest.mark.parametrize("radius", (0.5 * u.arcsec, "0.5s"))
+    def test_query_region_async_vector(self, temp_dir, radius):
         simbad = Simbad()
         simbad.cache_location = temp_dir
-        response1 = simbad.query_region_async(multicoords, radius=0.5*u.arcsec)
-        assert response1.request.body == 'script=votable+%7Bmain_id%2Ccoordinates%7D%0Avotable+open%0Aquery+coo+5%3A35%3A17.3+-80%3A52%3A00+radius%3D0.5s+frame%3DICRS+equi%3D2000.0%0Aquery+coo+17%3A47%3A20.4+-28%3A23%3A07.008+radius%3D0.5s+frame%3DICRS+equi%3D2000.0%0Avotable+close'   # noqa
+        response1 = simbad.query_region_async(multicoords, radius=radius)
+        assert response1.request.body == 'script=votable+%7Bmain_id%2Ccoordinates%7D%0Avotable+open%0Aquery+coo+5%3A35%3A17.3+-5%3A23%3A28+radius%3D0.5s+frame%3DICRS+equi%3D2000.0%0Aquery+coo+17%3A47%3A20.4+-28%3A23%3A07.008+radius%3D0.5s+frame%3DICRS+equi%3D2000.0%0Avotable+close'   # noqa
 
     def test_query_region(self, temp_dir):
         simbad = Simbad()
-        # TODO: rewise once ROW_LIMIT is working
         simbad.TIMEOUT = 100
+        simbad.ROW_LIMIT = 100
         simbad.cache_location = temp_dir
         result = simbad.query_region(ICRS_COORDS_M42, radius=2 * u.deg,
                                      equinox=2000.0, epoch='J2000')
@@ -148,7 +158,8 @@ class TestSimbad:
         assert len(result) == 2
         assert len(result.errors) == 0
 
-        result = simbad.query_objects(['M32', 'M81', 'gHer'])
+        with pytest.warns(BlankResponseWarning):
+            result = simbad.query_objects(['M32', 'M81', 'gHer'])
         # 'gHer' is not a valid Simbad identifier - it should be 'g Her' to
         # get the star
         assert len(result) == 2
@@ -181,29 +192,25 @@ class TestSimbad:
     def test_null_response(self, temp_dir, function):
         simbad = Simbad()
         simbad.cache_location = temp_dir
-        assert (simbad.__getattribute__(function)('idonotexist')
-                is None)
+        with pytest.warns(BlankResponseWarning):
+            assert (simbad.__getattribute__(function)('idonotexist')
+                    is None)
 
     # Special case of null test: list of nonexistent parameters
     def test_query_objects_null(self, temp_dir):
         simbad = Simbad()
         simbad.cache_location = temp_dir
-        assert simbad.query_objects(['idonotexist', 'idonotexisteither']) is None
+        with pytest.warns(BlankResponseWarning):
+            assert simbad.query_objects(['idonotexist', 'idonotexisteither']) is None
 
-    # Special case of null test: zero-sized region
-    def test_query_region_null(self, temp_dir):
+    # Special case of null test: zero-size and very small region
+    @pytest.mark.parametrize('radius', ["0d", 1.0*u.marcsec])
+    def test_query_region_null(self, temp_dir, radius):
         simbad = Simbad()
         simbad.cache_location = temp_dir
-        result = simbad.query_region(SkyCoord("00h01m0.0s 00h00m0.0s"), radius="0d",
-                                     equinox=2000.0, epoch='J2000')
-        assert result is None
-
-    # Special case of null test: very small region
-    def test_query_small_region_null(self, temp_dir):
-        simbad = Simbad()
-        simbad.cache_location = temp_dir
-        result = simbad.query_region(SkyCoord("00h01m0.0s 00h00m0.0s"), radius=1.0 * u.marcsec,
-                                     equinox=2000.0, epoch='J2000')
+        with pytest.warns(BlankResponseWarning):
+            result = simbad.query_region(SkyCoord("00h01m0.0s 00h00m0.0s"), radius=1.0 * u.marcsec,
+                                         equinox=2000.0, epoch='J2000')
         assert result is None
 
     # Special case : zero-sized region with one object
@@ -220,7 +227,7 @@ class TestSimbad:
         request = Simbad()
         request.add_votable_fields("flux_qual(V)")
         response = request.query_object('algol')
-        assert("FLUX_QUAL_V" in response.keys())
+        assert ("FLUX_QUAL_V" in response.keys())
 
     def test_multi_vo_fields(self):
         '''Regression test for issue 820'''
@@ -241,14 +248,55 @@ class TestSimbad:
         request.add_votable_fields("id(s)")
 
         response = request.query_object('algol')
-        assert("FLUX_QUAL_V" in response.keys())
-        assert("FLUX_QUAL_R" in response.keys())
-        assert("RA_d" in response.keys())
-        assert("RA_s" in response.keys())
-        assert("RA___A_ICRS_J2000" in response.keys())
-        assert("RA___A_fk5_J2000" in response.keys())
-        assert("OTYPE_S" in response.keys())
-        assert("OTYPE_3" in response.keys())
-        assert("ID_1" in response.keys())
-        assert("ID_2mass" in response.keys())
-        assert("ID_s" in response.keys())
+        assert ("FLUX_QUAL_V" in response.keys())
+        assert ("FLUX_QUAL_R" in response.keys())
+        assert ("RA_d" in response.keys())
+        assert ("RA_s" in response.keys())
+        assert ("RA___A_ICRS_J2000" in response.keys())
+        assert ("RA___A_fk5_J2000" in response.keys())
+        assert ("OTYPE_S" in response.keys())
+        assert ("OTYPE_3" in response.keys())
+        assert ("ID_1" in response.keys())
+        assert ("ID_2mass" in response.keys())
+        assert ("ID_s" in response.keys())
+
+    def test_query_tap(self):
+        # a robust query about something that should not change in Simbad
+        filtername = Simbad.query_tap("select filtername from filter where filtername='B'")
+        assert 'B' == filtername["filtername"][0]
+        # test uploads by joining two local tables
+        table_letters = Table([["a", "b", "c"]], names=["letters"])
+        table_numbers = Table([[1, 2, 3], ["a", "b", "c"]], names=["numbers", "letters"])
+        result = Simbad.query_tap("SELECT * FROM TAP_UPLOAD.numbers "
+                                  "JOIN TAP_UPLOAD.letters USING(letters)",
+                                  numbers=table_numbers, letters=table_letters)
+        expect = "letters numbers\n------- -------\n      a       1\n      b       2\n      c       3"
+        assert expect == str(result)
+        # Test query_tap raised errors
+        # DALOverflowWarning exists since pyvo 1.4
+        if version.parse(pyvo_version) > version.parse('1.4'):
+            with pytest.raises(DALOverflowWarning, match="Partial result set *"):
+                truncated_result = Simbad.query_tap("SELECT * from basic", maxrec=2)
+                assert len(truncated_result) == 2
+        with pytest.raises(ValueError, match="The maximum number of records cannot exceed 2000000."):
+            Simbad.query_tap("select top 5 * from basic", maxrec=10e10)
+        with pytest.raises(ValueError, match="Query string contains an odd number of single quotes.*"):
+            Simbad.query_tap("'''")
+
+    def test_simbad_list_tables(self):
+        tables = Simbad.list_tables()
+        # check the content
+        assert "basic" in str(tables)
+        # there might be new tables, we have 30 now.
+        assert len(tables) >= 30
+
+    def test_simbad_list_columns(self):
+        columns = Simbad.list_columns("ident", "biblio")
+        assert len(columns) == 4
+        assert "oidref" in str(columns)
+        columns = Simbad.list_columns(keyword="herschel")
+        assert {"mesHerschel"} == set(columns["table_name"])
+
+    def test_list_linked_tables(self):
+        links = Simbad.list_linked_tables("h_link")
+        assert {"basic"} == set(links["target_table"])
