@@ -3,16 +3,15 @@ import os.path
 import types
 
 import pytest
-from astropy import coordinates
+from astropy.coordinates import SkyCoord
+from astropy.coordinates.name_resolve import NameResolveError
 from astropy import units as u
 
-from ...utils import commons
 from astroquery.utils.mocks import MockResponse
 from ...skyview import SkyView
 
-objcoords = {'Eta Carinae': coordinates.SkyCoord(ra=161.264775 * u.deg,
-                                                 dec=-59.6844306 * u.deg,
-                                                 frame='icrs'), }
+objcoords = {"Eta Carinae": SkyCoord(ra=161.264775 * u.deg, dec=-59.6844306 * u.deg,
+                                     frame="icrs")}
 
 
 @pytest.fixture
@@ -23,19 +22,17 @@ def patch_fromname(request):
     except AttributeError:  # pytest < 3
         mp = request.getfuncargvalue("monkeypatch")
 
-    def fromname(self, name):
+    def fromname(self, name, frame=None):
         if isinstance(name, str):
             return objcoords[name]
         else:
-            raise coordinates.name_resolve.NameResolveError
-    mp.setattr(commons.ICRSCoord,
-               'from_name',
-               types.MethodType(fromname, commons.ICRSCoord))
+            raise NameResolveError
+    mp.setattr(SkyCoord, "from_name", types.MethodType(fromname, SkyCoord))
 
 
 class MockResponseSkyView(MockResponse):
     def __init__(self):
-        super(MockResponseSkyView, self).__init__()
+        super().__init__()
 
     def get_content(self):
         return self.content
@@ -43,7 +40,7 @@ class MockResponseSkyView(MockResponse):
 
 class MockResponseSkyviewForm(MockResponse):
     def __init__(self, method, url, cache=False, params=None, **kwargs):
-        super(MockResponseSkyviewForm, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.content = self.get_content(method, url)
 
     def get_content(self, method, url):
@@ -86,3 +83,20 @@ def test_survey_validation(patch_get):
     assert str(ex.value) == ("Survey is not among the surveys hosted "
                              "at skyview.  See list_surveys or "
                              "survey_dict for valid surveys.")
+
+
+def test_get_image_list_size(patch_get, patch_fromname):
+    # Test with Quantities (as expected)
+    SkyView.get_image_list(position='Eta Carinae',
+                           survey='DSS',
+                           width=1 * u.deg, height=1 * u.deg)
+    with pytest.raises(u.UnitConversionError):
+        # Test with invalid Quantities
+        SkyView.get_image_list(position='Eta Carinae',
+                               survey='DSS',
+                               width=1, height='1 meter')
+    with pytest.raises(ValueError):
+        # Test with incomplete input
+        SkyView.get_image_list(position='Eta Carinae',
+                               survey='DSS',
+                               width=1 * u.deg, height=None)
