@@ -16,8 +16,6 @@ Created on 30 jun. 2016
 """
 from astroquery.utils.tap import taputils
 
-from urllib.parse import urlencode
-
 import requests
 
 
@@ -29,8 +27,6 @@ class DummyConnHandler:
         self.fileExt = ".ext"
         self.defaultResponse = None
         self.responses = {}
-        self.errorFileOutput = None
-        self.errorReceivedResponse = None
         self.contentType = None
         self.verbose = None
         self.query = None
@@ -42,21 +38,6 @@ class DummyConnHandler:
     def get_default_response(self):
         return self.defaultResponse
 
-    def get_last_request(self):
-        return self.request
-
-    def get_last_data(self):
-        return self.data
-
-    def get_last_query(self):
-        return self.query
-
-    def get_error_file_output(self):
-        return self.errorFileOutput
-
-    def get_error_received_response(self):
-        return self.errorReceivedResponse
-
     def set_response(self, request, response):
         self.responses[str(request)] = response
 
@@ -102,8 +83,6 @@ class DummyConnHandler:
         return self.__get_response(self.request)
 
     def dump_to_file(self, fileOutput, response):
-        self.errorFileOutput = fileOutput
-        self.errorReceivedResponse = response
         print(f"DummyConnHandler - dump to file: file: '{fileOutput}', \
         response status: {response.status}, response msg: {response.reason}")
 
@@ -147,15 +126,9 @@ class DummyConnHandler:
         else:
             return isError
 
-    def url_encode(self, data):
-        return urlencode(data)
-
     def get_suitable_extension(self, headers):
         return self.fileExt
 
-    def set_suitable_extension(self, ext):
-        self.fileExt = ext
-
     def get_suitable_extension_by_format(self, output_format):
         return self.fileExt
 
@@ -176,3 +149,6 @@ class DummyConnHandler:
                            verbose=False):
         return self.__execute_post(subcontext="TableTool", data=data,
                                    content_type=content_type, verbose=verbose)
+
+    def execute_secure(self, subcontext=None, data=None, verbose=False):
+        return self.__execute_post(subcontext=subcontext, data=data, verbose=verbose)
