@@ -16,12 +16,14 @@ Created on 30 jun. 2016
 """
 
 import time
+from urllib.parse import urlencode
 
 from astroquery.utils.tap.model import modelutils
 from astroquery.utils.tap.xmlparser import utils
 from astroquery.utils.tap import taputils
+from astropy.logger import log
 import requests
-import sys
+
 
 __all__ = ['Job']
 
@@ -30,7 +32,7 @@ class Job:
     """Job class
     """
 
-    def __init__(self, async_job, query=None, connhandler=None):
+    def __init__(self, async_job, *, query=None, connhandler=None):
         """Constructor
 
         Parameters
@@ -86,7 +88,7 @@ class Job:
             raise ValueError("Cannot assign a phase when a job is finished")
         self._phase = phase
 
-    def start(self, verbose=False):
+    def start(self, *, verbose=False):
         """Starts the job (allowed in PENDING phase only)
 
         Parameters
@@ -96,7 +98,7 @@ class Job:
         """
         self.__change_phase(phase="RUN", verbose=verbose)
 
-    def abort(self, verbose=False):
+    def abort(self, *, verbose=False):
         """Aborts the job (allowed in PENDING phase only)
 
         Parameters
@@ -106,15 +108,12 @@ class Job:
         """
         self.__change_phase(phase="ABORT", verbose=verbose)
 
-    def __change_phase(self, phase, verbose=False):
+    def __change_phase(self, phase, *, verbose=False):
         if self._phase == 'PENDING':
             context = f"async/{self.jobid}/phase"
-            args = {
-                "PHASE": str(phase)}
-            data = self.connHandler.url_encode(args)
-            response = self.connHandler.execute_tappost(subcontext=context,
-                                                        data=data,
-                                                        verbose=verbose)
+            response = self.connHandler.execute_tappost(
+                subcontext=context, data=urlencode({"PHASE": phase}), verbose=verbose
+            )
             if verbose:
                 print(response.status, response.reason)
                 print(response.getheaders())
@@ -136,7 +135,7 @@ class Job:
         else:
             raise ValueError(f"Cannot start a job in phase: {self._phase}")
 
-    def send_parameter(self, name=None, value=None, verbose=False):
+    def send_parameter(self, *, name=None, value=None, verbose=False):
         """Sends a job parameter (allowed in PENDING phase only).
 
         Parameters
@@ -149,11 +148,8 @@ class Job:
         if self._phase == 'PENDING':
             # send post parameter/value
             context = f"async/{self.jobid}"
-            args = {
-                name: str(value)}
-            data = self.connHandler.url_encode(args)
             response = self.connHandler.execute_tappost(subcontext=context,
-                                                        data=data,
+                                                        data=urlencode({name: value}),
                                                         verbose=verbose)
             if verbose:
                 print(response.status, response.reason)
@@ -167,7 +163,7 @@ class Job:
         else:
             raise ValueError(f"Cannot start a job in phase: {self._phase}")
 
-    def get_phase(self, update=False):
+    def get_phase(self, *, update=False):
         """Returns the job phase. May optionally update the job's phase.
 
         Parameters
@@ -257,7 +253,7 @@ class Job:
         self.results = results
         self.__resultInMemory = True
 
-    def save_results(self, verbose=False):
+    def save_results(self, *, verbose=False):
         """Saves job results
         If the job is asynchronous, this method will block until the results
         are available.
@@ -274,10 +270,10 @@ class Job:
         else:
             if not self.async_:
                 # sync: cannot access server again
-                print("No results to save")
+                log.info("No results to save")
             else:
                 # Async
-                self.wait_for_job_end(verbose)
+                self.wait_for_job_end(verbose=verbose)
                 response = self.connHandler.execute_tapget(
                     f"async/{self.jobid}/results/result")
                 if verbose:
@@ -303,7 +299,7 @@ class Job:
                     print(f"Saving results to: {output}")
                 self.connHandler.dump_to_file(output, response)
 
-    def wait_for_job_end(self, verbose=False):
+    def wait_for_job_end(self, *, verbose=False):
         """Waits until a job is finished
 
         Parameters
@@ -316,9 +312,9 @@ class Job:
         lphase = None
         # execute job if not running
         if self._phase == 'PENDING':
-            print("Job in PENDING phase, sending phase=RUN request.")
+            log.info("Job in PENDING phase, sending phase=RUN request.")
             try:
-                self.start(verbose)
+                self.start(verbose=verbose)
             except Exception as ex:
                 # ignore
                 if verbose:
@@ -330,15 +326,14 @@ class Job:
             lphase = responseData.upper().strip()
             if verbose:
                 print(f"Job {self.jobid} status: {lphase}")
-            if ("PENDING" != lphase and "QUEUED" != lphase and
-                    "EXECUTING" != lphase):
+            if ("PENDING" != lphase and "QUEUED" != lphase and "EXECUTING" != lphase):
                 break
             # PENDING, QUEUED, EXECUTING, COMPLETED, ERROR, ABORTED, UNKNOWN,
             # HELD, SUSPENDED, ARCHIVED:
             time.sleep(0.5)
         return currentResponse, lphase
 
-    def __load_async_job_results(self, debug=False):
+    def __load_async_job_results(self, *, debug=False):
         wjResponse, phase = self.wait_for_job_end()
         subContext = f"async/{self.jobid}/results/result"
         resultsResponse = self.connHandler.execute_tapget(subContext)
@@ -348,14 +343,14 @@ class Job:
             print(resultsResponse.getheaders())
 
         resultsResponse = self.__handle_redirect_if_required(resultsResponse,
-                                                             debug)
+                                                             verbose=debug)
         isError = self.connHandler.\
             check_launch_response_status(resultsResponse,
                                          debug,
                                          200)
         self._phase = phase
         if phase == 'ERROR':
-            errMsg = self.get_error(debug)
+            errMsg = self.get_error(verbose=debug)
             raise SystemError(errMsg)
         else:
             if isError:
@@ -368,12 +363,10 @@ class Job:
                                                    outputFormat)
                 self.set_results(results)
 
-    def __handle_redirect_if_required(self, resultsResponse, verbose=False):
+    def __handle_redirect_if_required(self, resultsResponse, *, verbose=False):
         # Thanks @emeraldTree24
         numberOfRedirects = 0
-        while ((resultsResponse.status == 303 or
-                resultsResponse.status == 302) and
-                numberOfRedirects < 20):
+        while ((resultsResponse.status == 303 or resultsResponse.status == 302) and numberOfRedirects < 20):
             joblocation = self.connHandler.\
                 find_header(resultsResponse.getheaders(), "location")
             if verbose:
@@ -385,7 +378,7 @@ class Job:
                 print(resultsResponse.getheaders())
         return resultsResponse
 
-    def get_error(self, verbose=False):
+    def get_error(self, *, verbose=False):
         """Returns the error associated to a job
 
         Parameters
@@ -403,9 +396,7 @@ class Job:
         if verbose:
             print(resultsResponse.status, resultsResponse.reason)
             print(resultsResponse.getheaders())
-        if (resultsResponse.status != 200 and
-                resultsResponse.status != 303 and
-                resultsResponse.status != 302):
+        if (resultsResponse.status != 200 and resultsResponse.status != 303 and resultsResponse.status != 302):
             errMsg = taputils.get_http_response_error(resultsResponse)
             print(resultsResponse.status, errMsg)
             raise requests.exceptions.HTTPError(errMsg)
@@ -415,19 +406,16 @@ class Job:
                 location = self.connHandler.\
                     find_header(resultsResponse.getheaders(), "location")
                 if location is None:
-                    raise requests.exceptions.HTTPError("No location found " +
-                                                        "after redirection " +
-                                                        "was received (303)")
+                    raise requests.exceptions.HTTPError("No location found after redirection was received (303)")
                 if verbose:
                     print(f"Redirect to {location}")
                 # load
-                relativeLocation = self.__extract_relative_location(location,
-                                                                    self.jobid)
+                relativeLocation = self.__extract_relative_location(location, self.jobid)
                 relativeLocationSubContext = f"async/{self.jobid}/{relativeLocation}"
                 response = self.connHandler.\
                     execute_tapget(relativeLocationSubContext)
                 response = self.__handle_redirect_if_required(response,
-                                                              verbose)
+                                                              verbose=verbose)
                 isError = self.connHandler.\
                     check_launch_response_status(response, verbose, 200)
                 if isError:
@@ -443,9 +431,7 @@ class Job:
         """Returns whether the job is finished (ERROR, ABORTED, COMPLETED) or not
 
         """
-        if (self._phase == 'ERROR' or
-                self._phase == 'ABORTED' or
-                self._phase == 'COMPLETED'):
+        if (self._phase == 'ERROR' or self._phase == 'ABORTED' or self._phase == 'COMPLETED'):
             return True
         else:
             return False
