@@ -12,9 +12,11 @@ European Space Agency (ESA)
 
 Created on 30 jun. 2016
 """
-import os
+from pathlib import Path
 from unittest.mock import patch
+from urllib.parse import quote_plus, urlencode
 
+import gzip
 import numpy as np
 import pytest
 from requests import HTTPError
@@ -23,35 +25,32 @@ from astroquery.utils.tap.model.tapcolumn import TapColumn
 
 from astroquery.utils.tap.conn.tests.DummyConnHandler import DummyConnHandler
 from astroquery.utils.tap.conn.tests.DummyResponse import DummyResponse
-from astroquery.utils.tap.core import TapPlus, TAP_CLIENT_ID
-from astroquery.utils.tap.xmlparser import utils
+from astroquery.utils.tap.core import TapPlus
 from astroquery.utils.tap import taputils
 
 
-def data_path(filename):
-    data_dir = os.path.join(os.path.dirname(__file__), 'data')
-    return os.path.join(data_dir, filename)
+def read_file(filename):
+    if filename.name.endswith('.gz'):
+        with gzip.open(filename, 'rb') as file:
+            return file.read()
+    else:
+        return filename.read_text()
+
+
+TEST_DATA = {f.name: read_file(f) for f in Path(__file__).with_name("data").iterdir()}
 
 
 def test_load_tables():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    responseLoadTable = DummyResponse()
-    responseLoadTable.set_status_code(500)
-    responseLoadTable.set_message("ERROR")
-    tableDataFile = data_path('test_tables.xml')
-    tableData = utils.read_file_content(tableDataFile)
-    responseLoadTable.set_data(method='GET',
-                               context=None,
-                               body=tableData,
-                               headers=None)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    responseLoadTable = DummyResponse(500)
+    responseLoadTable.set_data(method='GET', body=TEST_DATA["test_tables.xml"])
     tableRequest = "tables"
     connHandler.set_response(tableRequest, responseLoadTable)
     with pytest.raises(Exception):
         tap.load_tables()
 
     responseLoadTable.set_status_code(200)
-    responseLoadTable.set_message("OK")
     res = tap.load_tables()
     assert len(res) == 2
 
@@ -80,69 +79,50 @@ def test_load_tables():
 
 def test_load_tables_parameters():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    responseLoadTable = DummyResponse()
-    responseLoadTable.set_status_code(200)
-    responseLoadTable.set_message("OK")
-    tableDataFile = data_path('test_tables.xml')
-    tableData = utils.read_file_content(tableDataFile)
-    responseLoadTable.set_data(method='GET',
-                               context=None,
-                               body=tableData,
-                               headers=None)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    responseLoadTable = DummyResponse(200)
+    responseLoadTable.set_data(method='GET', body=TEST_DATA["test_tables.xml"])
     tableRequest = "tables"
     connHandler.set_response(tableRequest, responseLoadTable)
 
     # empty request
     tap.load_tables()
-    request = connHandler.get_last_request()
-    assert request == tableRequest
+    assert connHandler.request == tableRequest
 
     # flag only_names=false & share_accessible=false: equals to
     # empty request
     tap.load_tables(only_names=False, include_shared_tables=False)
-    request = connHandler.get_last_request()
-    assert request == tableRequest
+    assert connHandler.request == tableRequest
 
     # flag only_names
     tableRequest = "tables?only_tables=true"
     connHandler.set_response(tableRequest, responseLoadTable)
     tap.load_tables(only_names=True)
-    request = connHandler.get_last_request()
-    assert request == tableRequest
+    assert connHandler.request == tableRequest
 
     # flag share_accessible=true
     tableRequest = "tables?share_accessible=true"
     connHandler.set_response(tableRequest, responseLoadTable)
     tap.load_tables(include_shared_tables=True)
-    request = connHandler.get_last_request()
-    assert request == tableRequest
+    assert connHandler.request == tableRequest
 
     # flag only_names=true & share_accessible=true
     tableRequest = "tables?only_tables=true&share_accessible=true"
     connHandler.set_response(tableRequest, responseLoadTable)
     tap.load_tables(only_names=True, include_shared_tables=True)
-    request = connHandler.get_last_request()
-    assert request == tableRequest
+    assert connHandler.request == tableRequest
 
 
 def test_load_table():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
 
     # No arguments
     with pytest.raises(Exception):
         tap.load_table()
 
-    responseLoadTable = DummyResponse()
-    responseLoadTable.set_status_code(500)
-    responseLoadTable.set_message("ERROR")
-    tableDataFile = data_path('test_table1.xml')
-    tableData = utils.read_file_content(tableDataFile)
-    responseLoadTable.set_data(method='GET',
-                               context=None,
-                               body=tableData,
-                               headers=None)
+    responseLoadTable = DummyResponse(500)
+    responseLoadTable.set_data(method='GET', body=TEST_DATA["test_table1.xml"])
     tableSchema = "public"
     tableName = "table1"
     fullQualifiedTableName = f"{tableSchema}.{tableName}"
@@ -153,7 +133,6 @@ def test_load_table():
         tap.load_table(fullQualifiedTableName)
 
     responseLoadTable.set_status_code(200)
-    responseLoadTable.set_message("OK")
     table = tap.load_table(fullQualifiedTableName)
     assert table is not None
     assert table.description == 'Table1 desc'
@@ -167,37 +146,80 @@ def test_load_table():
 
 def test_launch_sync_job():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    responseLaunchJob = DummyResponse()
-    responseLaunchJob.set_status_code(500)
-    responseLaunchJob.set_message("ERROR")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=jobData,
-                               headers=None)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    responseLaunchJob = DummyResponse(500)
+    responseLaunchJob.set_data(method='POST', body=TEST_DATA["job_1.vot"])
     query = 'select top 5 * from table'
-    dTmp = {"q": query}
-    dTmpEncoded = connHandler.url_encode(dTmp)
-    p = dTmpEncoded.find("=")
-    q = dTmpEncoded[p + 1:]
     dictTmp = {
         "REQUEST": "doQuery",
         "LANG": "ADQL",
         "FORMAT": "votable",
-        "tapclient": str(TAP_CLIENT_ID),
+        "tapclient": str(tap.tap_client_id),
         "PHASE": "RUN",
-        "QUERY": str(q)}
+        "QUERY": quote_plus(query)}
     sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
     jobRequest = f"sync?{sortedKey}"
     connHandler.set_response(jobRequest, responseLaunchJob)
 
     with pytest.raises(Exception):
-        tap.launch_job(query)
+        tap.launch_job(query, maxrec=10)
 
     responseLaunchJob.set_status_code(200)
-    responseLaunchJob.set_message("OK")
+
+    job = tap.launch_job(query)
+
+    assert job is not None
+    assert job.async_ is False
+    assert job.get_phase() == 'COMPLETED'
+    assert job.failed is False
+
+    # results
+    results = job.get_results()
+    assert len(results) == 3
+    __check_results_column(results,
+                           'ra',
+                           'ra',
+                           None,
+                           np.float64)
+    __check_results_column(results,
+                           'dec',
+                           'dec',
+                           None,
+                           np.float64)
+    __check_results_column(results,
+                           'source_id',
+                           'source_id',
+                           None,
+                           object)
+    __check_results_column(results,
+                           'table1_oid',
+                           'table1_oid',
+                           None,
+                           np.int32)
+
+
+def test_launch_sync_job_secure():
+    connHandler = DummyConnHandler()
+    tap = TapPlus(url="https://test:1111/tap", connhandler=connHandler)
+    responseLaunchJob = DummyResponse(500)
+    responseLaunchJob.set_data(method='POST', body=TEST_DATA["job_1.vot"])
+    query = 'select top 5 * from table'
+    dictTmp = {
+        "REQUEST": "doQuery",
+        "LANG": "ADQL",
+        "FORMAT": "votable",
+        "tapclient": str(tap.tap_client_id),
+        "PHASE": "RUN",
+        "QUERY": quote_plus(query)}
+    sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
+    jobRequest = f"sync?{sortedKey}"
+    connHandler.set_response(jobRequest, responseLaunchJob)
+
+    with pytest.raises(Exception):
+        tap.launch_job(query, maxrec=10)
+
+    responseLaunchJob.set_status_code(200)
+
     job = tap.launch_job(query)
 
     assert job is not None
@@ -232,45 +254,29 @@ def test_launch_sync_job():
 
 def test_launch_sync_job_redirect():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    responseLaunchJob = DummyResponse()
-    responseLaunchJob.set_status_code(500)
-    responseLaunchJob.set_message("ERROR")
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    responseLaunchJob = DummyResponse(500)
     jobid = '12345'
     resultsReq = f'sync/{jobid}'
     resultsLocation = f'http://test:1111/tap/{resultsReq}'
     launchResponseHeaders = [
         ['location', resultsLocation]
     ]
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=None,
-                               headers=None)
+    responseLaunchJob.set_data(method='POST')
     query = 'select top 5 * from table'
-    dTmp = {"q": query}
-    dTmpEncoded = connHandler.url_encode(dTmp)
-    p = dTmpEncoded.find("=")
-    q = dTmpEncoded[p + 1:]
     dictTmp = {
         "REQUEST": "doQuery",
         "LANG": "ADQL",
         "FORMAT": "votable",
-        "tapclient": str(TAP_CLIENT_ID),
+        "tapclient": str(tap.tap_client_id),
         "PHASE": "RUN",
-        "QUERY": str(q)}
+        "QUERY": quote_plus(query)}
     sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
     jobRequest = f"sync?{sortedKey}"
     connHandler.set_response(jobRequest, responseLaunchJob)
     # Results response
-    responseResultsJob = DummyResponse()
-    responseResultsJob.set_status_code(500)
-    responseResultsJob.set_message("ERROR")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseResultsJob.set_data(method='GET',
-                                context=None,
-                                body=jobData,
-                                headers=None)
+    responseResultsJob = DummyResponse(500)
+    responseResultsJob.set_data(method='GET', body=TEST_DATA["job_1.vot"])
     connHandler.set_response(resultsReq, responseResultsJob)
 
     with pytest.raises(Exception):
@@ -279,7 +285,6 @@ def test_launch_sync_job_redirect():
     # Response is redirect (303)
     # No location available
     responseLaunchJob.set_status_code(303)
-    responseLaunchJob.set_message("OK")
     with pytest.raises(Exception):
         tap.launch_job(query)
 
@@ -287,13 +292,8 @@ def test_launch_sync_job_redirect():
     # Location available
     # Results raises error (500)
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=None,
-                               headers=launchResponseHeaders)
+    responseLaunchJob.set_data(method='POST', headers=launchResponseHeaders)
     responseResultsJob.set_status_code(500)
-    responseResultsJob.set_message("ERROR")
     with pytest.raises(Exception):
         tap.launch_job(query)
 
@@ -301,7 +301,6 @@ def test_launch_sync_job_redirect():
     # Results is 200
     # Location available
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
     job = tap.launch_job(query)
     assert job is not None
     assert job.async_ is False
@@ -335,51 +334,34 @@ def test_launch_sync_job_redirect():
 
 def test_launch_async_job():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
     jobid = '12345'
     # Launch response
-    responseLaunchJob = DummyResponse()
-    responseLaunchJob.set_status_code(500)
-    responseLaunchJob.set_message("ERROR")
+    responseLaunchJob = DummyResponse(500)
     # list of list (httplib implementation for headers in response)
     launchResponseHeaders = [
         ['location', f'http://test:1111/tap/async/{jobid}']
     ]
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=None,
-                               headers=launchResponseHeaders)
+    responseLaunchJob.set_data(method='POST', headers=launchResponseHeaders)
     query = 'query'
     dictTmp = {
         "REQUEST": "doQuery",
         "LANG": "ADQL",
         "FORMAT": "votable",
-        "tapclient": str(TAP_CLIENT_ID),
+        "tapclient": str(tap.tap_client_id),
         "PHASE": "RUN",
         "QUERY": str(query)}
     sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
     req = f"async?{sortedKey}"
     connHandler.set_response(req, responseLaunchJob)
     # Phase response
-    responsePhase = DummyResponse()
-    responsePhase.set_status_code(500)
-    responsePhase.set_message("ERROR")
-    responsePhase.set_data(method='GET',
-                           context=None,
-                           body="COMPLETED",
-                           headers=None)
+    responsePhase = DummyResponse(500)
+    responsePhase.set_data(method='GET', body="COMPLETED")
     req = f"async/{jobid}/phase"
     connHandler.set_response(req, responsePhase)
     # Results response
-    responseResultsJob = DummyResponse()
-    responseResultsJob.set_status_code(500)
-    responseResultsJob.set_message("ERROR")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseResultsJob.set_data(method='GET',
-                                context=None,
-                                body=jobData,
-                                headers=None)
+    responseResultsJob = DummyResponse(500)
+    responseResultsJob.set_data(method='GET', body=TEST_DATA["job_1.vot"])
     req = f"async/{jobid}/results/result"
     connHandler.set_response(req, responseResultsJob)
 
@@ -387,17 +369,14 @@ def test_launch_async_job():
         tap.launch_job_async(query)
 
     responseLaunchJob.set_status_code(303)
-    responseLaunchJob.set_message("OK")
     with pytest.raises(Exception):
         tap.launch_job_async(query)
 
     responsePhase.set_status_code(200)
-    responsePhase.set_message("OK")
     with pytest.raises(Exception):
         tap.launch_job_async(query)
 
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
     job = tap.launch_job_async(query)
     assert job is not None
     assert job.async_ is True
@@ -431,65 +410,42 @@ def test_launch_async_job():
 
 def test_start_job():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
     jobid = '12345'
     # Phase POST response
-    responsePhase = DummyResponse()
-    responsePhase.set_status_code(200)
-    responsePhase.set_message("OK")
-    responsePhase.set_data(method='POST',
-                           context=None,
-                           body=None,
-                           headers=None)
+    responsePhase = DummyResponse(200)
+    responsePhase.set_data(method='POST')
     req = f"async/{jobid}/phase?PHASE=RUN"
     connHandler.set_response(req, responsePhase)
     # Launch response
-    responseLaunchJob = DummyResponse()
-    responseLaunchJob.set_status_code(303)
-    responseLaunchJob.set_message("OK")
+    responseLaunchJob = DummyResponse(303)
     # list of list (httplib implementation for headers in response)
     launchResponseHeaders = [
         ['location', f'http://test:1111/tap/async/{jobid}']
     ]
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=None,
-                               headers=launchResponseHeaders)
+    responseLaunchJob.set_data(method='POST', headers=launchResponseHeaders)
     query = 'query'
     dictTmp = {
         "REQUEST": "doQuery",
         "LANG": "ADQL",
         "FORMAT": "votable",
-        "tapclient": str(TAP_CLIENT_ID),
+        "tapclient": str(tap.tap_client_id),
         "QUERY": str(query)}
     sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
     req = f"async?{sortedKey}"
     connHandler.set_response(req, responseLaunchJob)
     # Phase response
-    responsePhase = DummyResponse()
-    responsePhase.set_status_code(200)
-    responsePhase.set_message("OK")
-    responsePhase.set_data(method='GET',
-                           context=None,
-                           body="COMPLETED",
-                           headers=None)
+    responsePhase = DummyResponse(200)
+    responsePhase.set_data(method='GET', body="COMPLETED")
     req = f"async/{jobid}/phase"
     connHandler.set_response(req, responsePhase)
     # Results response
-    responseResultsJob = DummyResponse()
-    responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseResultsJob.set_data(method='GET',
-                                context=None,
-                                body=jobData,
-                                headers=None)
+    responseResultsJob = DummyResponse(200)
+    responseResultsJob.set_data(method='GET', body=TEST_DATA["job_1.vot"])
     req = f"async/{jobid}/results/result"
     connHandler.set_response(req, responseResultsJob)
 
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
     job = tap.launch_job_async(query, autorun=False)
     assert job is not None
     assert job.get_phase() == 'PENDING'
@@ -509,42 +465,33 @@ def test_start_job():
 
 def test_abort_job():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
     jobid = '12345'
     # Phase POST response
-    responsePhase = DummyResponse()
-    responsePhase.set_status_code(200)
-    responsePhase.set_message("OK")
-    responsePhase.set_data(method='POST',
-                           context=None,
-                           body=None,
-                           headers=None)
+    responsePhase = DummyResponse(200)
+    responsePhase.set_data(method='POST')
     req = f"async/{jobid}/phase?PHASE=ABORT"
     connHandler.set_response(req, responsePhase)
     # Launch response
-    responseLaunchJob = DummyResponse()
-    responseLaunchJob.set_status_code(303)
-    responseLaunchJob.set_message("OK")
+    responseLaunchJob = DummyResponse(303)
     # list of list (httplib implementation for headers in response)
     launchResponseHeaders = [
         ['location', f'http://test:1111/tap/async/{jobid}']
     ]
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=None,
-                               headers=launchResponseHeaders)
+    responseLaunchJob.set_data(method='POST', headers=launchResponseHeaders)
     query = 'query'
     dictTmp = {
         "REQUEST": "doQuery",
         "LANG": "ADQL",
         "FORMAT": "votable",
-        "tapclient": str(TAP_CLIENT_ID),
+        "MAXREC": 10,
+        "tapclient": str(tap.tap_client_id),
         "QUERY": str(query)}
     sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
     req = f"async?{sortedKey}"
     connHandler.set_response(req, responseLaunchJob)
 
-    job = tap.launch_job_async(query, autorun=False)
+    job = tap.launch_job_async(query, autorun=False, maxrec=10)
     assert job is not None
     assert job.get_phase() == 'PENDING'
     # abort job
@@ -557,109 +504,74 @@ def test_abort_job():
 
 def test_job_parameters():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
     jobid = '12345'
     # Launch response
-    responseLaunchJob = DummyResponse()
-    responseLaunchJob.set_status_code(303)
-    responseLaunchJob.set_message("OK")
+    responseLaunchJob = DummyResponse(303)
     # list of list (httplib implementation for headers in response)
     launchResponseHeaders = [
         ['location', f'http://test:1111/tap/async/{jobid}']
     ]
-    responseLaunchJob.set_data(method='POST',
-                               context=None,
-                               body=None,
-                               headers=launchResponseHeaders)
+    responseLaunchJob.set_data(method='POST', headers=launchResponseHeaders)
     query = 'query'
     dictTmp = {
         "REQUEST": "doQuery",
         "LANG": "ADQL",
         "FORMAT": "votable",
-        "tapclient": str(TAP_CLIENT_ID),
+        "MAXREC": 10,
+        "tapclient": str(tap.tap_client_id),
         "QUERY": str(query)}
     sortedKey = taputils.taputil_create_sorted_dict_key(dictTmp)
     req = f"async?{sortedKey}"
     connHandler.set_response(req, responseLaunchJob)
     # Phase response
-    responsePhase = DummyResponse()
-    responsePhase.set_status_code(200)
-    responsePhase.set_message("OK")
-    responsePhase.set_data(method='GET',
-                           context=None,
-                           body="COMPLETED",
-                           headers=None)
+    responsePhase = DummyResponse(200)
+    responsePhase.set_data(method='GET', body="COMPLETED")
     req = f"async/{jobid}/phase"
     connHandler.set_response(req, responsePhase)
     # Results response
-    responseResultsJob = DummyResponse()
-    responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseResultsJob.set_data(method='GET',
-                                context=None,
-                                body=jobData,
-                                headers=None)
+    responseResultsJob = DummyResponse(200)
+    responseResultsJob.set_data(method='GET', body=TEST_DATA["job_1.vot"])
     req = f"async/{jobid}/results/result"
     connHandler.set_response(req, responseResultsJob)
 
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
-    job = tap.launch_job_async(query, autorun=False)
+    job = tap.launch_job_async(query, maxrec=10, autorun=False)
     assert job is not None
     assert job.get_phase() == 'PENDING'
 
     # parameter response
-    responseParameters = DummyResponse()
-    responseParameters.set_status_code(200)
-    responseParameters.set_message("OK")
-    responseParameters.set_data(method='GET',
-                                context=None,
-                                body=None,
-                                headers=None)
+    responseParameters = DummyResponse(200)
+    responseParameters.set_data(method='GET')
     req = f"async/{jobid}?param1=value1"
     connHandler.set_response(req, responseParameters)
     # Phase POST response
-    responsePhase = DummyResponse()
-    responsePhase.set_status_code(200)
-    responsePhase.set_message("OK")
-    responsePhase.set_data(method='POST',
-                           context=None,
-                           body=None,
-                           headers=None)
+    responsePhase = DummyResponse(200)
+    responsePhase.set_data(method='POST')
     req = f"async/{jobid}/phase?PHASE=RUN"
     connHandler.set_response(req, responsePhase)
 
     # send parameter OK
-    job.send_parameter("param1", "value1")
+    job.send_parameter(name="param1", value="value1")
     # start job
     job.start()
     assert job.get_phase() == 'QUEUED'
     # try to send a parameter after execution
     with pytest.raises(Exception):
-        job.send_parameter("param2", "value2")
+        job.send_parameter(name="param2", value="value2")
 
 
 def test_list_async_jobs():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    response = DummyResponse()
-    response.set_status_code(500)
-    response.set_message("ERROR")
-    jobDataFile = data_path('jobs_list.xml')
-    jobData = utils.read_file_content(jobDataFile)
-    response.set_data(method='GET',
-                      context=None,
-                      body=jobData,
-                      headers=None)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    response = DummyResponse(500)
+    response.set_data(method='GET', body=TEST_DATA["jobs_list.xml"])
     req = "async"
     connHandler.set_response(req, response)
     with pytest.raises(Exception):
         tap.list_async_jobs()
 
     response.set_status_code(200)
-    response.set_message("OK")
     jobs = tap.list_async_jobs()
     assert len(jobs) == 2
     assert jobs[0].jobid == '12345'
@@ -670,18 +582,11 @@ def test_list_async_jobs():
 
 def test_data():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap",
+    tap = TapPlus(url="http://test:1111/tap",
                   data_context="data",
                   connhandler=connHandler)
-    responseResultsJob = DummyResponse()
-    responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseResultsJob.set_data(method='GET',
-                                context=None,
-                                body=jobData,
-                                headers=None)
+    responseResultsJob = DummyResponse(200)
+    responseResultsJob.set_data(method='GET', body=TEST_DATA["job_1.vot"])
     req = "?ID=1%2C2&format=votable"
     connHandler.set_response(req, responseResultsJob)
     req = "?ID=1%2C2"
@@ -689,55 +594,43 @@ def test_data():
 
     # error
     responseResultsJob.set_status_code(500)
-    responseResultsJob.set_message("ERROR")
-    params_dict = {}
-    params_dict['ID'] = "1,2"
+    params_dict = {'ID': "1,2"}
     with pytest.raises(Exception):
-        tap.load_data(params_dict)
+        tap.load_data(params_dict=params_dict)
 
     # OK
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
 
     # results
-    results = tap.load_data(params_dict)
+    results = tap.load_data(params_dict=params_dict)
     assert len(results) == 3
     # error: no params dictionary
     with pytest.raises(Exception):
         # no dictionary: exception
-        tap.load_data("1,2")
+        tap.load_data(params_dict="1,2")
     params_dict['format'] = "votable"
-    results = tap.load_data(params_dict)
+    results = tap.load_data(params_dict=params_dict)
     assert len(results) == 3
 
 
 def test_datalink():
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap",
+    tap = TapPlus(url="http://test:1111/tap",
                   datalink_context="datalink",
                   connhandler=connHandler)
-    responseResultsJob = DummyResponse()
-    responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
-    jobDataFile = data_path('job_1.vot')
-    jobData = utils.read_file_content(jobDataFile)
-    responseResultsJob.set_data(method='GET',
-                                context=None,
-                                body=jobData,
-                                headers=None)
+    responseResultsJob = DummyResponse(200)
+    responseResultsJob.set_data(method='GET', body=TEST_DATA["job_1.vot"])
     req = "links?ID=1,2"
     connHandler.set_response(req, responseResultsJob)
 
     # error
     responseResultsJob.set_status_code(500)
-    responseResultsJob.set_message("ERROR")
     with pytest.raises(Exception):
         # missing IDS parameter
         tap.get_datalinks(ids=None)
 
     # OK
     responseResultsJob.set_status_code(200)
-    responseResultsJob.set_message("OK")
     # results
     results = tap.get_datalinks("1,2")
     assert len(results) == 3
@@ -887,16 +780,9 @@ def test_get_current_column_values_for_update():
 def test_update_user_table():
     tableName = 'table'
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    dummyResponse = DummyResponse()
-    dummyResponse.set_status_code(200)
-    dummyResponse.set_message("OK")
-    tableDataFile = data_path('test_table_update.xml')
-    tableData = utils.read_file_content(tableDataFile)
-    dummyResponse.set_data(method='GET',
-                           context=None,
-                           body=tableData,
-                           headers=None)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    dummyResponse = DummyResponse(200)
+    dummyResponse.set_data(method='GET', body=TEST_DATA["test_table_update.xml"])
     tableRequest = f"tables?tables={tableName}"
     connHandler.set_response(tableRequest, dummyResponse)
 
@@ -920,9 +806,7 @@ def test_update_user_table():
         tap.update_user_table(table_name=tableName, list_of_changes=list_of_changes)
 
     # OK
-    responseEditTable = DummyResponse()
-    responseEditTable.set_status_code(200)
-    responseEditTable.set_message("OK")
+    responseEditTable = DummyResponse(200)
     dictTmp = {
         "ACTION": "edit",
         "NUMTABLES": "1",
@@ -962,16 +846,9 @@ def test_rename_table():
     newTableName = 'user_test.table_test_rename_new'
     newColumnNames = {'ra': 'alpha', 'dec': 'delta'}
     connHandler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=connHandler)
-    dummyResponse = DummyResponse()
-    dummyResponse.set_status_code(200)
-    dummyResponse.set_message("OK")
-    tableDataFile = data_path('test_table_rename.xml')
-    tableData = utils.read_file_content(tableDataFile)
-    dummyResponse.set_data(method='GET',
-                           context=None,
-                           body=tableData,
-                           headers=None)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=connHandler)
+    dummyResponse = DummyResponse(200)
+    dummyResponse.set_data(method='GET', body=TEST_DATA["test_table_rename.xml"])
 
     with pytest.raises(Exception):
         tap.rename_table()
@@ -981,18 +858,14 @@ def test_rename_table():
         tap.rename_table(table_name=tableName, new_table_name=None, new_column_names_dict=None)
 
     # Test OK.
-    responseRenameTable = DummyResponse()
-    responseRenameTable.set_status_code(200)
-    responseRenameTable.set_message("OK")
+    responseRenameTable = DummyResponse(200)
     dictArgs = {
         "action": "rename",
         "new_column_names": "ra:alpha,dec:delta",
         "new_table_name": newTableName,
         "table_name": tableName,
     }
-    data = connHandler.url_encode(dictArgs)
-    req = f"TableTool?{data}"
-    connHandler.set_response(req, responseRenameTable)
+    connHandler.set_response(f"TableTool?{urlencode(dictArgs)}", responseRenameTable)
     tap.rename_table(table_name=tableName, new_table_name=newTableName, new_column_names_dict=newColumnNames)
 
 
@@ -1031,7 +904,7 @@ def __check_results_column(results, columnName, description, unit,
 @patch.object(TapPlus, 'login')
 def test_login(mock_login):
     conn_handler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=conn_handler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=conn_handler)
     tap.login("user", "password")
     assert (mock_login.call_count == 1)
     mock_login.side_effect = HTTPError("Login error")
@@ -1044,7 +917,7 @@ def test_login(mock_login):
 @patch.object(TapPlus, 'login')
 def test_login_gui(mock_login_gui, mock_login):
     conn_handler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=conn_handler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=conn_handler)
     tap.login_gui()
     assert (mock_login_gui.call_count == 0)
     mock_login_gui.side_effect = HTTPError("Login error")
@@ -1056,7 +929,7 @@ def test_login_gui(mock_login_gui, mock_login):
 @patch.object(TapPlus, 'logout')
 def test_logout(mock_logout):
     conn_handler = DummyConnHandler()
-    tap = TapPlus("http://test:1111/tap", connhandler=conn_handler)
+    tap = TapPlus(url="http://test:1111/tap", connhandler=conn_handler)
     tap.logout()
     assert (mock_logout.call_count == 1)
     mock_logout.side_effect = HTTPError("Login error")
