@@ -1,17 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import pytest
+import numpy as np
 
 import astropy.units as u
-from astropy import coordinates
-from ... import vizier
-from ...utils import commons
+from astropy.coordinates import SkyCoord
+
+from astroquery import vizier
+from astroquery.utils import commons
+from .conftest import scalar_skycoord, vector_skycoord
 
 
 @pytest.mark.remote_data
-class TestVizierRemote(object):
-
-    target = commons.ICRSCoordGenerator(ra=299.590, dec=35.201,
-                                        unit=(u.deg, u.deg))
+class TestVizierRemote:
 
     def test_query_object(self):
         result = vizier.core.Vizier.query_object(
@@ -31,35 +31,34 @@ class TestVizierRemote(object):
 
     def test_query_region(self):
         result = vizier.core.Vizier.query_region(
-            self.target, radius=5 * u.deg, catalog=["HIP", "NOMAD", "UCAC"])
+            scalar_skycoord, radius=5 * u.deg, catalog=["HIP", "NOMAD", "UCAC"])
 
         assert isinstance(result, commons.TableList)
 
     def test_query_region_async(self):
         response = vizier.core.Vizier.query_region_async(
-            self.target, radius=5 * u.deg, catalog=["HIP", "NOMAD", "UCAC"])
+            scalar_skycoord, radius=5 * u.deg, catalog=["HIP", "NOMAD", "UCAC"])
         assert response is not None
 
     def test_query_region_async_galactic(self):
         response = vizier.core.Vizier.query_region_async(
-            self.target, radius=0.5 * u.deg, catalog="HIP",
-            frame="galactic")
+            scalar_skycoord, radius=0.5 * u.deg, catalog="HIP", frame="galactic")
         assert response is not None
         payload = vizier.core.Vizier.query_region_async(
-            self.target, radius=0.5 * u.deg, catalog="HIP",
+            scalar_skycoord, radius=0.5 * u.deg, catalog="HIP",
             frame="galactic", get_query_payload=True)
         assert "-c=G" in payload
 
     def test_query_Vizier_instance(self):
-        v = vizier.core.Vizier(
-            columns=['_RAJ2000', 'DEJ2000', 'B-V', 'Vmag', 'Plx'],
-            column_filters={"Vmag": ">10"}, keywords=["optical", "xry"])
+        with pytest.warns(UserWarning, match="xry : No such keyword"):
+            v = vizier.core.Vizier(
+                columns=['_RAJ2000', 'DEJ2000', 'B-V', 'Vmag', 'Plx'],
+                column_filters={"Vmag": ">10"}, keywords=["optical", "xry"])
 
         result = v.query_object("HD 226868", catalog=["NOMAD", "UCAC"])
         assert isinstance(result, commons.TableList)
-        result = v.query_region(self.target,
-                                width="5d0m0s", height="3d0m0s",
-                                catalog=["NOMAD", "UCAC"])
+        result = v.query_region(
+            scalar_skycoord, width="5d0m0s", height="3d0m0s", catalog=["NOMAD", "UCAC"])
         assert isinstance(result, commons.TableList)
 
     def test_vizier_column_restriction(self):
@@ -67,7 +66,7 @@ class TestVizierRemote(object):
         # catalogs include Bmag's
         v = vizier.core.Vizier(
             columns=['_RAJ2000', 'DEJ2000', 'B-V', 'Vmag', 'Plx'],
-            column_filters={"Vmag": ">10"}, keywords=["optical", "xry"])
+            column_filters={"Vmag": ">10"}, keywords=["optical", "X-ray"])
 
         result = v.query_object("HD 226868", catalog=["NOMAD", "UCAC"])
         for table in result:
@@ -87,6 +86,10 @@ class TestVizierRemote(object):
         result = vizier.core.Vizier.get_catalogs('J/ApJ/706/83')
         assert isinstance(result, commons.TableList)
 
+    def test_get_catalog_metadata(self):
+        meta = vizier.core.Vizier(catalog="I/324").get_catalog_metadata()
+        assert meta['title'] == "The Initial Gaia Source List (IGSL)"
+
     def test_query_two_wavelengths(self):
         v = vizier.core.Vizier(
             columns=['_RAJ2000', 'DEJ2000', 'B-V', 'Vmag', 'Plx'],
@@ -98,22 +101,24 @@ class TestVizierRemote(object):
         V = vizier.core.Vizier(
             columns=['all'], ucd='(spect.dopplerVeloc*|phys.veloc*)',
             keywords=['Radio', 'IR'], row_limit=5000)
-        C = coordinates.SkyCoord(359.61687 * u.deg, -0.242457 * u.deg,
-                                 frame='galactic')
+        C = SkyCoord(359.61687 * u.deg, -0.242457 * u.deg, frame="galactic")
 
-        r2 = V.query_region(C, radius=2 * u.arcmin)
+        # With newer versions UnitsWarning may be issued as well
+        with pytest.warns() as w:
+            V.query_region(C, radius=2 * u.arcmin)
+
+        for i in w:
+            message = str(i.message)
+            assert ("VOTABLE parsing raised exception" in message or "not supported by the VOUnit standard" in message)
 
     def test_multicoord(self):
 
-        # Coordinate selection is entirely arbitrary
-        targets = commons.ICRSCoordGenerator(ra=[299.590, 299.90],
-                                             dec=[35.201, 35.201],
-                                             unit=(u.deg, u.deg))
         # Regression test: the columns of the default should never
         # be modified from default
         assert vizier.core.Vizier.columns == ['*']
+        # Coordinate selection is entirely arbitrary
         result = vizier.core.Vizier.query_region(
-            targets, radius=10 * u.arcsec, catalog=["HIP", "NOMAD", "UCAC"])
+            vector_skycoord, radius=10 * u.arcsec, catalog=["HIP", "NOMAD", "UCAC"])
 
         assert len(result) >= 5
         assert 'I/239/hip_main' in result.keys()
@@ -140,3 +145,18 @@ class TestVizierRemote(object):
         assert len(result) >= 628
         # important part: we're testing that UCD is parsed and some catalogs are ruled out
         assert len(ucdresult) < len(result)
+
+    def test_asu_tsv_return_type(self):
+        V = vizier.core.Vizier()
+        result = V.query_object("HD 226868", catalog=["NOMAD", "UCAC"], return_type='asu-tsv', cache=False)
+
+        assert isinstance(result, list)
+        assert len(result) == 3
+
+    def test_query_constraints(self):
+        V = vizier.core.Vizier(row_limit=3)
+        result = V.query_constraints(catalog="I/130/main", mB2="=14.7")[0]
+        # row_limit is taken in account
+        assert len(result) == 3
+        # the criteria is respected
+        assert np.all(np.isclose(result["mB2"], 14.7, rtol=1e-09, atol=1e-09))
